#ifndef MATRIX_H_
#define MATRIX_H_
//#define GameGrid Matrix<MatrixContentObject>

//#include <oslib/oslib.h>
#include <stdlib.h>
#include "code/framework/utils.h"
using namespace utils;

template<class T>
class Matrix
{
	private:
		T** m_table; //tableau de pointeurs vers T
	
	protected:
		int m_cols; //nombre de colonnes
		int m_lines; //nombre de lignes
		
	public:
		Matrix(int cols, int lines)
		{
			m_table = new T*[cols*lines];
			for(int i=0;i<cols*lines;i++)
			{
				m_table[i]=NULL;
			}
			m_cols=cols;
			m_lines=lines;
		}
		
		virtual ~Matrix()
		{
			for(int i=0;i<m_cols*m_lines;i++)
			{
				//delete des contenus
				if(m_table[i]!=NULL)
					delete m_table[i];
				
			}
			//delete de la table
			delete[] m_table;
		}
		
		
		//getter et setter
		inline T* setAt(int col, int line, T* value)
		{

			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::setAt outOfBounds");
			/*if(m_table[col][line]!=NULL)
				delete m_table[col][line];*/
			m_table[col+line*m_cols]=value;
			return value;
		}
		
		//pareil que setAt, mais delete la case avant le set
		inline T* replaceAt(int col, int line, T* value)
		{
			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::replaceAt outOfBounds");
			int index=col+line*m_cols;
			if(m_table[index]!=NULL)
				delete m_table[index];
			m_table[index]=value;
			return value;
		}
		
		inline T* setAt(int col, int line, T value)
		{

			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::setAt outOfBounds");
			/*if(m_table[col][line]!=NULL)
				delete m_table[col][line];*/
			int index=col+line*m_cols;
			m_table[index]=new T(value); //construction par copie
			return m_table[index];
		}
		
		inline T* replaceAt(int col, int line, T value)
		{
			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::replaceAt outOfBounds");
			int index=col+line*m_cols;
			if(m_table[index]!=NULL)
				delete m_table[index];
			m_table[index]=new T(value); //construction par copie
			return m_table[index];
		}
		
		inline T* getAt(int col, int line)
		{
			if(col>=m_cols || line>=m_lines)
				//return NULL;//TODO: mettre un fatalerror?
				fatalError("Matrix::getAt outOfBounds");
			return m_table[col+line*m_cols];
		}
		
		inline T getBasicAt(int col, int line)
		{
			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::getBasicAt outOfBounds");

			T *t=m_table[col+line*m_cols];
			T t2=*t;

			return t2;
		}
		
		inline int getNbCols()
		{
			return m_cols;
		}
		
		inline int getNbLines()
		{
			return m_lines;
		}
		
		bool cellIsEmpty(int col, int line)
		{
			//si on demande out of bounds
			if(col>=m_cols || line>=m_lines || col<0 || line<0)
				return false;
				
			//à l'intérieur
			return m_table[col+line*m_cols]==NULL;
		}
		
		void deleteAt(int col, int line)
		{
			if(col>=m_cols || line>=m_lines)
				fatalError("Matrix::deleteAt outOfBounds");
			int index=col+line*m_cols;
			if(m_table[index]!=NULL)
			{
				delete m_table[index];
				m_table[index]=NULL;
			}
		}
		
		bool isOutOfBounds(int col, int line)
		{
			if(col>=m_cols || line>=m_lines)
				return true;
			if(col<0 || line<0)
				return true;
			return false;
		}
};

#endif /*MATRIX_H_*/















