#include "code/framework/fx/impl/global/ProgressiveAlphaEffect.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
using namespace utils;


ProgressiveAlphaEffect::ProgressiveAlphaEffect(int timeAB, int timeBA, int from, int to, int nbLoops)
{
	m_running=false;
	m_curTick=0;
	m_nbLoops=nbLoops;
	m_currentLoop=0;
	m_aToB=true;
	m_timeAB=timeAB;
	m_timeBA=timeBA;
	m_from=from;
	m_to=to;
	
	m_alpha = new OSL_ALPHA_PARAMS;
	
	m_curColor=new u8[4];
	m_newAlpha=0;
}

ProgressiveAlphaEffect::~ProgressiveAlphaEffect()
{
	delete m_alpha;
	delete[] m_curColor;
}
	
void ProgressiveAlphaEffect::start()
{
	m_running=true;
}

void ProgressiveAlphaEffect::restart()
{
	m_curTick=0;
	m_running=true;
	m_currentLoop=0;
	m_aToB=true;
}

void ProgressiveAlphaEffect::reset()
{
	m_curTick=0;
	m_currentLoop=0;
	m_aToB=true;
}

void ProgressiveAlphaEffect::stop()
{
	m_running=false;
}

void ProgressiveAlphaEffect::update()
{
	if(m_running)
	{
		m_curTick++;
		if	(
				(m_aToB && m_curTick==m_timeAB) ||
				(!m_aToB && m_curTick==m_timeBA)
			)
		{
			m_currentLoop++;
			m_curTick=0;
			m_aToB=!m_aToB;
			if(finished())
			{
				m_running=false;
				return;	
			}
			//else
			
		}
	}
}

bool ProgressiveAlphaEffect::finished()
{
	if(m_nbLoops==-1)//infini
		return false;
		
	if(m_currentLoop==m_nbLoops)
		return true;
	return false;
}

bool ProgressiveAlphaEffect::isRunning()
{
	return m_running;
}

void ProgressiveAlphaEffect::activate()
{
	int res=0;
	if(m_from!=m_to)
	{
		int compFrom=m_aToB?m_from:m_to; //255
		int compTo=m_aToB?m_to:m_from; //0
	
		
		int time=m_aToB?m_timeAB:m_timeBA; //36
		if(time==0)
			res=compFrom;
		else
		{
			int curDist=int(((compTo-compFrom)*m_curTick)/time);
			res=int(curDist+compFrom);
		}
	}	
	else
	{
		res=m_from;
	}


	//oslSetAlpha(OSL_FX_TINT, RGBA(setres[0], setres[1], setres[2], setres[3]));


	oslGetAlphaEx(m_alpha);
	
	
	oslRgbaGet8888(m_alpha->coeff1, m_curColor[0], m_curColor[1], m_curColor[2], m_curColor[3]);
	
	m_alpha->effect=OSL_FX_ALPHA | OSL_FX_TINT;
//	for(unsigned short i=0;i<3;i++)
//	{	
//		newColor[i]=curColor[i];//*setres[i])/256.0f);
//	}
	m_newAlpha=u8((m_curColor[3]*res)/255.0f);
	m_alpha->coeff1=RGBA(m_curColor[0], m_curColor[1], m_curColor[2], m_newAlpha);
	
	oslSetAlphaEx(m_alpha);
	
}

ProgressiveAlphaEffect* ProgressiveAlphaEffect::clone() const
{
	return new ProgressiveAlphaEffect(m_timeAB, m_timeBA, m_from, m_to, m_nbLoops);
}
		
void ProgressiveAlphaEffect::setTimeForward(int time)
{
	m_timeAB=time;
}

int ProgressiveAlphaEffect::getTimeForward()
{ 
	return m_timeAB;
} 

void ProgressiveAlphaEffect::setTimeBackward(int time)
{
	m_timeBA=time;
}

int ProgressiveAlphaEffect::getTimeBackward()
{
	return m_timeBA;
}

void ProgressiveAlphaEffect::setNbLoops(int nb)
{
	m_nbLoops=nb;	
}		
		
int ProgressiveAlphaEffect::getCurrentAlpha()
{
	int res=0;
	if(m_from!=m_to)
	{
		int compFrom=m_aToB?m_from:m_to; //255
		int compTo=m_aToB?m_to:m_from; //0
	
		
		int time=m_aToB?m_timeAB:m_timeBA; //36
		if(time==0)
			res=compFrom;
		else
		{
			int curDist=int(((compTo-compFrom)*m_curTick)/time);
			res=int(curDist+compFrom);
		}
	}	
	else
	{
		res=m_from;
	}
	
	return res;
}	
		
		
		
		






