#include "code/framework/fx/impl/global/ProgressiveTintEffect.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
using namespace utils;


ProgressiveTintEffect::ProgressiveTintEffect(int timeAB, int timeBA, u32 colorFrom, u32 colorTo, int nbLoops)
{
	m_running=false;
	m_curTick=0;
	m_nbLoops=nbLoops;
	m_currentLoop=0;
	m_aToB=true;
	m_timeAB=timeAB;
	m_timeBA=timeBA;
	m_colorFrom=new u8[4];
	m_colorTo=new u8[4];
	oslRgbaGet8888(colorFrom, m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]);
	oslRgbaGet8888(colorTo, m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]);
	
	m_diffFrames=new float[4]; //tableau de difference entre frames
	m_current=new float[4]; //tableau avec la couleur actuelle
	
	for(int i=0;i<4;i++)
	{
		m_current[i]=m_colorFrom[i];
		m_diffFrames[i]=(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
	}
	
	if(timeAB==0)
		fatalError("ProgressiveTintEffect: time cannot be = 0");
		
	m_alpha = new OSL_ALPHA_PARAMS;
	
	m_curColor=new u8[4];
	m_newColor=new u8[4];
}
	
void ProgressiveTintEffect::start()
{
	m_running=true;
}

void ProgressiveTintEffect::restart()
{
	reset();
	start();
}

void ProgressiveTintEffect::reset()
{
	m_curTick=0;
	m_currentLoop=0;
	m_aToB=true;
	for(int i=0;i<4;i++)
	{
		m_current[i]=m_colorFrom[i];
		m_diffFrames[i]=(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
	}
}

void ProgressiveTintEffect::stop()
{
	m_running=false;
}

u32 ProgressiveTintEffect::getCurrentColor()
{
	return RGBA(int(m_current[0]), int(m_current[1]), int(m_current[2]), int(m_current[3]));
}

void ProgressiveTintEffect::update()
{
	if(m_running)
	{
		//bool past=false; //va nous dire si on est arrivé au bout d'une boucle
		m_curTick++;
		for(int i=0;i<4;i++)
		{
			m_current[i]+=m_diffFrames[i];
			if(m_current[i]>255)
				m_current[i]=255;
			if(m_current[i]<0)
				m_current[i]=0;
				
			//est ce qu'on est au bout?
//			if(m_diffFrames[i]>0)
//			{
//				if(m_aToB)
//				{
//					if(m_current[i]>=m_colorTo[i])
//						past=true;
//				}
//				else
//				{
//					if(m_current[i]>=m_colorFrom[i])
//						past=true;
//				}
//			}
//			else if(m_diffFrames[i]<0)
//			{
//				if(m_aToB)
//				{
//					if(m_current[i]<=m_colorTo[i])
//						past=true;
//				}
//				else
//				{
//					if(m_current[i]<=m_colorFrom[i])
//						past=true;
//				}
//			}
		}
		if	(
				(m_aToB && m_curTick>=m_timeAB) ||
				(!m_aToB && m_curTick>=m_timeBA)
			)
		//if(past)
		{
			m_curTick=0;
			m_aToB=!m_aToB;
			
			m_currentLoop++;
			if(finished())
			{
				m_running=false;
				return;	
			}
			
			if(m_aToB)
			{
				for(int i=0;i<4;i++)
				{
					m_current[i]=m_colorFrom[i];
					m_diffFrames[i]=float(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
				}
			}
			else
			{
				for(int i=0;i<4;i++)
				{
					m_current[i]=m_colorTo[i];
					m_diffFrames[i]=float(m_colorFrom[i]-m_colorTo[i])/float(m_timeBA);
				}
			}	
		}
	}
}

bool ProgressiveTintEffect::finished()
{
	if(m_nbLoops==-1)//infini
		return false;
		
	if(m_currentLoop==m_nbLoops)
		return true;
	return false;
}

bool ProgressiveTintEffect::isRunning()
{
	return m_running;
}

void ProgressiveTintEffect::activate()
{
	oslGetAlphaEx(m_alpha);
	
	oslRgbaGet8888(m_alpha->coeff1, m_curColor[0], m_curColor[1], m_curColor[2], m_curColor[3]);
	
	m_alpha->effect=OSL_FX_ALPHA | OSL_FX_TINT;
	for(int i=0;i<4;i++)
	{	
		m_newColor[i]=u8(float(m_curColor[i]*m_current[i])/255.0f);
	}
	
	m_alpha->coeff1=RGBA(m_newColor[0], m_newColor[1], m_newColor[2], m_newColor[3]);
	
	oslSetAlphaEx(m_alpha);
}


ProgressiveTintEffect::~ProgressiveTintEffect()
{
	delete[] m_colorFrom;
	delete[] m_colorTo;
	delete[] m_diffFrames;
	delete[] m_current;
	
	delete m_alpha;
	delete[] m_curColor;
	delete[] m_newColor;
}

ProgressiveTintEffect* ProgressiveTintEffect::clone() const
{
	return new ProgressiveTintEffect(m_timeAB, m_timeBA, RGBA(m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]), RGBA(m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]), m_nbLoops);
}
		
void ProgressiveTintEffect::setTimeForward(int time)
{
	m_curTick=int((time*m_curTick)/m_timeAB);
	m_timeAB=time;
	//proportion entre curtick, ancien timeAB, timeAB
	
}

int ProgressiveTintEffect::getTimeForward()
{ 
	return m_timeAB;
} 

void ProgressiveTintEffect::setTimeBackward(int time)
{
	m_curTick=int((time*m_curTick)/m_timeBA);
	m_timeBA=time;
	//proportion entre curtick, ancien timeBA, timeBA
}

int ProgressiveTintEffect::getTimeBackward()
{
	return m_timeBA;
}	
		
void ProgressiveTintEffect::setColorTo(u32 color)
{
	oslRgbaGet8888(color, m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]);
	for(int i=0;i<4;i++)
	{
		if(m_aToB)
		{	
			//m_current[i]=m_colorFrom[i];
			m_diffFrames[i]=(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
			m_current[i]=m_colorFrom[i] + m_diffFrames[i] * m_curTick;
		}
		else
		{
			//m_current[i]=m_colorTo[i];
			m_diffFrames[i]=(m_colorFrom[i]-m_colorTo[i])/float(m_timeBA);
			m_current[i]=m_colorTo[i] + m_diffFrames[i] * m_curTick;
		}
	}
}

void ProgressiveTintEffect::setColorFrom(u32 color)
{
	oslRgbaGet8888(color, m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]);	
	for(int i=0;i<4;i++)
	{
		if(m_aToB)
		{	
			//m_current[i]=m_colorFrom[i];
			m_diffFrames[i]=(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
			m_current[i]=m_colorFrom[i] + m_diffFrames[i] * m_curTick;
		}
		else
		{
			//m_current[i]=m_colorTo[i];
			m_diffFrames[i]=(m_colorFrom[i]-m_colorTo[i])/float(m_timeBA);
			m_current[i]=m_colorTo[i] + m_diffFrames[i] * m_curTick;
		}
	}
}
		
void ProgressiveTintEffect::setColors(u32 from, u32 to)
{
	oslRgbaGet8888(to, m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]);
	oslRgbaGet8888(from, m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]);
	for(int i=0;i<4;i++)
	{
		if(m_aToB)
		{	
			//m_current[i]=m_colorFrom[i];
			m_diffFrames[i]=(m_colorTo[i]-m_colorFrom[i])/float(m_timeAB);
			m_current[i]=m_colorFrom[i] + m_diffFrames[i] * m_curTick;
		}
		else
		{
			//m_current[i]=m_colorTo[i];
			m_diffFrames[i]=(m_colorFrom[i]-m_colorTo[i])/float(m_timeBA);
			m_current[i]=m_colorTo[i] + m_diffFrames[i] * m_curTick;
		}
	}
}	
		
void ProgressiveTintEffect::setNbLoops(int nb)
{
	m_nbLoops=nb;	
}	






