#include "code/framework/fx/impl/local/GravityEffect.h"

GravityEffect::GravityEffect(float originalVelocityX, float originalVelocityY, float accelerationX, float accelerationY, int stopAtNbTicks)
{
	m_running=false;
	m_originalVelocityX=originalVelocityX;
	m_originalVelocityY=originalVelocityY;
	m_velocityX=originalVelocityX; //current
	m_velocityY=originalVelocityY; //current
	m_accelerationX=accelerationX;
	m_accelerationY=accelerationY;
	
	m_nbTicks=0;
	m_posX=0;
	m_posY=0;
	
	m_stopAtNbTicks=stopAtNbTicks;
}

void GravityEffect::start()
{
	m_running=true;
}
 
void GravityEffect::restart()
{
	reset();
	start();
}

void GravityEffect::reset()
{
	m_velocityX=m_originalVelocityX; //current
	m_velocityY=m_originalVelocityY; //current
		
	m_nbTicks=0;
	m_posX=0;
	m_posY=0;
}

void GravityEffect::stop()
{
	m_running=false;
}

void GravityEffect::update()
{
	if(m_running)
	{
		m_posX+=m_velocityX;
		m_posY+=m_velocityY;
		
		m_velocityX+=m_accelerationX;
		m_velocityY+=m_accelerationY;
		
		if(finished())
			m_running=false;
	}
}

bool GravityEffect::finished()
{
	if(m_stopAtNbTicks==-1)
		return false;
	
	if(m_nbTicks==m_stopAtNbTicks)
		return true;
	
	return false;
}

bool GravityEffect::isRunning()
{
	return m_running;
}

OSL_IMAGE * GravityEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
		
	img->x+=int(m_posX);
	img->y+=int(m_posY);
	
	return img;
} 

Positionable *GravityEffect::applyOnPositionable(Positionable *obj)
{
	if(obj==NULL)
		return NULL;
	
	obj->setX(obj->getX()+int(m_posX));
	obj->setY(obj->getY()+int(m_posY));
	
	return obj;
}

GravityEffect* GravityEffect::clone() const
{
	return new GravityEffect(m_originalVelocityX, m_originalVelocityY, m_accelerationX, m_accelerationY, m_stopAtNbTicks);
}

void GravityEffect::setAccelerationX(float ax)
{
	m_accelerationX=ax;
}

void GravityEffect::setAccelerationY(float ay)
{
	m_accelerationY=ay;
}

float GravityEffect::getAccelerationX()
{
	return m_accelerationX;
}

float GravityEffect::getAccelerationY()
{
	return m_accelerationY;
}

int GravityEffect::getPosX()
{
	return int(m_posX);
}

int GravityEffect::getPosY()
{
	return int(m_posY);
}





