#include "code/framework/fx/impl/local/RemanentEffect.h"

RemanentEffect::RemanentEffect(unsigned int duration)
{
	m_running=false;
	m_duration=duration;
	m_current=0;
}


void RemanentEffect::start()
{
	m_running=true;
}

void RemanentEffect::restart()
{
	m_current=0;
	start();
}

void RemanentEffect::reset()
{
	m_current=0;
}

void RemanentEffect::stop()
{
	m_running=false;
}

void RemanentEffect::update()
{
	if(m_running)
	{
		if(m_current<m_duration)
			m_current++;
		else
			m_running=false;	
	}
}

bool RemanentEffect::finished()
{
	return m_current==m_duration;
}

bool RemanentEffect::isRunning()
{
	return m_running;	
}

OSL_IMAGE * RemanentEffect::applyOn(OSL_IMAGE * img)
{
	if(finished()) //une fois le temps écoulé, il ne faut plus afficher l'image
		return NULL;
	return img;
}

Positionable *RemanentEffect::applyOnPositionable(Positionable *obj)
{
	if(finished()) //une fois le temps écoulé, il ne faut plus afficher l'image
		return NULL;
	return obj;
}

Orientable *RemanentEffect::applyOnOrientable(Orientable *obj)
{
	if(finished()) //une fois le temps écoulé, il ne faut plus afficher l'image
		return NULL;
	return obj;
}

Resizable *RemanentEffect::applyOnResizable(Resizable *obj)
{
	if(finished()) //une fois le temps écoulé, il ne faut plus afficher l'image
		return NULL;
	return obj;
}

/*OSL_IMAGE * RemanentEffect::unapplyOn(OSL_IMAGE * img)
{
	//on n'a rien fait a l'image, on la renvoie telle quelle.
	return img;
}//*/

RemanentEffect* RemanentEffect::clone() const
{
	return new RemanentEffect(m_duration);
}











