#include "code/framework/fx/impl/local/ShakeEffect.h"
#include "code/framework/utils.h"
using namespace utils;

ShakeEffect::ShakeEffect(int width, int height, int time, bool direction, int nbLoops) //-1 = infinite
{

	m_curLoop=0;
	m_curTime=0;
	m_nbLoops=nbLoops;
	m_running=false;
	
	m_distXLeft=int(width/2);
	m_distXRight=width-m_distXLeft;
	m_distYUp=int(height/2);
	m_distYDown=height-m_distYUp;
	
	m_time=time;
	m_time_2=m_time/2.0f;
	
	m_direction=direction;  //=true; //droite
	m_position=direction; //=true; //droite
	m_startDirection=direction;
	
	m_pos_diff=Pos(0,0);
}
 
	
void ShakeEffect::start()
{
	m_running=true;
}
 
void ShakeEffect::restart()
{ 
	m_curLoop=0;
	m_curTime=0;
	m_running=true;
	m_direction=m_startDirection;
	m_position=m_startDirection;
}
 
void ShakeEffect::reset()
{
	m_curLoop=0;
	m_curTime=0;
	m_direction=m_startDirection;
	m_position=m_startDirection;
}
 
void ShakeEffect::stop()
{
	m_running=false;
}
 
void ShakeEffect::update()
{
	if(m_running)
	{
		m_curTime++;
		if(m_curTime==int(m_time_2))
		{
			m_direction=!m_direction;
		}
		if(m_curTime>=m_time)
		{
			m_position=!m_position;
			m_curLoop++;
			m_curTime=0;
		}
		if(finished())
		{
			m_running=false;
		}
		
		m_pos_diff=calculateDiff();
	}
}
 
bool ShakeEffect::finished()
{
	if(m_nbLoops==-1) //infini
	{
		return false; //ca fini jamais
	}
	
	if(m_nbLoops==m_curLoop)
	{
		return true;
	}
	return false;
}
 
bool ShakeEffect::isRunning()
{
	return m_running;
}
 
OSL_IMAGE * ShakeEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
	
	//Pos diff=calculateDiff();
	img->x+=m_pos_diff.x;
	img->y+=m_pos_diff.y;
	
	return img;
}
 
ShakeEffect* ShakeEffect::clone() const
{
	return new ShakeEffect(m_distXLeft+m_distXRight, m_distYUp+m_distYDown, m_time, m_startDirection, m_nbLoops);
}

Positionable *ShakeEffect::applyOnPositionable(Positionable *obj)
{
	if(obj==NULL)
		return NULL;
		
	//Pos diff=calculateDiff();
	obj->setX(obj->getX()+m_pos_diff.x);
	obj->setY(obj->getY()+m_pos_diff.y);
	
	return obj;
}

Pos ShakeEffect::calculateDiff()
{	
	//if(m_curTime==0)
	//	return Pos(0,0);
	
	float calculBuffer=m_curTime/m_time_2; //c'etait utilisé dans tous les calculs la en dessous
	
	
	if(m_position && m_direction) //on va vers la droite, on est a droite
	{
		//sur x
		int curDistX=int(calculBuffer*m_distXRight);
		
		//sur y
		int curDistY=int(calculBuffer*m_distYDown);
		
		return Pos(curDistX, curDistY);
	}
	if(!m_position && !m_direction) //on va vers la gauche, on est a gauche
	{
		//sur x
		int curDistX=int(calculBuffer*m_distXLeft);
		
		//sur y
		int curDistY=int(calculBuffer*m_distYUp);
		
		return Pos(-curDistX, -curDistY);
	}
	
	if(m_position && !m_direction) //on va vers la gauche, on est a droite
	{
		//sur x
		int curDistX=int(calculBuffer*m_distXRight);
		
		//sur y
		int curDistY=int(calculBuffer*m_distYDown);
		
		return Pos(m_distXRight-(curDistX-m_distXRight), m_distYDown-(curDistY-m_distYDown));
	}
	
	if(!m_position && m_direction) //on va vers la droite, on est a gauche
	{
		//sur x
		int curDistX=int(calculBuffer*m_distXLeft);
		
		//sur y
		int curDistY=int(calculBuffer*m_distYUp);
		
		return Pos(-m_distXLeft+curDistX-m_distXLeft, -m_distYUp+curDistY-m_distYUp);
	}
	
	//n'arrive jamais ici, mais satisfait le compilo
	return Pos();
}

void ShakeEffect::setTime(int time)
{
	m_time=time;
	m_time_2=time/2.0f;
}
int ShakeEffect::getTime()
{
	return m_time;	
}

//TODO: mettre le calcul (calculatediff) dans l'update
int ShakeEffect::getPosX()
{
	//Pos diff=calculateDiff();
	return m_pos_diff.x;
}

int ShakeEffect::getPosY()
{
	//Pos diff=calculateDiff();
	return m_pos_diff.y;
}













