#include "code/framework/fx/impl/local/multi/WaitLocalEffect.h"
#include "code/framework/check/PrettyFileLogger.h"

WaitLocalEffect::WaitLocalEffect(unsigned int time, bool showImage)
{
	m_running=false;
	m_time=time;
	m_currentTick=0;
	m_showImage=showImage;
	m_sizeXpercent=100.0f;
	m_sizeYpercent=100.0f;
	m_distX=0;
	m_distY=0;
	m_angle=0;
}

void WaitLocalEffect::setPosition(int x, int y)
{
	m_distX=x;
	m_distY=y;
}

void WaitLocalEffect::setSizePercent(float w, float h)
{
	m_sizeXpercent=w;
	m_sizeYpercent=h;
}

void WaitLocalEffect::setDirection(int angle)
{
	m_angle=angle;
}

	
void WaitLocalEffect::start()
{
	m_running=true;
} 
void WaitLocalEffect::restart()
{
	m_running=true;
	m_currentTick=0;
}
void WaitLocalEffect::reset()
{
	m_currentTick=0;
}
void WaitLocalEffect::stop()
{
	m_running=false;
}
void WaitLocalEffect::update()
{
	m_currentTick++;
	if(finished())
		m_running=false;
}
bool WaitLocalEffect::finished()
{
	if(m_currentTick>=m_time)
		return true;
	return false;
}
bool WaitLocalEffect::isRunning()
{
	return m_running;
}
OSL_IMAGE * WaitLocalEffect::applyOn(OSL_IMAGE *img)
{
	if(!m_showImage)
		return NULL;
	
	if(img==NULL)
		return NULL;
		
	if(m_sizeXpercent!=100) //100% on touche pas
	{
		img->stretchX=int((img->stretchX*m_sizeXpercent)/100.0f);
		//safety:
		if(img->stretchX<=0)
			return NULL;
	}
	if(m_sizeYpercent!=100) //100% on touche pas
	{
		img->stretchY=int((img->stretchY*m_sizeYpercent)/100.0f);
		//safety:
		if(img->stretchY<=0)
			return NULL;
	}	
	if(m_distX!=0)
	{
		img->x+=m_distX;
	}
	if(m_distY!=0)
	{
		img->y+=m_distY;
	}
	if(m_angle!=0)
	{
		img->angle+=m_angle;
	}
	return img;
}

Positionable *WaitLocalEffect::applyOnPositionable(Positionable *obj)
{
	if(!m_showImage)
		return NULL;
	
	if(obj==NULL)
		return NULL;
		
	if(m_distX!=0)
	{
		obj->setX(obj->getX()+m_distX);
	}
	if(m_distY!=0)
	{
		obj->setY(obj->getY()+m_distY);
	}
	
	return obj;
}

Orientable *WaitLocalEffect::applyOnOrientable(Orientable *obj)
{
	if(!m_showImage)
		return NULL;
	
	if(obj==NULL)
		return NULL;
		
	if(m_angle!=0)
	{
		obj->setOrientation(obj->getOrientation()+m_angle);
	}
	
	return obj;
}

Resizable *WaitLocalEffect::applyOnResizable(Resizable *obj)
{
	if(!m_showImage)
		return NULL;
	
	if(obj==NULL)
		return NULL;
		
	//TODO: ajouter partie resize
	return obj;
}

WaitLocalEffect* WaitLocalEffect::clone() const
{
	WaitLocalEffect *res=new WaitLocalEffect(m_time, m_showImage);
	res->setPosition(m_distX, m_distY);
	res->setSizePercent(m_sizeXpercent, m_sizeYpercent);
	res->setDirection(m_angle);
	return res;
}

unsigned int WaitLocalEffect::getTime()
{
	return m_time;
}

void WaitLocalEffect::setTime(unsigned int time)
{
	m_time=time;
}

int WaitLocalEffect::getPosX()
{
	return int(m_distX);
}

int WaitLocalEffect::getPosY()
{
	return int(m_distY);
}

