#include <oslib/oslib.h>
#include "code/framework/particles/complete/abstract/AbstractParticleGenerator.h"
#include "code/framework/utils.h"
#include "code/framework/fx/utils/AlphaParamsState.h"
#include "code/framework/particles/complete/fxFactories/PFXFExactSameLocalEffect.h"
#include "code/framework/particles/complete/fxFactories/PFXFExactSameGlobalEffect.h"
#include "code/framework/particles/complete/fxFactories/PFXFSimpleTranslation.h"
#include "code/framework/particles/complete/animFactories/ExactSameAnimationFactory.h"

using namespace utils;

AbstractParticleGenerator::AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
									int dist, int particleLifespan, AbstractAnimationFactory *animFactory, int nbLoops)
{
	m_running=false;
	
	m_x=x;
	m_y=y; 
	m_nbParticles=nbParticles;
	m_generationTime=generationTime;
	m_movingTime=movingTime;
	
	m_animFactory=animFactory;
	
	m_particles=new list<Particle *>();
	
	m_frameCounter=0;
	m_dist=dist;
	m_particleCounter=0;
	
	
	m_particlesStartFrame=new int[m_nbParticles];
	for(int i=0;i<m_nbParticles;i++)
	{
		m_particlesStartFrame[i]=int(ceil((m_generationTime*i)/m_nbParticles));
	}
	
	m_generationOn=true;
	m_nbLoops=nbLoops; //nombre de boucles, -1=infini
	m_currentLoop=1;
	
	//jusqu'a 5 effets locaux appliqués indépendamment à chaque particule
	m_lfx1=NULL;
	m_lfx2=NULL;
	m_lfx3=NULL;
	m_lfx4=NULL;
	m_lfx5=NULL;
	
	//jusqu'a 2 effets globaux appliqués indépendamment à chaque particule
	m_gfx1=NULL;		
	m_gfx2=NULL;
	
	m_particleLifespan=particleLifespan;
	
	m_orientation=direction%360;
	m_scale=scale;
	
	m_alphaparams=new AlphaParamsState();
}

AbstractParticleGenerator::AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles,
										int generationTime, int movingTime, int dist, int particleLifespan, 
										Anim *anim, int nbLoops)
{
	//initialize(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, anim, lfx1, lfx2, lfx3, lfx4, lfx5, gfx1, gfx2, nbLoops);
	m_running=false;
	
	m_x=x;
	m_y=y; 
	m_nbParticles=nbParticles;
	m_generationTime=generationTime;
	m_movingTime=movingTime;
	
	m_animFactory=new ExactSameAnimationFactory(anim);
	
	m_particles=new list<Particle *>();
	
	m_frameCounter=0;
	m_dist=dist;
	m_particleCounter=0;
	
	
	m_particlesStartFrame=new int[m_nbParticles];
	for(int i=0;i<m_nbParticles;i++)
	{
		m_particlesStartFrame[i]=int(ceil((m_generationTime*i)/m_nbParticles));
	}
	
	m_generationOn=true;
	m_nbLoops=nbLoops; //nombre de boucles, -1=infini
	m_currentLoop=1;
	
	//jusqu'a 5 effets locaux appliqués indépendamment à chaque particule
	m_lfx1=NULL;
	m_lfx2=NULL;
	m_lfx3=NULL;
	m_lfx4=NULL;
	m_lfx5=NULL;
	
	//jusqu'a 2 effets globaux appliqués indépendamment à chaque particule
	m_gfx1=NULL;		
	m_gfx2=NULL;
	
	m_particleLifespan=particleLifespan;
	
	m_orientation=direction%360;
	m_scale=scale;
	
	m_alphaparams=new AlphaParamsState();
}

AbstractParticleGenerator::AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles,
										int generationTime, int movingTime, int dist, int particleLifespan, 
										string filename, short transparency, int nbLoops)
{
	
	
	
	//initialize(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, anim, lfx1, lfx2, lfx3, lfx4, lfx5, gfx1, gfx2, nbLoops);
	m_running=false;
	
	m_x=x;
	m_y=y; 
	m_nbParticles=nbParticles;
	m_generationTime=generationTime;
	m_movingTime=movingTime;
	
	Anim *anim=new Anim(filename, transparency);
	anim->setReferencePointCenter();
	m_animFactory=new ExactSameAnimationFactory(anim);
	
	m_particles=new list<Particle *>();
	
	m_frameCounter=0;
	m_dist=dist;
	m_particleCounter=0;
	
	
	m_particlesStartFrame=new int[m_nbParticles];
	for(int i=0;i<m_nbParticles;i++)
	{
		m_particlesStartFrame[i]=int(ceil((m_generationTime*i)/m_nbParticles));
	}
	
	m_generationOn=true;
	m_nbLoops=nbLoops; //nombre de boucles, -1=infini
	m_currentLoop=1;
	
	//jusqu'a 5 effets locaux appliqués indépendamment à chaque particule
	m_lfx1=NULL;
	m_lfx2=NULL;
	m_lfx3=NULL;
	m_lfx4=NULL;
	m_lfx5=NULL;
	
	//jusqu'a 2 effets globaux appliqués indépendamment à chaque particule
	m_gfx1=NULL;		
	m_gfx2=NULL;
	
	m_particleLifespan=particleLifespan;
	
	m_orientation=direction%360;
	m_scale=scale;
	
	m_alphaparams=new AlphaParamsState();
}

//AbstractParticleGenerator *AbstractParticleGenerator::clone() const
//{
//	AbstractParticleGenerator *theclone=new AbstractParticleGenerator(	
//		m_x, m_y, m_orientation, m_scale, m_nbParticles, m_generationTime, m_movingTime, 
//		m_dist, m_particleLifespan, m_animFactory->clone(), m_nbLoops
//	);
//		
//	return theclone;
//}

void AbstractParticleGenerator::setThroughput(int nbParticles, int generationTime)
{
	delete[] m_particlesStartFrame;
	
	m_nbParticles=nbParticles;
	m_generationTime=generationTime;
	
	m_particlesStartFrame=new int[m_nbParticles];
	for(int i=0;i<m_nbParticles;i++)
	{
		//TODO: vérifier que ca marche
		m_particlesStartFrame[i]=int(
									float(m_generationTime*i)/float(m_nbParticles)
								)+1;
	}
	
	//attention de ne pas être à la fin du tableau m_particlesStartFrame
	if(m_particleCounter>=m_nbParticles)
	{
		m_particleCounter--;
	}
	if(m_frameCounter>=m_generationTime) //TODO: c'est utile ca??
	{
		m_frameCounter--;
	}
}

int AbstractParticleGenerator::getNbLoops() const
{
	return m_nbLoops;
}

AbstractAnimationFactory *AbstractParticleGenerator::getAnimationFactory() const
{
	return m_animFactory;
}

int AbstractParticleGenerator::getThroughputNbParticles() const
{
	return m_nbParticles;
}

int AbstractParticleGenerator::getThroughputFrameTime() const
{
	return m_generationTime;
}



/*void AbstractParticleGenerator::initialize(int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
						int dist, int particleLifespan, Anim *anim, AbstractLocalParticleEffectFactory *lfx1, 
						AbstractLocalParticleEffectFactory *lfx2, AbstractLocalParticleEffectFactory *lfx3, 
						AbstractLocalParticleEffectFactory *lfx4, AbstractLocalParticleEffectFactory *lfx5, 
						AbstractGlobalParticleEffectFactory *gfx1, AbstractGlobalParticleEffectFactory *gfx2, 
						int nbLoops)
{
	m_running=false;
	
	m_x=x;
	m_y=y; 
	m_nbParticles=nbParticles;
	m_generationTime=generationTime;
	m_movingTime=movingTime;
	
	m_anim=anim;
	m_anim->setReferencePointCenter();
	
	m_particles=new list<Particle *>();
	
	m_frameCounter=0;
	m_dist=dist;
	m_particleCounter=0;
	
	
	m_particlesStartFrame=new int[m_nbParticles];
	for(int i=0;i<m_nbParticles;i++)
	{
		m_particlesStartFrame[i]=int(ceil((m_generationTime*i)/m_nbParticles));
	}
	
	m_generationOn=true;
	m_nbLoops=nbLoops; //nombre de boucles, -1=infini
	m_currentLoop=1;
	
	//jusqu'a 5 effets locaux appliqués indépendamment à chaque particule
	m_lfx1=lfx1;
	m_lfx2=lfx2;
	m_lfx3=lfx3;
	m_lfx4=lfx4;
	m_lfx5=lfx5;
	
	//jusqu'a 2 effets globaux appliqués indépendamment à chaque particule
	m_gfx1=gfx1;		
	m_gfx2=gfx2;
	
	m_particleLifespan=particleLifespan;
	
	m_orientation=direction%360;
	m_scale=scale;
}*/

AbstractParticleGenerator::~AbstractParticleGenerator()
{
	list<Particle *>::iterator it;
	for (it=m_particles->begin(); it!=m_particles->end();it++)
    {
    	delete (*it);
    }
    delete m_particles;

	delete[] m_particlesStartFrame;
	
	delete m_animFactory;
	
	if(m_gfx1!=NULL)
		delete m_gfx1;
	if(m_gfx2!=NULL)
		delete m_gfx2;
	if(m_lfx1!=NULL)
		delete m_lfx1;
	if(m_lfx2!=NULL)
		delete m_lfx2;
	if(m_lfx3!=NULL)
		delete m_lfx3;
	if(m_lfx4!=NULL)
		delete m_lfx4;
	if(m_lfx5!=NULL)
		delete m_lfx5;
		
	delete m_alphaparams;
}


Anim *AbstractParticleGenerator::generateAnimation()
{
	return m_animFactory->newAnimInstance(); 
}

void AbstractParticleGenerator::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{	

	if(m_gfx1!=NULL) //celui la ou les 2
	{

		//alpha=new AlphaParamsState();
		m_alphaparams->saveState();
	}
	
	int lastLifeSpent=-1;
	

	list<Particle *>::iterator it;
	for (it=m_particles->begin(); it!=m_particles->end();it++)
    {
    	//particules de meme generation=memes effets alpha appliques
    	if(/*alpha!=NULL*/m_gfx1!=NULL && lastLifeSpent!=(*it)->getLifeSpent())
    	{

			m_alphaparams->restoreState();
			

			(*it)->getGlobalEffect1()->activate();

			if((*it)->getGlobalEffect2()!=NULL)
			{
				(*it)->getGlobalEffect2()->activate();
			}
			(*it)->draw(fx1, fx2, fx3, fx4, fx5);
			
			lastLifeSpent=(*it)->getLifeSpent();

    	}
    	else
    	{

    		(*it)->draw(fx1, fx2, fx3, fx4, fx5);

    	}
    }
    if(m_gfx1!=NULL /*alpha != NULL*/)
    {
    	m_alphaparams->restoreState();
    	//delete alpha;
    }

}

void AbstractParticleGenerator::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	//AlphaParamsState *alpha=NULL;
	if(m_gfx1!=NULL) //celui la ou les 2
	{
		//alpha=new AlphaParamsState();
		m_alphaparams->saveState();
	}
	
	int lastLifeSpent=-1;
	
	list<Particle *>::iterator it;
	for (it=m_particles->begin(); it!=m_particles->end();it++)
    {
    	//difference de position entre la particule et son generateur
		int diffx=(*it)->getX()-m_x;
		int diffy=(*it)->getY()-m_y;
    	
    	//particules de meme generation=memes effets alpha appliques
    	if(/*alpha!=NULL*/ m_gfx1!=NULL && lastLifeSpent!=(*it)->getLifeSpent())
    	{
			m_alphaparams->restoreState();
			
			(*it)->getGlobalEffect1()->activate();
			if((*it)->getGlobalEffect2()!=NULL)
				(*it)->getGlobalEffect2()->activate();
			
			(*it)->drawXY(x+diffx, y+diffy, fx1, fx2, fx3, fx4, fx5);
			
			lastLifeSpent=(*it)->getLifeSpent();
    	}
    	else
    	{
    		(*it)->drawXY(x+diffx, y+diffy, fx1, fx2, fx3, fx4, fx5);
    	}
    }
    if(/*alpha != NULL*/  m_gfx1!=NULL)
    {
    	m_alphaparams->restoreState();
    	//delete alpha;
    }
}

void AbstractParticleGenerator::update()
{
	//debug("update abstract partgen");
	if(m_running)
	{
		//TODO: regrouper les parcours?
		
		//nettoyage des particules terminées
		list<Particle *>::iterator it=m_particles->begin();
		bool finished=false;
		while(it!=m_particles->end() && !finished)
		{
			//debug("particle n");
			Particle * p = *it;
			if(p->isDead() || !p->isRunning())//particule terminée, tous les effets sont stoppés alors que le generateur tourne
			{
				//debug("particle dead or effects over: delete");
				delete p;
				it=m_particles->erase(it);
			}
			else //particule non finie, les suivantes n'auront pas fini non plus donc on arrete
			{
				finished=true;
			}
		}
		
		//on update le tout
		//list<Particle *>::iterator it;
		//debug("update particles");
		for (it=m_particles->begin(); it!=m_particles->end();it++)
	    {
	    	(*it)->update();
	    }

		
		//on créé les nouvelles particules
		//debug("new particles creation");
		if(m_generationOn)
		{
			m_frameCounter++;
			if(m_particleCounter<m_nbParticles)
			{
				bool continuee=true;
				while(continuee)
				{
					if(m_particlesStartFrame[m_particleCounter]<=m_frameCounter)
					{
						//debug("generate next particle");
						Particle *p=generateNextParticle();
						//debug("start particle");
						p->start();
						m_particles->push_back(p);
						//debug("push back ok");
						
						m_particleCounter++;
						if(m_particleCounter>=m_nbParticles-1)
						{
							continuee=false;
						}
					}
					else
					{
						continuee=false;
					}
				}
			}
		}
		//debug("fin new particles creation");

		
		if(m_nbLoops!=-1 && (m_particles->size()==0 && (m_frameCounter>=m_generationTime && m_currentLoop>=m_nbLoops)))
		{
			m_running=false;	
			

		}
		else if(m_nbLoops==-1 && m_frameCounter>=m_generationTime)
		{
			//on recommence (looping infini)
			m_frameCounter=0;
			m_particleCounter=0;
		}
		else
		{
			if(m_frameCounter>=m_generationTime && m_currentLoop<m_nbLoops)
			{
				//on recommence (next loop)
				m_frameCounter=0;
				m_particleCounter=0;
				m_currentLoop++;
			}
		}

	}
	//debug("end update abstract partgen");
}

/*
void AbstractParticleGenerator::update(AbstractEffectOnPositionable *fx1, AbstractEffectOnPositionable *fx2, AbstractEffectOnPositionable *fx3, AbstractEffectOnPositionable *fx4, AbstractEffectOnPositionable *fx5)
{
	State * state=new State();
	this->savestate(state);
	
	if(fx1!=NULL)
		fx1->applyOn(this);
	if(fx2!=NULL)
		fx2->applyOn(this);
	if(fx3!=NULL)
		fx3->applyOn(this);
	if(fx4!=NULL)
		fx4->applyOn(this);
	if(fx5!=NULL)
		fx5->applyOn(this);
	
	update();
	
	this->restorestate(state);
	delete state;
}
*/

void AbstractParticleGenerator::generationOn()
{
	m_generationOn=true;
}

void AbstractParticleGenerator::generationOff()
{
	m_generationOn=false;
}

bool AbstractParticleGenerator::isGenerationOn() const
{
	return m_generationOn;
}

void AbstractParticleGenerator::generationSwitchOnOff()
{
	m_generationOn=!m_generationOn;
}

int AbstractParticleGenerator::getCurrentNbParticles() const
{
	return m_particles->size();
}

void AbstractParticleGenerator::start()
{
	m_running=true;
}

void AbstractParticleGenerator::stop()
{
	m_running=false;
}

void AbstractParticleGenerator::reset()
{
	//m_generationOn=true;
	gotoStart();
	clearAllParticles();
}

void AbstractParticleGenerator::gotoStart()
{
	m_frameCounter=0;
	m_particleCounter=0;
	m_currentLoop=1;
}

void AbstractParticleGenerator::clearAllParticles()
{
	list<Particle *>::iterator it;
	for (it=m_particles->begin(); it!=m_particles->end();it++)
    {
    	delete *it;
    }
	m_particles->clear();
}

void AbstractParticleGenerator::restart()
{
	reset();
	start();
}

void AbstractParticleGenerator::setX(int x)
{
	m_x=x;
}

void AbstractParticleGenerator::setY(int y)
{
	m_y=y;
}

int AbstractParticleGenerator::getX()
{
	return m_x;
}

int AbstractParticleGenerator::getY()
{
	return m_y;
}

bool AbstractParticleGenerator::isRunning()
{
	return m_running;	
}

State * AbstractParticleGenerator::savestate(State * state)
{
	//position
	state->m_x=m_x;
	state->m_y=m_y;
	
	//rotation
	state->m_angle=m_orientation;
	
	return state;
}

void AbstractParticleGenerator::restorestate(State * state)
{
	//position
	m_x=state->m_x;
	m_y=state->m_y;
	
	//rotation
	m_orientation=state->m_angle;
}

void AbstractParticleGenerator::setPDistance(int distance)
{
	m_dist=distance;
}

int AbstractParticleGenerator::getPDistance() const
{
	return m_dist;
}

void AbstractParticleGenerator::setPTranslTime(int time)
{
	m_movingTime=time;
}

int AbstractParticleGenerator::getPTranslTime() const
{
	return m_movingTime;
}

void AbstractParticleGenerator::setPLifespan(int life)
{
	m_particleLifespan=life;
}

int AbstractParticleGenerator::getPLifespan() const
{
	return m_particleLifespan;
}

void AbstractParticleGenerator::setPAnimTime(float time)
{
	m_animFactory->setDelayTotal(time);
}

float AbstractParticleGenerator::getPAnimTime() const
{
	return m_animFactory->getDelayTotal();
}

void AbstractParticleGenerator::setScale(int scale)
{
	m_scale=scale;
}

int AbstractParticleGenerator::getScale() const
{
	return m_scale;
}

void AbstractParticleGenerator::setOrientation(int orientation)
{
	m_orientation=orientation;
}

int AbstractParticleGenerator::getOrientation()
{
	return m_orientation;
}

void AbstractParticleGenerator::setLocalEffect1(AbstractLocalEffect *fx)
{
	if(m_lfx1!=NULL)
		delete m_lfx1;
	
	m_lfx1=new PFXFExactSameLocalEffect(fx);
}

void AbstractParticleGenerator::setLocalEffect2(AbstractLocalEffect *fx)
{
	if(m_lfx2!=NULL)
		delete m_lfx2;
	
	m_lfx2=new PFXFExactSameLocalEffect(fx);
}

void AbstractParticleGenerator::setLocalEffect3(AbstractLocalEffect *fx)
{
	if(m_lfx3!=NULL)
		delete m_lfx3;
	
	m_lfx3=new PFXFExactSameLocalEffect(fx);
}

void AbstractParticleGenerator::setLocalEffect4(AbstractLocalEffect *fx)
{
	if(m_lfx4!=NULL)
		delete m_lfx4;
	
	m_lfx4=new PFXFExactSameLocalEffect(fx);
}

void AbstractParticleGenerator::setLocalEffect5(AbstractLocalEffect *fx)
{
	if(m_lfx5!=NULL)
		delete m_lfx5;
	
	m_lfx5=new PFXFExactSameLocalEffect(fx);
}

void AbstractParticleGenerator::setLocalEffect1(AbstractLocalParticleEffectFactory *fxf)
{
	if(m_lfx1!=NULL)
		delete m_lfx1;
	
	m_lfx1=fxf;
}

void AbstractParticleGenerator::setLocalEffect2(AbstractLocalParticleEffectFactory *fxf)
{
	if(m_lfx2!=NULL)
		delete m_lfx2;
	
	m_lfx2=fxf;
}

void AbstractParticleGenerator::setLocalEffect3(AbstractLocalParticleEffectFactory *fxf)
{
	if(m_lfx3!=NULL)
		delete m_lfx3;
	
	m_lfx3=fxf;
}

void AbstractParticleGenerator::setLocalEffect4(AbstractLocalParticleEffectFactory *fxf)
{
	if(m_lfx4!=NULL)
		delete m_lfx4;
	
	m_lfx4=fxf;
}

void AbstractParticleGenerator::setLocalEffect5(AbstractLocalParticleEffectFactory *fxf)
{
	if(m_lfx5!=NULL)
		delete m_lfx5;
	
	m_lfx5=fxf;
}

AbstractLocalParticleEffectFactory *AbstractParticleGenerator::getLocalEffect1() const
{
	return m_lfx1;
}

AbstractLocalParticleEffectFactory *AbstractParticleGenerator::getLocalEffect2() const
{
	return m_lfx2;
}

AbstractLocalParticleEffectFactory *AbstractParticleGenerator::getLocalEffect3() const
{
	return m_lfx3;
}

AbstractLocalParticleEffectFactory *AbstractParticleGenerator::getLocalEffect4() const
{
	return m_lfx4;
}

AbstractLocalParticleEffectFactory *AbstractParticleGenerator::getLocalEffect5() const
{
	return m_lfx5;
}


void AbstractParticleGenerator::setGlobalEffect1(AbstractGlobalEffect *fx)
{
	if(m_gfx1!=NULL)
		delete m_gfx1;
	
	m_gfx1=new PFXFExactSameGlobalEffect(fx);
}

void AbstractParticleGenerator::setGlobalEffect2(AbstractGlobalEffect *fx)
{
	if(m_gfx2!=NULL)
		delete m_gfx2;
	
	m_gfx2=new PFXFExactSameGlobalEffect(fx);
}

void AbstractParticleGenerator::setGlobalEffect1(AbstractGlobalParticleEffectFactory *fxf)
{
	if(m_gfx1!=NULL)
		delete m_gfx1;
	
	m_gfx1=fxf;
}

void AbstractParticleGenerator::setGlobalEffect2(AbstractGlobalParticleEffectFactory *fxf)
{
	if(m_gfx2!=NULL)
		delete m_gfx2;
	
	m_gfx2=fxf;
}

AbstractGlobalParticleEffectFactory *AbstractParticleGenerator::getGlobalEffect1() const
{
	return m_gfx1;
}

AbstractGlobalParticleEffectFactory *AbstractParticleGenerator::getGlobalEffect2() const
{
	return m_gfx2;
}

void AbstractParticleGenerator::setLocalEffect1SimpleTranslation(int angleShift)
{
	if(m_lfx1!=NULL)
		delete m_lfx1;
	
	m_lfx1=new PFXFSimpleTranslation(angleShift);
}

void AbstractParticleGenerator::setAnimation(string filename, short transparency)
{
	if(m_animFactory!=NULL)
		delete m_animFactory;
	
	Anim *anim=new Anim(filename, transparency);
	anim->setReferencePointCenter();
	m_animFactory=new ExactSameAnimationFactory(anim);
}

void AbstractParticleGenerator::setAnimation(Anim *anim)
{
	if(m_animFactory!=NULL)
		delete m_animFactory;
		
	m_animFactory=new ExactSameAnimationFactory(anim);
}

void AbstractParticleGenerator::setAnimation(AbstractAnimationFactory *animf)
{
	if(m_animFactory!=NULL)
		delete m_animFactory;
		
	m_animFactory=animf;
}

list<Particle*> *AbstractParticleGenerator::getParticles() const
{
	return m_particles;
}

//void AbstractParticleGenerator::destroyParticle(Particle *p)
//{
////	for (list<Particle*>::iterator it=m_particles->begin(); it!=m_particles->end(); ++it)
////	{
////		
////	}
//	//ce delete n'est pas de trop (checked)
//	m_particles->remove(p);
//	delete p;
//	debug("destruction particle ok");
//}

list<Particle *>::iterator AbstractParticleGenerator::destroyParticle(list<Particle *>::iterator it)
{
	delete *it;
	return m_particles->erase(it);
}






