#ifndef ABSTRACTPARTICLEGENERATOR_H_
#define ABSTRACTPARTICLEGENERATOR_H_

#include <oslib/oslib.h>
#include <string>
#include "code/framework/particles/complete/Particle.h"
#include <list>
#include "code/framework/interfaces/Drawable.h"
#include "code/framework/interfaces/Positionable.h"
#include "code/framework/interfaces/Playable.h"
#include "code/framework/interfaces/Orientable.h"
#include "code/framework/interfaces/StateSavable.h"
#include "code/framework/State.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/particles/complete/fxFactories/abstract/AbstractGlobalParticleEffectFactory.h"
#include "code/framework/particles/complete/fxFactories/abstract/AbstractLocalParticleEffectFactory.h"
#include "code/framework/particles/complete/animFactories/abstract/AbstractAnimationFactory.h"
#include "code/framework/fx/utils/AlphaParamsState.h"

using namespace std;

class AbstractParticleGenerator: public Drawable, public Positionable, public Playable, public Orientable
{
	private:
		//durée de generation
		int m_nbLoops; //nombre de boucles, -1=infini
		int m_currentLoop;
		
		bool m_generationOn;
		
		int m_running;
		
		//image partagée par les particules
		//OSL_IMAGE *m_img;
		AbstractAnimationFactory *m_animFactory;
		
		int m_frameCounter;
		int m_particleCounter; //a quelle particule on en est
		
		int m_nbParticles;
		int m_generationTime;
		
		list<Particle*> * m_particles;
		
		int *m_particlesStartFrame; //tableau d'entiers, décrit a quelle frame la particule démarre
				
		
//		void initialize(int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
//						int dist, int particleLifespan, Anim *anim, AbstractLocalParticleEffectFactory *lfx1, 
//						AbstractLocalParticleEffectFactory *lfx2, AbstractLocalParticleEffectFactory *lfx3, 
//						AbstractLocalParticleEffectFactory *lfx4, AbstractLocalParticleEffectFactory *lfx5, 
//						AbstractGlobalParticleEffectFactory *gfx1, AbstractGlobalParticleEffectFactory *gfx2, 
//						int nbLoops);
		AlphaParamsState *m_alphaparams;
		
	protected:
		//position
		int m_x;
		int m_y;
		
		int m_dist;//distance a parcourir
		int m_movingTime;//temps de parcours
		int m_orientation; //angle en degrés
		int m_scale; //envergure/taille
		int m_particleLifespan; //duree de vie d'une particule, -1=on attend la fin de tous les effets appliqués, et là elle meurt
		
	
		//a surcharger pour fabriquer d'autres types de générateurs (multi anims, translations différentes...)
		virtual Particle *generateNextParticle()=0;
		virtual Anim *generateAnimation();
		
		//jusqu'a 5 effets locaux appliqués indépendamment à chaque particule
		AbstractLocalParticleEffectFactory *m_lfx1;
		AbstractLocalParticleEffectFactory *m_lfx2;
		AbstractLocalParticleEffectFactory *m_lfx3;
		AbstractLocalParticleEffectFactory *m_lfx4;
		AbstractLocalParticleEffectFactory *m_lfx5;
		
		//jusqu'a 2 effets globaux appliqués indépendamment à chaque particule
		AbstractGlobalParticleEffectFactory *m_gfx1;		
		AbstractGlobalParticleEffectFactory *m_gfx2;
		
	public:
		AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
									int dist, int particleLifespan, AbstractAnimationFactory *animFactory, int nbLoops=-1);
		AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
									int dist, int particleLifespan, Anim *anim, int nbLoops=-1);
		AbstractParticleGenerator(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, 
									int dist, int particleLifespan, string filename, short transparency, int nbLoops=-1);
									
		virtual ~AbstractParticleGenerator();
		
		virtual AbstractParticleGenerator *clone() const=0;
		
		//set nombre de particules par nombre de frame
		void setThroughput(int nbParticles, int generationTime);
		int getThroughputNbParticles() const;
		int getThroughputFrameTime() const;
		
		//drawable
		void draw(AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		void drawXY(int x, int y, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		
		//statesavable
		State * savestate(State * state);
		void restorestate(State * state);
		
		//playable
		void update();
		//void update(AbstractEffectOnPositionable *fx1, AbstractEffectOnPositionable *fx2=NULL, AbstractEffectOnPositionable *fx3=NULL, AbstractEffectOnPositionable *fx4=NULL, AbstractEffectOnPositionable *fx5=NULL);
		void start();
		void stop();
		void reset();
		void restart();
		bool isRunning();
		//addition to playable functions, to not destroy existing particles
		void gotoStart();
		
		//positionable
		void setX(int x);
		void setY(int y);
		int getX();
		int getY();
		
		int getCurrentNbParticles() const;
		void generationOn();
		void generationOff();
		bool isGenerationOn() const;
		void generationSwitchOnOff();
		
		//getters setters pour modifications dynamiques
		void setPDistance(int distance);
		int getPDistance() const;
		void setPTranslTime(int time);
		int getPTranslTime() const;
		void setPLifespan(int life);
		int getPLifespan() const;
		void setPAnimTime(float time);
		float getPAnimTime() const;
		virtual void setScale(int scale);
		int getScale() const;
		virtual void setOrientation(int orientation);
		int getOrientation();
		int getNbLoops() const;
		
		void setLocalEffect1(AbstractLocalEffect *fx);
		void setLocalEffect2(AbstractLocalEffect *fx);
		void setLocalEffect3(AbstractLocalEffect *fx);
		void setLocalEffect4(AbstractLocalEffect *fx);
		void setLocalEffect5(AbstractLocalEffect *fx);
		void setLocalEffect1(AbstractLocalParticleEffectFactory *fxf);
		void setLocalEffect2(AbstractLocalParticleEffectFactory *fxf);
		void setLocalEffect3(AbstractLocalParticleEffectFactory *fxf);
		void setLocalEffect4(AbstractLocalParticleEffectFactory *fxf);
		void setLocalEffect5(AbstractLocalParticleEffectFactory *fxf);
		AbstractLocalParticleEffectFactory *getLocalEffect1() const;
		AbstractLocalParticleEffectFactory *getLocalEffect2() const;
		AbstractLocalParticleEffectFactory *getLocalEffect3() const;
		AbstractLocalParticleEffectFactory *getLocalEffect4() const;
		AbstractLocalParticleEffectFactory *getLocalEffect5() const;
		
		void setGlobalEffect1(AbstractGlobalEffect *fx);
		void setGlobalEffect2(AbstractGlobalEffect *fx);
		void setGlobalEffect1(AbstractGlobalParticleEffectFactory *fxf);
		void setGlobalEffect2(AbstractGlobalParticleEffectFactory *fxf);
		AbstractGlobalParticleEffectFactory *getGlobalEffect1() const;
		AbstractGlobalParticleEffectFactory *getGlobalEffect2() const;
		
		void setLocalEffect1SimpleTranslation(int angleShift=0);
		
		void setAnimation(string filename, short transparency);
		void setAnimation(Anim *anim);
		void setAnimation(AbstractAnimationFactory *animf);
		AbstractAnimationFactory *getAnimationFactory() const;
		
		list<Particle*> *getParticles() const;
		//void destroyParticle(Particle *p);
		list<Particle *>::iterator destroyParticle(list<Particle *>::iterator it);
		void clearAllParticles();
};

#endif /*ABSTRACTPARTICLEGENERATOR_H_*/




