#include <oslib/oslib.h>
#include "code/framework/particles/complete/impl/ParticleGeneratorPoint.h"
#include "code/framework/utils.h"
#include "code/framework/fx/utils/AlphaParamsState.h"
#include "code/framework/particles/complete/fxFactories/PFXFExactSameLocalEffect.h"
#include "code/framework/particles/complete/fxFactories/PFXFExactSameGlobalEffect.h"

ParticleGeneratorPoint::ParticleGeneratorPoint(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, AbstractAnimationFactory *animFactory, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, animFactory, nbLoops)
{
	m_scale_small=getOrientation()-int(getScale()/2);
	m_scale_big=m_scale_small+m_scale;
}

ParticleGeneratorPoint::ParticleGeneratorPoint(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, string filename, short transparency, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, filename, transparency, nbLoops)
{
	
	m_scale_small=getOrientation()-int(getScale()/2);
	m_scale_big=m_scale_small+m_scale;
}



ParticleGeneratorPoint::ParticleGeneratorPoint(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, Anim *anim, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, anim, nbLoops)
{
	m_scale_small=m_orientation-int(m_scale/2);
	m_scale_big=m_scale_small+m_scale;
}

ParticleGeneratorPoint *ParticleGeneratorPoint::clone() const
{
	ParticleGeneratorPoint *theclone = new ParticleGeneratorPoint(
		m_x, m_y, m_orientation, m_scale, 
		getThroughputNbParticles(), getThroughputFrameTime(), m_movingTime, 
		m_dist, m_particleLifespan, getAnimationFactory()->clone(), getNbLoops()
	);
	
	
	if(m_lfx1!=NULL)
		theclone->setLocalEffect1(m_lfx1->clone());
	if(m_lfx2!=NULL)
		theclone->setLocalEffect2(m_lfx2->clone());
	if(m_lfx3!=NULL)
		theclone->setLocalEffect3(m_lfx3->clone());
	if(m_lfx4!=NULL)
		theclone->setLocalEffect4(m_lfx4->clone());
	if(m_lfx5!=NULL)
		theclone->setLocalEffect5(m_lfx5->clone());
	if(m_gfx1!=NULL)
		theclone->setGlobalEffect1(m_gfx1->clone());
	if(m_gfx2!=NULL)
		theclone->setGlobalEffect2(m_gfx2->clone());
	
	return theclone;
}

ParticleGeneratorPoint::~ParticleGeneratorPoint()
{
	//le parent fait tout
}

Particle *ParticleGeneratorPoint::generateNextParticle()
{	
	//debug("particlegenpoint: generateNextParticle");
	int randomAngle=utils::random(m_scale_small, m_scale_big);
	
	//creation des effets pour la particule
	AbstractLocalEffect *lfx1=NULL;	
	if(m_lfx1!=NULL)
		lfx1=m_lfx1->newLocalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx2=NULL;
	if(m_lfx2!=NULL)
		lfx2=m_lfx2->newLocalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx3=NULL;
	if(m_lfx3!=NULL)
		lfx3=m_lfx3->newLocalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx4=NULL;
	if(m_lfx4!=NULL)
		lfx4=m_lfx4->newLocalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx5=NULL;
	if(m_lfx5!=NULL)
		lfx5=m_lfx5->newLocalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	AbstractGlobalEffect *gfx1=NULL;		
	if(m_gfx1!=NULL)
		gfx1=m_gfx1->newGlobalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);	
	AbstractGlobalEffect *gfx2=NULL;	
	if(m_gfx2!=NULL)
		gfx2=m_gfx2->newGlobalEffectInstance(0, 0, m_dist, m_movingTime, randomAngle, m_scale, m_particleLifespan);
	
	//debug("fx ok");
	Particle * p=new Particle(m_x, m_y, generateAnimation(), m_particleLifespan, lfx1, lfx2, lfx3, lfx4, lfx5, gfx1, gfx2);
	//debug("new particle ok");
		
	return p;
}



void ParticleGeneratorPoint::setScale(int scale)
{
	AbstractParticleGenerator::setScale(scale);
	m_scale_small=m_orientation-int(m_scale/2);
	m_scale_big=m_scale_small+m_scale;
}



void ParticleGeneratorPoint::setOrientation(int orientation)
{
	AbstractParticleGenerator::setOrientation(orientation);
	m_scale_small=m_orientation-int(m_scale/2);
	m_scale_big=m_scale_small+m_scale;
}

















