#include "code/framework/resRepo/abstract/AbstractResRepo.h"
#include <istream>
#include <iostream>
#include <sstream>
#include "code/framework/utils.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/bmpFont/BmpFont.h"

using namespace std;
using namespace utils;


AbstractResRepo::~AbstractResRepo()
{
	
}

Properties *AbstractResRepo::readPropertiesFile(string filename)
{
	istream *stream=readTextFileStream(filename);
	Properties *props=new Properties(stream);
	delete stream;
	return props;
}

Anim *AbstractResRepo::readAnim(unsigned int nbImages, string header, string where, float delay_total, short transparency, int maxLoops)
{
	
	//1ere image avec construction anim
	string filename=where+header+"1.png";
	string imageId=getName()+"::"+filename;
	Anim *anim=NULL;
	OSL_IMAGE *img=NULL;
	ImageAndAttr *iaa=Anim::getImageWithName(imageId);
	if(iaa==NULL)
	{
		img=readPngFile(filename, OSL_IN_RAM|OSL_SWIZZLED, transparency);
		anim=new Anim(img, imageId, delay_total, maxLoops);
	}
	else
	{
		anim=new Anim(imageId, delay_total, transparency, maxLoops); //ira la chercher en buffer
	}
	
	
	//images suivantes
	for(unsigned int i=2;i<=nbImages;i++)
	{
		filename=where+header+toString(i)+".png";
		imageId=getName()+"::"+filename;
		iaa=Anim::getImageWithName(imageId);
		if(iaa==NULL)
		{
			img=readPngFile(filename, OSL_IN_RAM|OSL_SWIZZLED, transparency);
			anim->addImage(img, imageId);
		}
		else
		{
			anim->addImage(imageId, transparency); //ira la chercher en buffer
		}
	}
	
	return anim;
}

//anim a 1 image
Anim *AbstractResRepo::readAnim(string filename, float delay_total, short transparency, int maxLoops)
{
	if(delay_total==0.0f)
		maxLoops=0;
	
	string imageId=getName()+"::"+filename;
	Anim *anim=NULL;
	OSL_IMAGE *img=NULL;
	ImageAndAttr *iaa=Anim::getImageWithName(imageId);
	if(iaa==NULL)
	{
		img=readPngFile(filename, OSL_IN_RAM|OSL_SWIZZLED, transparency);
		anim=new Anim(img, imageId, delay_total, maxLoops);
	}
	else
	{
		anim=new Anim(imageId, delay_total, transparency, maxLoops); //ira la chercher en buffer
	}
	
	return anim;
}

void AbstractResRepo::addImageToAnim(Anim *anim, string filename, int transparency)
{
	string imageId=getName()+"::"+filename;
	ImageAndAttr *iaa=Anim::getImageWithName(imageId);
	if(iaa==NULL)
	{
		OSL_IMAGE *image=readPngFile(filename, OSL_IN_RAM|OSL_SWIZZLED, transparency);
		anim->addImage(image, imageId);
	}
	else //iaa!=NULL
	{
		anim->addImage(imageId, transparency); //ira la chercher en buffer
	}
}

BmpFont *AbstractResRepo::readBmpFont(string where, string filename, short transparency, bool cursive)
{
	string imagename=where+filename+".png";
	string sizefilename=where+filename+".ini";
	
	OSL_IMAGE *font_img=readPngFile(imagename, OSL_IN_RAM|OSL_SWIZZLED, transparency);
	istream *font_sizes=readTextFileStream(sizefilename);
	BmpFont *font=new BmpFont(font_img, font_sizes, cursive);
	delete font_sizes;
	
	return font;
}


















