#ifndef ABSTRACTRESREPO_H_
#define ABSTRACTRESREPO_H_

#include <vector>
#include <istream>
#include <string>
#include <string>
#include <map>
#include <oslib/oslib.h>
#include "code/framework/resRepo/ResFileDetails.h"
#include "code/framework/Properties.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/bmpFont/BmpFont.h"

using namespace std;

//resources repository, can be an archive of any kind, or simply the filesystem
//depending on the implementation chosen
//use the factory to initialise and get the right repository
class Anim; //pour friend

class AbstractResRepo
{
	private:
		
		
	public:
		//AbstractResRepo(string filename); //pas besoin on n'y fait rien
		virtual ~AbstractResRepo();
		
		virtual long getNbFiles()=0;
		virtual vector<string> getFilesNames()=0;
		virtual vector<ResFileDetails> getFilesDetails()=0;
		virtual ResFileDetails getFileDetails(string filename)=0;
		virtual FILE *open(string filename)=0;
		virtual string getName()=0;
		
		virtual string readTextFileString(string filename)=0;
		virtual istream *readTextFileStream(string filename)=0; //il faudra faire un delete sur l'istream apres utilisation!
		virtual OSL_IMAGE *readPngFile(string filename, int location, int pixelFormat)=0;
		virtual OSL_SOUND *readBgmFile(string filename)=0;
		virtual bool extractFileTo(string archivefile, string memorystickfile)=0; //renvoie true si succès
		virtual bool extractAllToFolder(string dir)=0;
		
		//not to be overloaded:
		Properties *readPropertiesFile(string filename);
		Anim *readAnim(unsigned int nbImages, string header, string where, float delay_total, short transparency=OSL_PF_8888, int maxLoops=-1);
		Anim *readAnim(string filename, float delay_total=0.0f, short transparency=OSL_PF_8888, int maxLoops=-1);
		void addImageToAnim(Anim *anim, string filename, int transparency=OSL_PF_8888);
		BmpFont *readBmpFont(string where, string filename, short transparency, bool cursive=false);

		friend class Anim;
};


#endif /*ABSTRACTRESREPO_H_*/
