#include "code/framework/sound/ChannelManager.h"
#include "code/framework/utils.h"

using namespace utils;

#define CHANNELMANAGER_INFINITE -1
#define CHANNELMANAGER_NOTRESERVED -2

ChannelManager::ChannelManager()
{
	m_timers=new Timer*[8]; //tableau de timers
	m_limits=new int[8]; //tableau d'entiers temps, de réservation
	//m_available=new bool[8]; //tableau de booleens, channel dispo ou pas
		
	m_sounds=new OSL_SOUND *[8]; //tableau de pointeurs sur sons 
	
	for(int i=0;i<8;i++)
	{
		m_sounds[i]=NULL;
		//m_available[i]=true;
		m_timers[i]=new Timer();
		m_limits[i]=0;
	}
	
	m_currentChoice=0;
}

ChannelManager::~ChannelManager()
{
	for(int i=0;i<8;i++)
	{
		delete m_timers[i];
	}
	delete[] m_timers;
	
	delete[] m_limits;
	//delete[] m_available;
	delete[] m_sounds;
}

void ChannelManager::blockChannel(int num)
{
	m_limits[num]=CHANNELMANAGER_INFINITE;
}

int ChannelManager::lookForSound(OSL_SOUND *snd)
{
	for(int i=0;i<8;i++)
	{
		if(m_sounds[i]==snd)
			return i;
	}
	
	return -1;
}


//libère toutes les channels
void ChannelManager::reset()
{
	for(int i=0;i<8;i++)
	{
		m_sounds[i]=NULL;
		//m_available[i]=true;
		m_timers[i]->stop();
		m_timers[i]->reset();
	}
}


int ChannelManager::searchBestChannel() //recherche la channel a utiliser
{
	//recherche d'une channel libre
	for(int i=0;i<8;i++)
	{
		if(m_limits[i]>=0 && (m_limits[i]==0 || m_timers[i]->getTime()>=m_limits[i]))
		{
			return i;
		}
	}
		
		
	//si pas trouvé, recherche une channel non bloquée
	for(int i=1;i<8;i++)
	{
		int num=(m_currentChoice+i)%8;
		if(m_limits[num]==CHANNELMANAGER_NOTRESERVED)
		{
			m_currentChoice=num;
			return num;
		}
	}
	
	
	//sinon, utilisation de la channel la plus proche de finir
	int chan=0;
	int time;
	
	if(m_limits[0]==CHANNELMANAGER_INFINITE)
	{
		time=-1;
	}
	else
	{
		time=m_limits[0]-m_timers[0]->getTime();
	}
	for(int i=1;i<8;i++)
	{
		int time2=m_limits[i]-m_timers[i]->getTime();
		if(time==-1 || time2<time)
		{
			time=time2;
			chan=i;
		}
	}
	
	return chan;
}

//renvoient le num de channel
int ChannelManager::playSoundAndReserve(OSL_SOUND *snd, int millisec) //-1 infinite
{
	int chan=searchBestChannel();
	
	m_sounds[chan]=snd;
	m_limits[chan]=millisec;
	m_timers[chan]->restart();
	
	oslPlaySound(snd, chan);
	
	return chan;
}

//mais ne reserve pas, donc s'il n'y a pas de channel libre,
//il risque de se couper avant de finir
int ChannelManager::playSound(OSL_SOUND *snd)
{
	int chan=searchBestChannel();
	
	m_sounds[chan]=snd;
	m_limits[chan]=CHANNELMANAGER_NOTRESERVED;
	m_timers[chan]->restart();
	
	oslPlaySound(snd, chan);
	
	return chan;
}

void ChannelManager::pauseSound(OSL_SOUND *snd)
{
	int chan=lookForSound(snd);
	//int chan=oslGetSoundChannel(snd);
	if(chan==-1)
		fatalError("ChannelManager::pauseSound: sound not found");
	
	oslPauseSound(snd, 1);
	m_timers[chan]->stop();
}

void ChannelManager::resumeSound(OSL_SOUND *snd)
{
	//int chan=oslGetSoundChannel(snd);
	int chan=lookForSound(snd);
	if(chan==-1)
		fatalError("ChannelManager::resumeSound: sound not found");
	
	oslPauseSound(snd, 0);
	m_timers[chan]->stop();
}

//libere la channel
void ChannelManager::stopSound(OSL_SOUND *snd)
{
	//int chan=oslGetSoundChannel(snd);
	int chan=lookForSound(snd);
	if(chan==-1)
		fatalError("ChannelManager::stopSound: sound not found");
		
	oslStopSound(snd);
	m_timers[chan]->stop();
	m_timers[chan]->reset();
	m_limits[chan]=0;
	m_sounds[chan]=NULL;
}

/*void ChannelManager::pauseAll()
{
	
}

void ChannelManager::resumeAll()
{
	
}*/

int ChannelManager::restartSound(OSL_SOUND *snd)
{
	//int chan=oslGetSoundChannel(snd);
	int chan=lookForSound(snd);
	if(chan==-1)
	{
		chan=searchBestChannel();
		m_sounds[chan]=snd;
		m_limits[chan]=CHANNELMANAGER_NOTRESERVED;
	}
	//else c'est deja bon
	
	
	m_timers[chan]->restart();
	
	oslPlaySound(snd, chan);
	
	return chan;
}

int ChannelManager::restartSoundAndReserve(OSL_SOUND *snd, int millisec)
{
	//int chan=oslGetSoundChannel(snd);
	int chan=lookForSound(snd);
	if(chan==-1)
	{
		chan=searchBestChannel();
		m_sounds[chan]=snd;
		m_limits[chan]=millisec;
	}
	//else c'est deja bon
	
	m_timers[chan]->restart();
	
	oslPlaySound(snd, chan);
	
	return chan;
}












