#include "code/game/Pair.h"
//#include "code/def.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include <string>
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/abstract/AbstractPiece.h"

using namespace std;

Pair::~Pair()
{
	if(m_piece[0]!=NULL)
		delete m_piece[0];
	if(m_piece[1]!=NULL)
		delete m_piece[1];
	
	delete[] m_piece;
}

Pair::Pair(GameGrid *grid) //random
//: Trackable("Pair")
{
	m_orientation=ORIENTATION_EAST;
	m_gridPosX=int(grid->getNbCols()/2)-1;
	m_gridPosY=-1; //au dessus de tout en haut
	m_piece=new AbstractPiece*[2];
	m_piece[0]=new PieceSimple();
	m_piece[1]=new PieceSimple();
}

Pair::Pair(GameGrid *grid, int shape1, int color1, int shape2, int color2)
//: Trackable("Pair")
{
	m_orientation=ORIENTATION_EAST;
	m_gridPosX=int(grid->getWidth()/2)-1;
	m_gridPosY=-1; //au dessus de tout en haut
	m_piece=new AbstractPiece*[2];
	m_piece[0]=new PieceSimple(shape1, color1);
	m_piece[1]=new PieceSimple(shape2, color2);
}

Pair::Pair(GameGrid *grid, AbstractPiece *piece0, AbstractPiece *piece1)
//: Trackable("Pair")
{
	m_orientation=ORIENTATION_EAST;
	m_gridPosX=int(grid->getWidth()/2)-1;
	m_gridPosY=-1; //au dessus de tout en haut
	m_piece=new AbstractPiece*[2];
	m_piece[0]=piece0;
	m_piece[1]=piece1;
}

Pair::Pair(AbstractPiece *piece0, AbstractPiece *piece1)
//: Trackable("Pair")
{
	m_orientation=ORIENTATION_EAST;
	m_gridPosX=0;
	m_gridPosY=-1; //au dessus de tout en haut
	m_piece=new AbstractPiece*[2];
	m_piece[0]=piece0;
	m_piece[1]=piece1;
}

Pos Pair::getGridPos(int pieceNum)
{
	if(pieceNum!=0 && pieceNum!=1)
	{
		oslFatalError("Pair::getGridPos: pieceNum doit etre egal a 0 ou 1");
	}
	
	if(pieceNum==0)
	{
		return Pos(m_gridPosX, m_gridPosY);	
	}	
	
	//else
	switch(m_orientation)
	{
		case ORIENTATION_EAST:
			return Pos(m_gridPosX+1, m_gridPosY);
			break;	
		
		case ORIENTATION_NORTH:
			return Pos(m_gridPosX, m_gridPosY-1);
			break;
			
		case ORIENTATION_SOUTH:
			return Pos(m_gridPosX, m_gridPosY+1);
			break;
			
		case ORIENTATION_WEST:
			return Pos(m_gridPosX-1, m_gridPosY);
			break;
	}
	
	//ne sera jamais appelé mais satisfait le compilo
	return Pos(m_gridPosX, m_gridPosY);	
}



AbstractPiece *Pair::getPiece(int num)
{
	return m_piece[num];
}

int Pair::getOrientation()
{
	return m_orientation;
}

void Pair::setOrientation(int orientation)
{
	m_orientation=orientation;
}

//ne fait aucune vérif
int Pair::rotate(bool clockwise)
{
	if(clockwise)
	{
		switch(m_orientation)
		{
			case ORIENTATION_EAST:
				m_orientation=ORIENTATION_SOUTH;
				break;	
			
			case ORIENTATION_NORTH:
				m_orientation=ORIENTATION_EAST;
				break;
				
			case ORIENTATION_SOUTH:
				m_orientation=ORIENTATION_WEST;
				break;
				
			case ORIENTATION_WEST:
				m_orientation=ORIENTATION_NORTH;
				break;
		}
	}
	else //counter clockwise
	{
		switch(m_orientation)
		{
			case ORIENTATION_EAST:
				m_orientation=ORIENTATION_NORTH;
				break;	
			
			case ORIENTATION_NORTH:
				m_orientation=ORIENTATION_WEST;
				break;
				
			case ORIENTATION_SOUTH:
				m_orientation=ORIENTATION_EAST;
				break;
				
			case ORIENTATION_WEST:
				m_orientation=ORIENTATION_SOUTH;
				break;
		}
	}
	
	return m_orientation;
}

void Pair::draw(GameGrid *grid, SerialLocalEffect *fx_transl, int applyFxOn, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4)
{
	//debug("dessin d'une paire");
	//debug("position piece 0 :("+toString(m_gridPosX)+","+toString(m_gridPosY)+")");
	Pos posPiece0=grid->getContentPosition(m_gridPosX, m_gridPosY);
	int x2=m_gridPosX, y2=m_gridPosY;
	switch(m_orientation)
	{
		case ORIENTATION_EAST:
			x2++;
			break;	
		
		case ORIENTATION_NORTH:
			y2--;
			break;
			
		case ORIENTATION_SOUTH:
			y2++;
			break;
			
		case ORIENTATION_WEST:
			x2--;
			break;
	}
	
	//debug("position piece 1:("+toString(x2)+","+toString(y2)+")");
	Pos posPiece1=grid->getContentPosition(x2, y2);
	//debug("positions ok.");
	drawXY(posPiece0.x, posPiece0.y, posPiece1.x, posPiece1.y, fx_transl, applyFxOn, fx1, fx2, fx3, fx4);
}

void Pair::drawXY(int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl, int applyFxOn, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(m_orientation)
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
	
	drawXY(x, y, x2, y2, fx_transl, applyFxOn, fx1, fx2, fx3, fx4);
}

void Pair::drawXY(int x, int y, int x2, int y2, SerialLocalEffect *fx_transl, int applyFxOn, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4)
{
	//debug("dessin x, y, x2, y2:"+toString(x)+", "+toString(y)+", "+toString(x2)+", "+toString(y2));
	switch(applyFxOn)
	{
		case APPLY_FX_ON_1:
			getPiece(1)->drawXY(x2, y2, fx_transl);
			getPiece(0)->drawXY(x, y, fx_transl, fx1, fx2, fx3, fx4);
			break;
			
		case APPLY_FX_ON_2:
			getPiece(0)->drawXY(x, y, fx_transl);
			getPiece(1)->drawXY(x2, y2, fx_transl, fx1, fx2, fx3, fx4);
			break;
			
		case APPLY_FX_ON_BOTH:
			getPiece(0)->drawXY(x, y, fx_transl, fx1, fx2, fx3, fx4);
			getPiece(1)->drawXY(x2, y2, fx_transl, fx1, fx2, fx3, fx4);
			break;
		
		default:
			getPiece(0)->drawXY(x, y, fx_transl);
			getPiece(1)->drawXY(x2, y2, fx_transl);
	}
}


//ne fait aucune vérif
void Pair::move(int direction)
{
	switch(direction)
	{
		case DIRECTION_DOWN:
			m_gridPosY++;
			break;
			
		case DIRECTION_LEFT:
			m_gridPosX--;
			break;
			
		case DIRECTION_RIGHT:
			m_gridPosX++;
			break;
			
		case DIRECTION_UP:
			m_gridPosY--;
			break;
	}
	
	
	//calculatePixPos();
}

AbstractPiece *Pair::detach(int num)
{
	AbstractPiece *ptemp=m_piece[num];
	m_piece[num]=NULL;
	return ptemp;
}

void Pair::update()
{
	getPiece(0)->update();
	getPiece(1)->update();
}

void Pair::switchPieces()
{
	AbstractPiece *ptemp=m_piece[0];
	m_piece[0]=m_piece[1];
	m_piece[1]=ptemp;
}

void Pair::goTo(int col, int line)
{
	m_gridPosX=col;
	m_gridPosY=line;
}

void Pair::setPiece0(AbstractPiece *piece0)
{
	if(m_piece[0]!=NULL)
		delete m_piece[0];
	m_piece[0]=piece0;
}

void Pair::setPiece1(AbstractPiece *piece1)
{
	if(m_piece[1]!=NULL)
		delete m_piece[1];
	m_piece[1]=piece1;
}





