#include "code/game/combos/ComboNumberDrawer.h"
#include <oslib/oslib.h>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/utils.h"
#include "code/framework/fx/EffectsLib.h"
#include <string>
#include "code/game/gamedef.h"

using namespace std;
using namespace utils;

#define COMBO_NUMBER_APPEARDOWN_TIME 10
#define COMBO_NUMBER_WAIT_TIME 55
#define COMBO_NUMBER_VANISH_TIME 30
#define COMBO_NUMBER_COLORCHANGE_DELAY 10
#define COMBO_NUMBER_TRANSPARENCY 200

void ComboNumberDrawer::init(OSL_COLOR color_normal_in, OSL_COLOR color_out, OSL_COLOR color_hot1, OSL_COLOR color_hot2, OSL_COLOR color_hot3, string fontin_folder, string fontin_name, string fontout_folder, string fontout_name, bool invert)
{
	//message("debut: "+toString(mallinfo().fordblks));
	BmpFont * font_in=new BmpFont(invert?fontout_folder:fontin_folder, invert?fontout_name:fontin_name, OSL_PF_8888);
	BmpFont * font_out=new BmpFont(invert?fontin_folder:fontout_folder, invert?fontin_name:fontout_name, OSL_PF_8888);
	//message("apres 2 fonts: "+toString(mallinfo().fordblks));
	
	BmpText *txtcombo_in=new BmpText("x combo", font_in); //TODO: internationaliser ca
	txtcombo_in->compile();
	//message("apres txtcomboin: "+toString(mallinfo().fordblks));
	
	BmpText *txtcombo_out=new BmpText("x combo", font_out); //TODO: internationaliser ca
	txtcombo_out->compile();
	//message("apres txtcomboout: "+toString(mallinfo().fordblks));
	
	m_img_combo_in=txtcombo_in->createSwizzledImageCopy();
	m_img_combo_out=txtcombo_out->createSwizzledImageCopy();
	delete txtcombo_in;
	delete txtcombo_out;
	//message("apres delete txtcombo: "+toString(mallinfo().fordblks));
	
	m_img_numbers_in=new OSL_IMAGE *[10]; //de 0 a 9
	m_img_numbers_out=new OSL_IMAGE *[10]; //de 0 a 9
	//message("apres decl imgnumbers: "+toString(mallinfo().fordblks));
	for(int i=0;i<10;i++)
	{
		BmpText *txt_in=new BmpText(toString(i), font_in);
		txt_in->compile();
		m_img_numbers_in[i]=txt_in->createSwizzledImageCopy();
		delete txt_in;
		BmpText *txt_out=new BmpText(toString(i), font_out);
		txt_out->compile();
		m_img_numbers_out[i]=txt_out->createSwizzledImageCopy();
		delete txt_out;
	}
	//message("apres remplissage: "+toString(mallinfo().fordblks));
	
	delete font_in;
	delete font_out;
	//message("apres delete font: "+toString(mallinfo().fordblks));
	
	m_img_num1_in=NULL;
	m_img_num2_in=NULL;
	m_img_num3_in=NULL;
	m_img_num1_out=NULL;
	m_img_num2_out=NULL;
	m_img_num3_out=NULL;
	
	m_fx_appearWait=new SerialLocalEffect(1);
	m_fx_appearWait->addEffect(new AppearDownEffect(COMBO_NUMBER_APPEARDOWN_TIME));
	m_fx_appearWait->addEffect(new WaitLocalEffect(COMBO_NUMBER_WAIT_TIME+COMBO_NUMBER_VANISH_TIME));
	//message("apres appearwait: "+toString(mallinfo().fordblks));
	
	m_fx_waitVanish=new SerialGlobalEffect(1);
	m_fx_waitVanish->addEffect(new WaitGlobalEffect(COMBO_NUMBER_APPEARDOWN_TIME+COMBO_NUMBER_WAIT_TIME));
	m_fx_waitVanish->addEffect(new ProgressiveAlphaEffect(COMBO_NUMBER_VANISH_TIME, COMBO_NUMBER_VANISH_TIME, 255, 0, 1));
	//message("apres waitvanish: "+toString(mallinfo().fordblks));
	
	m_colors_hot[0]=color_hot1; //RGBA(255,0,0,COMBO_NUMBER_TRANSPARENCY);
	m_colors_hot[1]=color_hot2; //RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	m_colors_hot[2]=color_hot3; //RGBA(255,255,0,COMBO_NUMBER_TRANSPARENCY);
	//message("apres colorshot: "+toString(mallinfo().fordblks));
	
	m_color_normal_in=color_normal_in;//RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	m_color_out=color_out;
	
	m_colornum=0;
	
	m_colorChangeDelayer=new Delayer(COMBO_NUMBER_COLORCHANGE_DELAY);
	//message("apres delayer: "+toString(mallinfo().fordblks));
	
	m_currentComboNumber=0;
	
	m_fx_tint_in=new ProgressiveTintEffect(1, 1, m_color_normal_in, m_color_normal_in); //on l'updatera pas
	m_fx_tint_out=new ProgressiveTintEffect(1, 1, m_color_out, m_color_out); //on l'updatera pas
	//message("apres fxtint: "+toString(mallinfo().fordblks));
	
	//debug("construction combonumberdrawer ok");
	//enableDebug(false);
	
	m_x=0;
	m_y=0;
}

ComboNumberDrawer::ComboNumberDrawer()
{
	int color_normal_in=RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	int color_out=RGBA(0,0,0,COMBO_NUMBER_TRANSPARENCY);
	int color_hot1=RGBA(255,0,0,COMBO_NUMBER_TRANSPARENCY);
	int color_hot2=RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	int color_hot3=RGBA(255,255,0,COMBO_NUMBER_TRANSPARENCY);
	
	string fontin_folder="res/fonts/bmpFonts/unlearned23/inout/";
	string fontin_name="in";
	string fontout_folder="res/fonts/bmpFonts/unlearned23/inout/";
	string fontout_name="out";
	
	m_fx_appearWait=NULL;
	init(color_normal_in, color_out, color_hot1, color_hot2, color_hot3, fontin_folder, fontin_name, fontout_folder, fontout_name);
}

ComboNumberDrawer::ComboNumberDrawer(OSL_COLOR color_normal_in, OSL_COLOR color_out, OSL_COLOR color_hot1, OSL_COLOR color_hot2, OSL_COLOR color_hot3, bool invert)
{
	string fontin_folder="res/fonts/bmpFonts/unlearned23/inout/";
	string fontin_name="in";
	string fontout_folder="res/fonts/bmpFonts/unlearned23/inout/";
	string fontout_name="out";
	
	init(color_normal_in, color_out, color_hot1, color_hot2, color_hot3, fontin_folder, fontin_name, fontout_folder, fontout_name, invert);
}

ComboNumberDrawer::ComboNumberDrawer(OSL_COLOR color_normal_in, OSL_COLOR color_out, OSL_COLOR color_hot1, OSL_COLOR color_hot2, OSL_COLOR color_hot3, string fontin_folder, string fontin_name, string fontout_folder, string fontout_name, bool invert)
{
	init(color_normal_in, color_out, color_hot1, color_hot2, color_hot3, fontin_folder, fontin_name, fontout_folder, fontout_name, invert);
}

ComboNumberDrawer::~ComboNumberDrawer()
{
	oslDeleteImage(m_img_combo_in);
	oslDeleteImage(m_img_combo_out);
	for(int i=0;i<10;i++)
	{
		oslDeleteImage(m_img_numbers_in[i]);
		oslDeleteImage(m_img_numbers_out[i]);
	}
	delete[] m_img_numbers_in;
	delete[] m_img_numbers_out;
		
	delete m_colorChangeDelayer;
	delete m_fx_tint_in;
	delete m_fx_tint_out;
	
	delete m_fx_appearWait;
	delete m_fx_waitVanish;
}

void ComboNumberDrawer::reset()
{
	m_currentComboNumber=0;
	
	m_img_num1_in=NULL;
	m_img_num2_in=NULL;
	m_img_num3_in=NULL;
	m_img_num1_out=NULL;
	m_img_num2_out=NULL;
	m_img_num3_out=NULL;
	
	m_fx_appearWait->stop();
	m_fx_appearWait->reset();
	
	m_fx_waitVanish->stop();
	m_fx_waitVanish->reset();
}

void ComboNumberDrawer::sync(ComboCounter *cc)
{
	int nb=cc->getCombos(); //nombre d'explosions successives.
	if(m_currentComboNumber!=nb)
	{
		m_currentComboNumber=nb;
		
		if(nb<=1)
		{
			/*m_img_num1=NULL;
			m_img_num2=NULL;
			
			m_fx_appearWait->stop();
			m_fx_appearWait->reset();
			m_fx_waitVanish->stop();
			m_fx_waitVanish->reset();		*/
		}
		else
		{
			if(cc->isHot())
			{
				//enableDebug();
				//debug("sync: hot!");
				m_fx_tint_in->setColorFrom(m_colors_hot[m_colornum]);
				m_fx_tint_in->setColorTo(m_colors_hot[m_colornum]);
				//enableDebug(false);
			}
			else
			{
				m_fx_tint_in->setColorFrom(m_color_normal_in);
				m_fx_tint_in->setColorTo(m_color_normal_in);
			}
			
			int nbCombos=nb-1; //car nb=nombre d'explosions successives
			
			int units=nbCombos%10;
			int tens=int(nbCombos/10.0f)%10;
			int hundreds=int(nbCombos/100.0f);
			
			m_img_num1_in=NULL;
			m_img_num1_out=NULL;
			m_img_num2_in=NULL;
			m_img_num2_out=NULL;
			m_img_num3_in=NULL;
			m_img_num3_out=NULL;
			
			if(hundreds>0)
			{
				m_img_num1_in=m_img_numbers_in[hundreds];
				m_img_num1_out=m_img_numbers_out[hundreds];
			}
			if(tens>0 || hundreds>0)
			{
				m_img_num2_in=m_img_numbers_in[tens];
				m_img_num2_out=m_img_numbers_out[tens];
			}
			m_img_num3_in=m_img_numbers_in[units];
			m_img_num3_out=m_img_numbers_out[units];
			
			m_fx_appearWait->restart();
			m_fx_waitVanish->restart();
		}
	}
}

int ComboNumberDrawer::getComboNum()
{
	//TODO: c'est juste?
	return m_currentComboNumber;
}

bool ComboNumberDrawer::isInSync(ComboCounter *cc)
{
	int nb=cc->getCombos();
	return m_currentComboNumber!=nb;
}

void ComboNumberDrawer::update()
{
	if(m_fx_appearWait->isRunning())
	{
		m_fx_appearWait->update();
		m_fx_waitVanish->update();
		
		if(m_currentComboNumber>GAMEDEF_COMBO_IS_HOT)
		{
			m_colorChangeDelayer->update();
			if(m_colorChangeDelayer->get())
			{
				m_colornum++;
				m_colornum=m_colornum%3;
				
				m_fx_tint_in->setColorFrom(m_colors_hot[m_colornum]);
				m_fx_tint_in->setColorTo(m_colors_hot[m_colornum]);
			}
		}
	}
}

//Positionable
void ComboNumberDrawer::setX(int x)
{
	m_x=x;
}

void ComboNumberDrawer::setY(int y)
{
	m_y=y;
}

int ComboNumberDrawer::getX()
{
	return m_x;
}

int ComboNumberDrawer::getY()
{
	return m_y;
}

//Drawable
//y centre du texte total.
//x en haut
void ComboNumberDrawer::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	if(m_fx_appearWait->isRunning())
	{
		int width=oslGetImageWidth(m_img_combo_out);
		if(m_img_num1_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num1_out);
		}
		if(m_img_num2_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num2_out);
		}
		if(m_img_num3_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num3_out);
		}
		
		width=int(width/2.0f);
				
		//out
		AlphaParamsState alpha;
		alpha.saveState();
		
		m_fx_tint_out->activate();
		m_fx_waitVanish->activate();
		
		int px=m_x-width;
		if(m_img_num1_out!=NULL)
		{		
			fx::drawXY(m_img_num1_out, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
			px+=oslGetImageWidth(m_img_num1_out);
		}
		if(m_img_num2_out!=NULL)
		{		
			fx::drawXY(m_img_num2_out, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
			px+=oslGetImageWidth(m_img_num2_out);
		}
		fx::drawXY(m_img_num3_out, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
		px+=oslGetImageWidth(m_img_num3_out);
		
		fx::drawXY(m_img_combo_out, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
		
		alpha.restoreState();
		
		
		//in
		m_fx_tint_in->activate();
		m_fx_waitVanish->activate();
		
		px=m_x-width;
		if(m_img_num1_in!=NULL)
		{		
			fx::drawXY(m_img_num1_in, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
			px+=oslGetImageWidth(m_img_num1_in);
		}
		if(m_img_num2_in!=NULL)
		{		
			fx::drawXY(m_img_num2_in, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
			px+=oslGetImageWidth(m_img_num2_in);
		}
		fx::drawXY(m_img_num3_in, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
		px+=oslGetImageWidth(m_img_num3_in);
		
		fx::drawXY(m_img_combo_in, px, m_y, m_fx_appearWait, fx1, fx2, fx3, fx4);
		
		alpha.restoreState();
	}
}


void ComboNumberDrawer::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	int oldx=m_x;
	int oldy=m_y;
	m_x=x;
	m_y=y;
	draw(fx1, fx2, fx3, fx4, fx5);
	m_x=oldx;
	m_y=oldy;
}
		
State *ComboNumberDrawer::savestate(State * state)
{
	state->m_x=m_x;
	state->m_y=m_y;
	
	return state;
}

void ComboNumberDrawer::restorestate(State * state)
{
	m_x=state->m_x;
	m_y=state->m_y;
}







