#include "code/game/combos/pdr4c/PDR4CComboNumberDrawer.h"
#include <oslib/oslib.h>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/utils.h"
#include "code/framework/fx/EffectsLib.h"
#include <string>
#include "code/game/gamedef.h"

using namespace std;
using namespace utils;

#define COMBO_NUMBER_APPEARDOWN_TIME 10
#define COMBO_NUMBER_WAIT_TIME 55
#define COMBO_NUMBER_VANISH_TIME 30
#define COMBO_NUMBER_COLORCHANGE_DELAY 10
#define COMBO_NUMBER_TRANSPARENCY 255

PDR4CComboNumberDrawer::PDR4CComboNumberDrawer(OSL_COLOR color_normal_in, OSL_COLOR color_out, OSL_COLOR color_hot1, OSL_COLOR color_hot2, OSL_COLOR color_hot3, bool invert)
{
	//BmpFont::initBmpFont();
	BmpFont * font_in=new BmpFont("res/fonts/bmpFonts/unlearned23/inout/", invert?"out":"in", OSL_PF_4444);
	BmpFont * font_out=new BmpFont("res/fonts/bmpFonts/unlearned23/inout/", invert?"in":"out", OSL_PF_4444);
	
	
	
	BmpText *txtcombo_in=new BmpText("x combo", font_in); //TODO: internationaliser ca
	txtcombo_in->compile();
	BmpText *txtcombo_out=new BmpText("x combo", font_out); //TODO: internationaliser ca
	txtcombo_out->compile();
	m_img_combo_in=txtcombo_in->createSwizzledImageCopy();
	m_img_combo_out=txtcombo_out->createSwizzledImageCopy();
	delete txtcombo_in;
	delete txtcombo_out;
	
	
	m_img_numbers_in=new OSL_IMAGE *[10]; //de 0 a 9
	m_img_numbers_out=new OSL_IMAGE *[10]; //de 0 a 9
	for(int i=0;i<10;i++)
	{
		BmpText *txt_in=new BmpText(toString(i), font_in);
		txt_in->compile();
		m_img_numbers_in[i]=txt_in->createSwizzledImageCopy();
		delete txt_in;
		BmpText *txt_out=new BmpText(toString(i), font_out);
		txt_out->compile();
		m_img_numbers_out[i]=txt_out->createSwizzledImageCopy();
		delete txt_out;
		
	}
	
	delete font_in;
	delete font_out;
	//BmpFont::freeRes();
	
	m_img_num1_in=NULL;
	m_img_num2_in=NULL;
	m_img_num3_in=NULL;
	m_img_num1_out=NULL;
	m_img_num2_out=NULL;
	m_img_num3_out=NULL;
	
	m_fx_appearWaitVanish=new SerialLocalEffect(1);
	m_fx_appearWaitVanish->addEffect(new AppearDownEffect(COMBO_NUMBER_APPEARDOWN_TIME));
	m_fx_appearWaitVanish->addEffect(new WaitLocalEffect(COMBO_NUMBER_WAIT_TIME));
	m_fx_appearWaitVanish->addEffect(new VanishUpEffect(COMBO_NUMBER_VANISH_TIME));
	
	
	
	m_colors_hot[0]=color_hot1; //RGBA(255,0,0,COMBO_NUMBER_TRANSPARENCY);
	m_colors_hot[1]=color_hot2; //RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	m_colors_hot[2]=color_hot3; //RGBA(255,255,0,COMBO_NUMBER_TRANSPARENCY);
	
	m_color_normal_in=color_normal_in;//RGBA(0,0,255,COMBO_NUMBER_TRANSPARENCY);
	m_color_out=color_out;
	
	m_colornum=0;
	
	m_colorChangeDelayer=new Delayer(COMBO_NUMBER_COLORCHANGE_DELAY);
	
	m_currentComboNumber=0;
	
	m_fx_tint_in=new ProgressiveTintEffect(1, 1, m_color_normal_in, m_color_normal_in); //on l'updatera pas
	m_fx_tint_out=new ProgressiveTintEffect(1, 1, m_color_out, m_color_out); //on l'updatera pas
}

PDR4CComboNumberDrawer::~PDR4CComboNumberDrawer()
{
	oslDeleteImage(m_img_combo_in);
	oslDeleteImage(m_img_combo_out);
	for(int i=0;i<10;i++)
	{
		oslDeleteImage(m_img_numbers_in[i]);
		oslDeleteImage(m_img_numbers_out[i]);
	}
	delete[] m_img_numbers_in;
	delete[] m_img_numbers_out;
		
	delete m_fx_appearWaitVanish;
	delete m_colorChangeDelayer;
	delete m_fx_tint_in;
	delete m_fx_tint_out;
}

void PDR4CComboNumberDrawer::reset()
{
	m_currentComboNumber=0;
	
	m_img_num1_in=NULL;
	m_img_num2_in=NULL;
	m_img_num3_in=NULL;
	m_img_num1_out=NULL;
	m_img_num2_out=NULL;
	m_img_num3_out=NULL;
	
	m_fx_appearWaitVanish->stop();
	m_fx_appearWaitVanish->reset();
}

void PDR4CComboNumberDrawer::sync(ComboCounter *cc)
{
	int nb=cc->getCombos(); //nombre d'explosions successives.
	if(m_currentComboNumber!=nb)
	{
		m_currentComboNumber=nb;
		
		if(nb<=1)
		{
			/*m_img_num1=NULL;
			m_img_num2=NULL;
			
			m_fx_appearWait->stop();
			m_fx_appearWait->reset();
			m_fx_waitVanish->stop();
			m_fx_waitVanish->reset();		*/
		}
		else
		{
			if(cc->isHot())
			{
				//enableDebug();
				//debug("sync: hot!");
				m_fx_tint_in->setColorFrom(m_colors_hot[m_colornum]);
				m_fx_tint_in->setColorTo(m_colors_hot[m_colornum]);
				//enableDebug(false);
			}
			else
			{
				m_fx_tint_in->setColorFrom(m_color_normal_in);
				m_fx_tint_in->setColorTo(m_color_normal_in);
			}
			
			int nbCombos=nb-1; //car nb=nombre d'explosions successives
			int units=nbCombos%10;
			int tens=int(nbCombos/10.0f)%10;
			int hundreds=int(nbCombos/100.0f);
			
			m_img_num1_in=NULL;
			m_img_num1_out=NULL;
			m_img_num2_in=NULL;
			m_img_num2_out=NULL;
			m_img_num3_in=NULL;
			m_img_num3_out=NULL;
			if(hundreds>0)
			{
				m_img_num1_in=m_img_numbers_in[hundreds];
				m_img_num1_out=m_img_numbers_out[hundreds];				
			}
			if(tens>0)
			{
				m_img_num2_in=m_img_numbers_in[tens];
				m_img_num2_out=m_img_numbers_out[tens];
			}
			m_img_num3_in=m_img_numbers_in[units];
			m_img_num3_out=m_img_numbers_out[units];
			
			m_fx_appearWaitVanish->restart();
		}
	}
}

void PDR4CComboNumberDrawer::drawXY(int x, int y)
{
	if(m_fx_appearWaitVanish->isRunning())
	{
		//calcul de la largeur
		int width=m_img_combo_out->sizeX;
		if(m_img_num1_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num1_out);
		}
		if(m_img_num2_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num2_out);
		}
		if(m_img_num3_out!=NULL)
		{
			width+=oslGetImageWidth(m_img_num3_out);
		}
		
		int startx=x-int(width/2.0f);
		
				
		
		//out
		AlphaParamsState alpha;
		alpha.saveState();
				
		m_fx_tint_out->activate();
		
		int px=startx;
		
		if(m_img_num1_out!=NULL)
		{		
			fx::drawXY(m_img_num1_out, px, y, m_fx_appearWaitVanish);
			px+=oslGetImageWidth(m_img_num1_out);
		}
		if(m_img_num2_out!=NULL)
		{		
			fx::drawXY(m_img_num2_out, px, y, m_fx_appearWaitVanish);
			px+=oslGetImageWidth(m_img_num2_out);
		}
		fx::drawXY(m_img_num3_out, px, y, m_fx_appearWaitVanish);
		px+=oslGetImageWidth(m_img_num3_out);
				
		fx::drawXY(m_img_combo_out, px, y, m_fx_appearWaitVanish);
		
		alpha.restoreState();
		
		//in
		m_fx_tint_in->activate();
		
		px=startx;
		if(m_img_num1_in!=NULL)
		{		
			fx::drawXY(m_img_num1_in, px, y, m_fx_appearWaitVanish);
			px+=oslGetImageWidth(m_img_num1_in);
		}
		if(m_img_num2_in!=NULL)
		{		
			fx::drawXY(m_img_num2_in, px, y, m_fx_appearWaitVanish);
			px+=oslGetImageWidth(m_img_num2_in);
		}
		fx::drawXY(m_img_num3_in, px, y, m_fx_appearWaitVanish);
		px+=oslGetImageWidth(m_img_num3_in);
		
		fx::drawXY(m_img_combo_in, px, y, m_fx_appearWaitVanish);
		
		alpha.restoreState();
	}
}

void PDR4CComboNumberDrawer::update()
{
	if(m_fx_appearWaitVanish->isRunning())
	{
		m_fx_appearWaitVanish->update();
		
		if(m_currentComboNumber>GAMEDEF_COMBO_IS_HOT)
		{
			m_colorChangeDelayer->update();
			if(m_colorChangeDelayer->get())
			{
				//enableDebug();
				//debug("update: delayer get");
				m_colornum++;
				m_colornum=m_colornum%3;
				
				m_fx_tint_in->setColorFrom(m_colors_hot[m_colornum]);
				m_fx_tint_in->setColorTo(m_colors_hot[m_colornum]);
				//enableDebug(false);
			}
		}
	}
}










