#ifndef GAMEGRID_H_
#define GAMEGRID_H_

#include "code/game/grid/MatrixContentObject.h"
#include <oslib/oslib.h>
#include <string>
#include "code/game/PositionsBuffer.h"
#include "code/framework/Pos.h"
#include "code/framework/Matrix.h"
#include "code/game/FlagMatrix.h"
#include <list>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/grid/GridEventResult.h"


using namespace std;

class GameGrid : public Matrix<MatrixContentObject>
{
	private:
		int m_posX;
		int m_posY;
		int m_cellWidth;
		int m_cellHeight;
		PositionsBuffer * m_positions;
		
		FlagMatrix *findExplosions(Pos p, int mode, bool withshielded, FlagMatrix *flags=NULL, int target=-1); //mode: MODE_COLOR ou MODE_SHAPE
	 	bool pieceIsRightType(int type, bool withshielded); //piece normal et autorise piece shielded si bool=true
	 	int explodeAll(FlagMatrix *flags, int mode);
	 	
	public:
		GameGrid(int cols, int lines, int cellWidth, int cellHeight);
		GameGrid(int cols, int lines, int cellWidth, int cellHeight, int posX, int posY);
		~GameGrid();
		int getPosX();
		int getPosY();
		int getCellWidth();
		int getCellHeight();
		int getWidth();
		int getHeight();
		void update();
		Pos getContentPosition(int col, int line); //ne pas deleter ensuite
		bool cellIsEmpty(int col, int line); //surcharge 
		void fill(); //rempli avec des MatrixContentObject sans piece
		
	 	int explodeAll(int mode);
		
		GridEventResult startExplosions(Pos p, FlagMatrix *pack=NULL); 	//pack sert à savoir quel "paquet" vient de tomber
																//car les explosions doivent avoir lieu entre le "paquet"
																//et le sol, et pas uniquement dans l'un ou l'autre
																//si NULL, pas de check 
		GridEventResult startExplosions(FlagMatrix *pack); 
		
		//enlève les pièces terminées
		void clean(); 
		
		//fait tomber les pieces en suspension, renvoie les pieces qui ont atteri (point de depart de combos)
		FlagMatrix *collapse(); 
		
		FlagMatrix *cleanAndCollapse(); //version optimisée 2 en 1
		
		//averti si un evenement est en cours: piece qui degringole, qui explose...
		bool isEvent();
		
		int notifyCombo(); //il y a eu un combo, on diminue les pieces combo de 1, renvoie le nombre de pieces combo présentes
		int explodeComboPieces(); //fait exploser les pieces de type combo qui en ont besoin, true si au - 1 explose
		
		//attention, cette fonction fait des clones des pièces de la liste, elles ne sont donc pas nettoyées
		bool addLine(list<AbstractPiece *> *pline); //false=gameover
		
		//ne fait pas exploser la piece mais l'enlève directement
		void immediatePieceDestruction(int x, int y);
		
		void reset();
};

#endif /*GAMEGRID_H_*/


















