#ifndef MATRIXCONTENTOBJECT_H_
#define MATRIXCONTENTOBJECT_H_

#include <string>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/game/grid/AbstractMCOContent.h"
//#include "code/framework/check/Trackable.h"

using namespace std;

#define MCO_STATE_FALLING 1
#define MCO_STATE_FIXED 2

class MatrixContentObject //: public Trackable
{
	private:
		AbstractPiece *m_piece;
		
		//flags
		bool m_colorExploding; //explose a cause de la couleur
		bool m_shapeExploding; //explose a cause de la forme
		bool m_specialExploding; //explose pour une autre raison (pieces speciales)
		
		int m_state; //etat de la case: falling/fixed
		TranslationEffect * m_fx_fall;
		
		AbstractMCOContent *m_mcoContent;
	
	public:
		MatrixContentObject(AbstractPiece *p);
		~MatrixContentObject();
		
		MatrixContentObject(const MatrixContentObject &mco);//copie
   		MatrixContentObject& operator =(const MatrixContentObject& Other);
   		
   		AbstractPiece *getPiece();
   		
   		bool isColorExploding();
   		bool isShapeExploding();
   		bool isSpecialExploding();
   		bool isExploding(int mode=PIECE_MODE_ANY);
   		void setColorExploding(bool b=true);
   		void setShapeExploding(bool b=true);
   		void setSpecialExploding(bool b=true);
   		void startExplosion(int mode=PIECE_MODE_SPECIAL);
   		void setState(int state);
   		int getState();
   		void update();
   		bool tryCleaning(bool force=false); //detruit la piece si elle a fini d'exploser, renvoie true si elle l'a fait
   		void startFalling(); //commence a tomber ou continue si elle etait en train
   		void restartFalling(); //commence a tomber et recommence si elle etait en train (translation remise a 0)
   		void dontFall(); //la laisse fixe
   		AbstractPiece *detachPiece();
   		void setPiece(AbstractPiece *p);
   		bool isFinishedFalling();
   		bool isFalling();
   		bool isEmpty(); //est ce que la piece est NULL?
   		
   		void setMCOContent(AbstractMCOContent *cont);
   		AbstractMCOContent *getMCOContent();
   		void deleteMCOContent();
};

#endif /*MATRIXCONTENTOBJECT_H_*/
