#ifndef ABSTRACTLEVEL_H_
#define ABSTRACTLEVEL_H_
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/combos/ComboCounter.h"
#include <oslib/oslib.h>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/grid/GameGrid.h"
#include "code/game/gamedef.h"
#include "code/game/score/Scorer.h"
#include <string>

using namespace std;


class AbstractLevel
{
	private:
		
		
	protected:
		int m_initialDropSpeed;
		int m_initialUpSpeed; //vitesse ou montent les lignes, -1 = jamais.
		//int m_frameCounter;
		int m_difficulty;
		int m_nbPlayers;
		
		int m_nbLines; //nombre de lignes dans la grille
		int m_nbCols; //nombre de colonnes dans la grille
		
		bool m_paused;
		virtual void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)=0;
		virtual void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
		{
//			enableDebug();
//			debug("AbstractLevel::update_pause_specific, should not happen if child overloads this function");
//			enableDebug(false);
		}
	
	public:
		virtual void start_specific()=0;
		virtual void pause_specific()=0;
		virtual void resume_specific()=0;
		virtual void reset(){};
		
		//AbstractLevel(int nbLines, int nbCols, int initialDropSpeed, int initialUpSpeed=-1, int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		AbstractLevel(int initialDropSpeed, int initialUpSpeed=-1, int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		virtual ~AbstractLevel(){}
		virtual string getCode()=0;
		int getInitialDropSpeed();
		int getInitialUpSpeed();
		int getDifficulty();
		int getNbPlayers();
		
		int getNbLines();
		int getNbCols();
		
		virtual int getDropSpeed()=0; //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		virtual int getUpSpeed()=0; //la vitesse ou les lignes montent a un instant t
		virtual int getFastDropSpeed(); //la vitesse qd on appuie sur bas (peut etre différente suivant les niveaux)
		virtual int getPieceUnconnectedDropSpeed();
		virtual int getMinimumWaitBeforeNewPair();
		virtual void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)=0;
		virtual void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)=0;
		virtual void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)=0;
		void start(); //démarre la musique, le timer etc
		
		//synchronise le cc avec son dessin pendant l'update, 
		//peut modifier la grille & la paire suivante
		void update(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans=NULL); 
		
		void pause(); //pause, pour par exemple utiliser spliff politics
		bool isPaused();
		void resume(); //remet en route
		virtual float getDuration()=0;
		
		virtual void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)=0;
		
		//virtual LinePattern *getLevelPieces()=0; 
		virtual bool getAddLineEvent()=0; //dit quand une ligne doit monter, dû au scénario
		
		virtual bool finished()=0; //averti si le niveau est terminé, et c'est gagné alors.
		
		virtual list<AbstractPiece *> *getNextLine()=0;
		
		virtual Pair *getNewPair();
		
		virtual bool isGameOver();
		
		virtual int getNbSpecialExplosions(){return 0;}
		
};

#endif /*ABSTRACTLEVEL_H_*/
