#include "code/game/levels/impl/djbeat/PDR4CLevelDjbeat.h"
#include "code/game/levels/utils/MCOContentSimpleAnimated.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/Properties.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/framework/anim/Anim.h"
#include <math.h>
#include "code/framework/check/loaders.h"

using namespace utils;

#define PDR4C_LEVEL_DJBEAT_COLOR_RED RGB(255,0,0)
#define PDR4C_LEVEL_DJBEAT_COLOR_GREEN RGB(0,255,0)
#define PDR4C_LEVEL_DJBEAT_COLOR_BLUE RGB(0,0,255)
#define PDR4C_LEVEL_DJBEAT_COLOR_BLACK RGB(0,0,0)
#define PDR4C_LEVEL_DJBEAT_DURATION 182000.0f

string PDR4CLevelDjbeat::getCode()
{
	return "pdr4cdjbeat";
}

PDR4CLevelDjbeat::PDR4CLevelDjbeat(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{	
	Properties *props=new Properties("res/pdr4c/levels/djbeat/djbeat.properties");
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/15/nosmooth/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	
	m_comboDrawer=new PDR4CComboNumberDrawer(PDR4C_LEVEL_DJBEAT_COLOR_BLUE, PDR4C_LEVEL_DJBEAT_COLOR_BLACK, PDR4C_LEVEL_DJBEAT_COLOR_RED, PDR4C_LEVEL_DJBEAT_COLOR_GREEN, PDR4C_LEVEL_DJBEAT_COLOR_BLUE);
	
	m_scenario=new EventsScenario("res/pdr4c/levels/djbeat/djbeat.scenario");
	
	m_music=fwkLoadSoundFileMP3("res/pdr4c/levels/djbeat/djbeat.mp3", OSL_FMT_STREAM);

	m_addline=6;
	
	m_anim_pause=new Anim(2, "pause", "res/pdr4c/levels/djbeat/pause/", 2, OSL_PF_8888);
	m_anim_pause->setReferencePointCenter();
	m_anim_pause->start();
	
	m_anims_next = new Anim*[6]; 
	m_anims_gauge = new Anim*[6];
	
	for(int i=1; i<=6; i++)
	{
		m_anims_next[i-1]=new Anim(2, "next"+toString(i)+"_", "res/pdr4c/levels/djbeat/next/", 2, OSL_PF_8888);
		m_anims_next[i-1]->setReferencePoint(0, 0);
		m_anims_next[i-1]->start();
		
		m_anims_gauge[i-1]=new Anim(2, "combo"+toString(i)+"_", "res/pdr4c/levels/djbeat/combo/", 2, OSL_PF_8888);
		m_anims_gauge[i-1]->setReferencePoint(0, 252);
		m_anims_gauge[i-1]->start();
	}
	m_currentAnimNum=0;
	m_delayer=NULL;
	m_delayer_time=props->getIntValue("delayer.time");
	m_delayer_loops=props->getIntValue("delayer.loops");
	m_delayer2=NULL;
	
	m_fx_gauge_vanish=new VanishUpEffect(GAMEDEF_COMBO_TIME);
	m_fx_gauge_move=new TranslationEffect(0, 0, 0, 252, GAMEDEF_COMBO_TIME, GAMEDEF_COMBO_TIME, true, 1);
	
	m_difficulty=difficulty;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_nbColors=props->getIntValue("easy.colors");
		m_dropspeed_slow=props->getIntValue("easy.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("easy.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("easy.upspeed.slow");
		m_upspeed_fast=props->getIntValue("easy.upspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/djbeat/djbeat_easy.pattern");
		m_nbCols=12;
		m_anim_grid=new Anim(2, "grid12_", "res/pdr4c/levels/djbeat/grid/g12/", 2, OSL_PF_8888);
		m_anim_grid->start();
		m_anim_grid->setX(113);
		m_anim_grid->setY(9);
		m_pos_next=new Pos(props->getIntValue("easy.next.x"), props->getIntValue("easy.next.y"));
		m_pos_gauge=new Pos(props->getIntValue("easy.gauge.x"), props->getIntValue("easy.gauge.y"));
		
	}
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_nbColors=props->getIntValue("normal.colors");
		m_dropspeed_slow=props->getIntValue("normal.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("normal.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("normal.upspeed.slow");
		m_upspeed_fast=props->getIntValue("normal.upspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/djbeat/djbeat_normal.pattern");
		m_nbCols=13;
		m_anim_grid=new Anim(2, "grid13_", "res/pdr4c/levels/djbeat/grid/g13/", 2, OSL_PF_8888);
		m_anim_grid->start();
		m_anim_grid->setX(102);
		m_anim_grid->setY(9);
		m_pos_next=new Pos(props->getIntValue("normal.next.x"), props->getIntValue("normal.next.y"));
		m_pos_gauge=new Pos(props->getIntValue("normal.gauge.x"), props->getIntValue("normal.gauge.y"));
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_nbColors=props->getIntValue("hard.colors");
		m_dropspeed_slow=props->getIntValue("hard.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("hard.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("hard.upspeed.slow");
		m_upspeed_fast=props->getIntValue("hard.upspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/djbeat/djbeat_hard.pattern");
		m_nbCols=14;
		m_anim_grid=new Anim(2, "grid14_", "res/pdr4c/levels/djbeat/grid/g14/", 2, OSL_PF_8888);
		m_anim_grid->start();
		m_anim_grid->setX(92);
		m_anim_grid->setY(9);
		m_pos_next=new Pos(props->getIntValue("hard.next.x"), props->getIntValue("hard.next.y"));
		m_pos_gauge=new Pos(props->getIntValue("hard.gauge.x"), props->getIntValue("hard.gauge.y"));
	}
	m_dropspeed=props->getIntValue("dropspeed.start");
	m_upspeed=-1;
		
	//anims
	m_anim_overlays=new Anim *[6];//les pieces quand elles n'explosent pas
	m_anim_bgs=new Anim *[6];//les pieces quand elles n'explosent pas
	m_anim_explosions=new Anim *[6];//les pieces quand elles explosent	
	initAnimsPieces(props);
	initAnimsCorners();
	
	m_beginning=true;
	
	m_anim_djbeat=new Anim(2, "djbeat", "res/pdr4c/levels/djbeat/djbeat/", 2, OSL_PF_8888);
	m_anim_djbeat->start();
	m_anim_djbeat->setReferencePointCenter();
	m_anim_djbeat->setX(240); //480/2
	m_anim_djbeat->setY(136); //272/2
	
	m_img_circle_blue_small=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_blue_small.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_blue_small);
	
	m_img_circle_blue_big=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_blue_big.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_blue_big);
	
	m_img_circle_red_small=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_red_small.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_red_small);
	
	m_img_circle_red_big=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_red_big.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_red_big);
	
	m_img_circle_green_small=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_green_small.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_green_small);
	
	m_img_circle_green_big=fwkLoadImageFilePNG("res/pdr4c/levels/djbeat/circles/circle_green_big.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_circle_green_big);
	
	m_speedFast=props->getFloatValue("circles.speed.fast");
	m_speedSlow=props->getFloatValue("circles.speed.slow");
	
	m_fx_circle_blue=nextTranslationEffect();
	m_fx_circle_red=nextTranslationEffect();
	m_fx_circle_green=nextTranslationEffect();
	m_fx_circle_blue->start();
	m_fx_circle_red->start();
	m_fx_circle_green->start();
	
	m_frame_counter=0;
	m_frame_even=true;
	
	m_translation_fast=false;
	m_circles_big=false;
	
	delete props;
}

void PDR4CLevelDjbeat::reset()
{
	Properties *props=new Properties("res/pdr4c/levels/djbeat/djbeat.properties");
	
	m_songbanner->reset();
	m_comboDrawer->reset();
	m_scenario->reset();
	oslStopSound(m_music);
	m_linepattern->reset();
	
	m_addline=6;
	
	/*if(m_delayer!=NULL)
	{
		delete m_delayer;
		m_delayer=NULL
	if(m_delayer2!=NULL)
		delete m_delayer2;
	*/
	m_dropspeed=props->getIntValue("dropspeed.start");
	m_upspeed=-1;
	
	m_beginning=true;
	
	m_translation_fast=false;
	m_circles_big=false;
	
	delete props;
}

PDR4CLevelDjbeat::~PDR4CLevelDjbeat()
{
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	
	oslDeleteSound(m_music);
	
	
	delete m_fx_gauge_move;
	delete m_fx_gauge_vanish;
	
	
	for(int i=0;i<6;i++)
	{
		delete m_anim_bgs[i];
		delete m_anim_overlays[i];
		delete m_anim_explosions[i];
		
		delete m_anims_next[i];
		delete m_anims_gauge[i];
		
		delete m_anims_corner_downleft[i];
		delete m_anims_corner_downright[i];
		delete m_anims_corner_upleft[i];
		delete m_anims_corner_upright[i];
	}
	
	delete[] m_anim_overlays;
	delete[] m_anim_bgs;
	delete[] m_anim_explosions;
	
	delete[] m_anims_next;
	delete[] m_anims_gauge;
	
	delete[] m_anims_corner_downleft;
	delete[] m_anims_corner_downright;
	delete[] m_anims_corner_upleft;
	delete[] m_anims_corner_upright;
	
	delete m_pos_gauge;
	delete m_pos_next;
	
	if(m_delayer!=NULL)
		delete m_delayer;
	if(m_delayer2!=NULL)
		delete m_delayer2;
	
	delete m_anim_djbeat;
		
	oslDeleteImage(m_img_circle_blue_small);
	oslDeleteImage(m_img_circle_blue_big);
	oslDeleteImage(m_img_circle_red_small);
	oslDeleteImage(m_img_circle_red_big);
	oslDeleteImage(m_img_circle_green_small);
	oslDeleteImage(m_img_circle_green_big);
	
	delete m_fx_circle_blue;
	delete m_fx_circle_green;
	delete m_fx_circle_red;
	
	delete m_anim_grid;
	delete m_anim_pause;
	
	//enableDebug();
	//debug("nb images in anim buffer: "+toString(Anim::getNbBufferedImages()));
	//enableDebug(false);
	
}


//dessin avec une paire
void PDR4CLevelDjbeat::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawPairXY(currentPair, pixPos.x, pixPos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void PDR4CLevelDjbeat::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	drawPieceXY(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceXY(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void PDR4CLevelDjbeat::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void PDR4CLevelDjbeat::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	/*
	if(m_even)
		//oslClearScreen(PDR4C_LEVEL_DJBEAT_COLOR_GREEN);
		oslDrawImageXY(m_img_bg1, 0, 0);
	else
		//oslClearScreen(PDR4C_LEVEL_DJBEAT_COLOR_RED);
		oslDrawImageXY(m_img_bg2, 0, 0);
	*/
	
	//oslClearScreen(RGB(255,0,255));
	oslClearScreen(PDR4C_LEVEL_DJBEAT_COLOR_BLACK);
	
	
	
	//m_anim_grid->draw();
	
//	if(test==0)
//		oslDrawImageXY(m_test_img_grid1, 113, 10);
//	else
//		oslDrawImageXY(m_test_img_grid2, 113, 10);
	
	
	if(!m_beginning)
	{
		drawCircles();
		if(!isPaused())
			m_anim_grid->draw();
	}
			
	if(isPaused())
	{
		
	}
	else 
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL)
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->drawXY(pos.x,pos.y);
						}
					}
					else
					{
						p->drawXY(pos.x,pos.y);
					}
				}
			}	
		}
		
	}
	
}

void PDR4CLevelDjbeat::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//coins
	int spacement=5;
	fx::drawXY(m_anims_corner_upleft[m_currentAnimNum]->getImage(), spacement,spacement);
	fx::drawXY(m_anims_corner_downleft[m_currentAnimNum]->getImage(), spacement, 272-spacement);
	fx::drawXY(m_anims_corner_downright[m_currentAnimNum]->getImage(), 480-spacement, 272-spacement);
	fx::drawXY(m_anims_corner_upright[m_currentAnimNum]->getImage(), 480-spacement, spacement);
	
	if(m_beginning)
	{
		m_anim_grid->draw();
		drawCircles();
		if(!isPaused())
		{
			m_anim_djbeat->draw();
		}
	}
	else
	{
		//barre de combo
		if(m_fx_gauge_vanish->isRunning())
		{
			fx::drawXY(m_anims_gauge[m_currentAnimNum]->getImage(), m_pos_gauge->x, m_pos_gauge->y, m_fx_gauge_move, m_fx_gauge_vanish);
			
		}
	
	
		
		//next
		fx::drawXY(m_anims_next[m_currentAnimNum]->getImage(), m_pos_next->x, m_pos_next->y);
		if(!isPaused())
			drawPairXY(nextPair, m_pos_next->x+10, m_pos_next->y+40, grid);	
		
		m_comboDrawer->drawXY(240, 180);		
	}
	
	if(isPaused())
	{
		m_anim_pause->drawXY(240,136);
	}
	
	
	
	if(m_frame_counter<10)
	{
		oslSetAlpha(OSL_FX_TINT, PDR4C_LEVEL_DJBEAT_COLOR_GREEN);
	}
	else if(m_frame_counter<20)
	{
		oslSetAlpha(OSL_FX_TINT, PDR4C_LEVEL_DJBEAT_COLOR_RED);
	}
	else
	{
		oslSetAlpha(OSL_FX_TINT, PDR4C_LEVEL_DJBEAT_COLOR_BLUE);
	}
	
	m_songbanner->draw();
	oslSetAlpha(OSL_FX_DEFAULT, 0);
		
	 	
}

void PDR4CLevelDjbeat::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

void PDR4CLevelDjbeat::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_anim_pause->update();
}

//appelé uniquement si le level n'est pas en pause.
void PDR4CLevelDjbeat::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	//cas special pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						mco->setMCOContent(mcoContentFromColor(p->getColor()));
					}				
				}
				
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_EXPLODE);
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->update();
			}
		}
	}
	
	if(m_delayer!=NULL)
		m_delayer->update();
	if(m_delayer2!=NULL)
		m_delayer2->update();
	
	m_frame_even=!m_frame_even;
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_gauge_move->update();
	m_fx_gauge_vanish->update();
	m_anims_next[m_currentAnimNum]->update();
	m_anims_gauge[m_currentAnimNum]->update();
	m_anims_corner_upright[m_currentAnimNum]->update();
	m_anims_corner_upleft[m_currentAnimNum]->update();
	m_anim_djbeat->update();
	m_fx_circle_blue->update();
	m_fx_circle_red->update();
	m_fx_circle_green->update();
	//m_anim_pause->update();
	
	if(!m_beginning) //sinon ca clignote a la meme vitesse que les pieces = on voit qu'une couleur des pieces
		m_anim_grid->update();
	
	//les cercles, une fois arrivés on les fait aller ailleurs
	if(m_fx_circle_blue->finished())
	{
		TranslationEffect *newfx=nextTranslationEffect(m_fx_circle_blue);
		delete m_fx_circle_blue;
		m_fx_circle_blue=newfx;
		m_fx_circle_blue->start();
	}
	if(m_fx_circle_red->finished())
	{
		TranslationEffect *newfx=nextTranslationEffect(m_fx_circle_red);
		delete m_fx_circle_red;
		m_fx_circle_red=newfx;
		m_fx_circle_red->start();
	}
	if(m_fx_circle_green->finished())
	{
		TranslationEffect *newfx=nextTranslationEffect(m_fx_circle_green);
		delete m_fx_circle_green;
		m_fx_circle_green=newfx;
		m_fx_circle_green->start();
	}
	
	
	m_frame_counter++;
	if(m_frame_counter>=30)
		m_frame_counter=0;
	
	m_anims_corner_downright[m_currentAnimNum]->update();
	m_anims_corner_downleft[m_currentAnimNum]->update();

	
	//on démarre l'effet de la jauge qd le timer est complet
	if(cc->getTime()>=cc->getTimeTotalAvailable()/*-1*/) //-1 pour etre sur de le chopper
	{
		m_fx_gauge_move->restart();
		m_fx_gauge_vanish->restart();
	}
	if(cc->isEvent())
	{
		m_fx_gauge_move->stop();
		m_fx_gauge_vanish->stop();
	}
	
	for(int i=0;i<6;i++)
	{
		m_anim_overlays[i]->update();
		m_anim_bgs[i]->update();
	}
	

	//evenements du scenario
	if(m_scenario->hasEvent(1)) //debut des tams tams
	{
		m_beginning=false;
	}
	if(m_scenario->hasEvent(2)) //fin des tams tams
	{
		
	}
	if(m_scenario->hasEvent(3)) //debut metal
	{
		
	}
	if(m_scenario->hasEvent(4)) //fin metal
	{
		
	}
	if(m_scenario->hasEvent(5)) //debut instrument seul
	{
		m_dropspeed=m_dropspeed_slow;
		m_upspeed=m_upspeed_slow;
		m_circles_big=false;
	}
	if(m_scenario->hasEvent(6)) //fin instrument seul
	{
		m_dropspeed=m_dropspeed_fast;
		m_upspeed=m_upspeed_fast;
		m_circles_big=true;
	}
	if(m_scenario->hasEvent(7)) //boum final
	{
		
	}
	
	if(m_scenario->hasEvent(8)) //fiou iou iou iou
	{
		if(m_delayer!=NULL)
			delete m_delayer;
		m_delayer=new Delayer(m_delayer_time, m_delayer_loops);
		if(m_delayer2!=NULL)
			delete m_delayer2;
		m_delayer2=new Delayer(m_delayer_time*m_delayer_loops, 1);
		
		m_translation_fast=true;
		changeCirclesSpeed();
		
	}
	if(m_delayer!=NULL && m_delayer->get())
	{
		m_currentAnimNum++;
		if(m_currentAnimNum>=6)
			m_currentAnimNum=0;
	}
	if(m_delayer2!=NULL && m_delayer2->get())
	{
		m_translation_fast=false;
		changeCirclesSpeed();
	}
}

void PDR4CLevelDjbeat::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void PDR4CLevelDjbeat::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}
		


list<AbstractPiece *> *PDR4CLevelDjbeat::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool PDR4CLevelDjbeat::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int PDR4CLevelDjbeat::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	if(m_beginning)
		return -1;
	return m_dropspeed;
}

int PDR4CLevelDjbeat::getFastDropSpeed()
{
	if(m_beginning)
		return -1;
	return AbstractLevel::getFastDropSpeed();
}

int PDR4CLevelDjbeat::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void PDR4CLevelDjbeat::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		
	}
}

bool PDR4CLevelDjbeat::finished()
{
	if(m_scenario->getTime()>=PDR4C_LEVEL_DJBEAT_DURATION)
	{
		return true;
	}
	return false;
}

float PDR4CLevelDjbeat::getDuration()
{
	return PDR4C_LEVEL_DJBEAT_DURATION;
}

Pair *PDR4CLevelDjbeat::getNewPair()
{	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY=-1; 
	p->goTo(posX, posY);
	
	return p;
}



AbstractPiece *PDR4CLevelDjbeat::randomPiece()
{
	//suivant la difficulté, il y a + ou - de couleurs à l'écran
	int color=random(0,m_nbColors-1);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
	}
	
	return p;
}

void PDR4CLevelDjbeat::drawPairXY(Pair *pair, int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
	fx::drawXY(m_anim_bgs[rankFromColor(pair->getPiece(1)->getColor())]->getImage(), x2, y2, fx_transl, fx1);
	fx::drawXY(m_anim_bgs[rankFromColor(pair->getPiece(0)->getColor())]->getImage(), x, y, fx_transl, fx1);
	fx::drawXY(m_anim_overlays[rankFromColor(pair->getPiece(1)->getColor())]->getImage(), x2, y2, fx_transl, fx1);
	fx::drawXY(m_anim_overlays[rankFromColor(pair->getPiece(0)->getColor())]->getImage(), x, y, fx_transl, fx1);
	//fx::drawXY(animPieceFromColor(pair->getPiece(1)->getColor())->getImage(), x2, y2, fx_transl, fx1);
	//fx::drawXY(animPieceFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
	//pair->drawXY(x, y, grid, fx_transl, APPLY_FX_ON_BOTH, fx1);
}

void PDR4CLevelDjbeat::drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl)
{
	//fx::drawXY(animPieceFromColor(piece->getColor())->getImage(), x, y, fx_transl);
	fx::drawXY(m_anim_bgs[rankFromColor(piece->getColor())]->getImage(), x, y, fx_transl);
	fx::drawXY(m_anim_overlays[rankFromColor(piece->getColor())]->getImage(), x, y, fx_transl);
	//piece->drawXY(x, y, fx_transl);
}

MCOContentSimpleAnimated *PDR4CLevelDjbeat::mcoContentFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[0])), new Anim(*(m_anim_overlays[0])), new Anim(*(m_anim_explosions[0])));
			break;
		}
		case PIECE_COLOR_RED:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[1])), new Anim(*(m_anim_overlays[1])), new Anim(*(m_anim_explosions[1])));
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[2])), new Anim(*(m_anim_overlays[2])), new Anim(*(m_anim_explosions[2])));
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[3])), new Anim(*(m_anim_overlays[3])), new Anim(*(m_anim_explosions[3])));
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[4])), new Anim(*(m_anim_overlays[4])), new Anim(*(m_anim_explosions[4])));
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return new MCOContentSimpleAnimated(new Anim(*(m_anim_bgs[5])), new Anim(*(m_anim_overlays[5])), new Anim(*(m_anim_explosions[5])));
			break;
		}
	}
	
	return NULL;
}

int PDR4CLevelDjbeat::rankFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return 0;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return 1;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return 2;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return 3;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return 4;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return 5;
			break;
		}
	}
	
	return -1;
}


void PDR4CLevelDjbeat::initAnimsPieces(Properties *props)
{
	
	
	int delay_total_green=props->getIntValue("block.green.delay");
	int nb_images_green=props->getIntValue("block.green.fillers");
	
	int delay_total_pink=props->getIntValue("block.pink.delay");
	int nb_images_pink=props->getIntValue("block.pink.fillers");
	
	int delay_total_yellow=props->getIntValue("block.yellow.delay");
	int nb_images_yellow=props->getIntValue("block.yellow.fillers");
	
	int delay_total_red=props->getIntValue("block.red.delay");
	int nb_images_red=props->getIntValue("block.red.fillers");
	
	int delay_total_cyan=props->getIntValue("block.cyan.delay");
	int nb_images_cyan=props->getIntValue("block.cyan.fillers");
	
	
	
	//1) pieces normal
	
	//blue
	//m_anim_bgs[0]=new Anim(3, "bg_blue", "res/pdr4c/levels/djbeat/pieces/blue/bg/", 3, OSL_PF_8888);
	m_anim_bgs[0]=new Anim(2, "bg_blue", "res/pdr4c/levels/djbeat/pieces/blue/bg/", 2, OSL_PF_8888);
	m_anim_overlays[0]=new Anim("res/pdr4c/levels/djbeat/pieces/blue/overlay/overlay_blue1.png", OSL_PF_8888);
	
	//red
	m_anim_bgs[1]=new Anim("res/pdr4c/levels/djbeat/pieces/red/bg/bg_red1.png", OSL_PF_8888);
	m_anim_overlays[1]=new Anim("res/pdr4c/levels/djbeat/pieces/red/overlay/overlay_red1.png", delay_total_red, OSL_PF_8888);
	for(int i=0;i<nb_images_red;i++)
	{
		m_anim_overlays[1]->addImage("res/pdr4c/levels/djbeat/pieces/red/overlay/overlay_red1.png", OSL_PF_8888); //ca va utiliser la meme image deja en mémoire
	}
	m_anim_overlays[1]->addImage("res/pdr4c/levels/djbeat/pieces/red/overlay/overlay_red2.png", OSL_PF_8888);
	m_anim_overlays[1]->addImage("res/pdr4c/levels/djbeat/pieces/red/overlay/overlay_red3.png", OSL_PF_8888);
	
	
	//green
	m_anim_bgs[2]=new Anim("res/pdr4c/levels/djbeat/pieces/green/bg/bg_green1.png", OSL_PF_8888);
	m_anim_overlays[2]=new Anim("res/pdr4c/levels/djbeat/pieces/green/overlay/overlay_green1.png", delay_total_green, OSL_PF_8888);
	for(int i=0;i<nb_images_green;i++)
	{
		m_anim_overlays[2]->addImage("res/pdr4c/levels/djbeat/pieces/green/overlay/overlay_green1.png", OSL_PF_8888); //ca va utiliser la meme image deja en mémoire
	}
	m_anim_overlays[2]->addImage("res/pdr4c/levels/djbeat/pieces/green/overlay/overlay_green2.png", OSL_PF_8888);
	m_anim_overlays[2]->addImage("res/pdr4c/levels/djbeat/pieces/green/overlay/overlay_green3.png", OSL_PF_8888);
	
	//yellow
	m_anim_bgs[3]=new Anim(2, "bg_yellow", "res/pdr4c/levels/djbeat/pieces/yellow/bg/", 2, OSL_PF_8888);
	//m_anim_bgs[3]=new Anim("res/pdr4c/levels/djbeat/pieces/yellow/bg/bg_yellow1.png", OSL_PF_8888);
	m_anim_overlays[3]=new Anim("res/pdr4c/levels/djbeat/pieces/yellow/overlay/overlay_yellow1.png", delay_total_yellow, OSL_PF_8888);
	for(int i=0;i<nb_images_yellow;i++)
	{
		m_anim_overlays[3]->addImage("res/pdr4c/levels/djbeat/pieces/yellow/overlay/overlay_yellow1.png", OSL_PF_8888); //ca va utiliser la meme image deja en mémoire
	}
	m_anim_overlays[3]->addImage("res/pdr4c/levels/djbeat/pieces/yellow/overlay/overlay_yellow2.png", OSL_PF_8888);
	m_anim_overlays[3]->addImage("res/pdr4c/levels/djbeat/pieces/yellow/overlay/overlay_yellow3.png", OSL_PF_8888);
	
	//pink
	m_anim_bgs[4]=new Anim(2, "bg_pink", "res/pdr4c/levels/djbeat/pieces/pink/bg/", 2, OSL_PF_8888);
	m_anim_overlays[4]=new Anim("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink1.png", delay_total_pink, OSL_PF_8888);
	for(int i=0;i<nb_images_pink;i++)
	{
		m_anim_overlays[4]->addImage("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink1.png", OSL_PF_8888); //ca va utiliser la meme image deja en mémoire
	}
	m_anim_overlays[4]->addImage("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink2.png", OSL_PF_8888);
	m_anim_overlays[4]->addImage("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink3.png", OSL_PF_8888);
	m_anim_overlays[4]->addImage("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink2.png", OSL_PF_8888);
	m_anim_overlays[4]->addImage("res/pdr4c/levels/djbeat/pieces/pink/overlay/overlay_pink4.png", OSL_PF_8888);
	
	//cyan
	m_anim_bgs[5]=new Anim(2, "bg_cyan", "res/pdr4c/levels/djbeat/pieces/cyan/bg/", 2, OSL_PF_8888);
	m_anim_overlays[5]=new Anim("res/pdr4c/levels/djbeat/pieces/cyan/overlay/overlay_cyan1.png", delay_total_cyan, OSL_PF_8888);
	for(int i=0;i<nb_images_cyan;i++)
	{
		m_anim_overlays[5]->addImage("res/pdr4c/levels/djbeat/pieces/cyan/overlay/overlay_cyan1.png", OSL_PF_8888); //ca va utiliser la meme image deja en mémoire
	}
	m_anim_overlays[5]->addImage("res/pdr4c/levels/djbeat/pieces/cyan/overlay/overlay_cyan2.png", OSL_PF_8888);
	m_anim_overlays[5]->addImage("res/pdr4c/levels/djbeat/pieces/cyan/overlay/overlay_cyan3.png", OSL_PF_8888);
	m_anim_overlays[5]->addImage("res/pdr4c/levels/djbeat/pieces/cyan/overlay/overlay_cyan2.png", OSL_PF_8888);
	
	
	
	//2) pieces explosion
	m_anim_explosions[0]=new Anim(10, "explosion_blue", "res/pdr4c/levels/djbeat/pieces/blue/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_explosions[1]=new Anim(10, "explosion_red", "res/pdr4c/levels/djbeat/pieces/red/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_explosions[2]=new Anim(10, "explosion_green", "res/pdr4c/levels/djbeat/pieces/green/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_explosions[3]=new Anim(10, "explosion_yellow", "res/pdr4c/levels/djbeat/pieces/yellow/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_explosions[4]=new Anim(10, "explosion_pink", "res/pdr4c/levels/djbeat/pieces/pink/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_explosions[5]=new Anim(10, "explosion_cyan", "res/pdr4c/levels/djbeat/pieces/cyan/explosion/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	
	
	
	
	
	//on démarre toutes les anims (elles tournent continuellement)
	for(int i=0;i<6;i++)
	{
		m_anim_overlays[i]->setReferencePointCenter();
		m_anim_bgs[i]->setReferencePointCenter();
		m_anim_explosions[i]->setReferencePointCenter();
		
		m_anim_overlays[i]->start();
		m_anim_bgs[i]->start();
	}
}

void PDR4CLevelDjbeat::initAnimsCorners()
{
	m_anims_corner_upleft = new Anim*[6];
	m_anims_corner_upright = new Anim*[6];
	m_anims_corner_downleft = new Anim*[6];
	m_anims_corner_downright = new Anim*[6];
	
	m_anims_corner_upleft[0]=new Anim(2, "corner_ul_cyan", "res/pdr4c/levels/djbeat/corners/upleft/", 2, OSL_PF_8888);
	m_anims_corner_upleft[1]=new Anim("res/pdr4c/levels/djbeat/corners/upleft/corner_ul_blue.png", OSL_PF_8888);
	m_anims_corner_upleft[2]=new Anim(2, "corner_ul_pink", "res/pdr4c/levels/djbeat/corners/upleft/", 2, OSL_PF_8888);
	m_anims_corner_upleft[3]=new Anim("res/pdr4c/levels/djbeat/corners/upleft/corner_ul_red.png", OSL_PF_8888);
	m_anims_corner_upleft[4]=new Anim(2, "corner_ul_yellow", "res/pdr4c/levels/djbeat/corners/upleft/", 2, OSL_PF_8888);
	m_anims_corner_upleft[5]=new Anim("res/pdr4c/levels/djbeat/corners/upleft/corner_ul_green.png", OSL_PF_8888);
	
	m_anims_corner_downleft[0]=new Anim("res/pdr4c/levels/djbeat/corners/downleft/corner_dl_blue.png", OSL_PF_8888);
	m_anims_corner_downleft[1]=new Anim(2, "corner_dl_pink", "res/pdr4c/levels/djbeat/corners/downleft/", 2, OSL_PF_8888);
	m_anims_corner_downleft[2]=new Anim("res/pdr4c/levels/djbeat/corners/downleft/corner_dl_red.png", OSL_PF_8888);
	m_anims_corner_downleft[3]=new Anim(2, "corner_dl_yellow", "res/pdr4c/levels/djbeat/corners/downleft/", 2, OSL_PF_8888);
	m_anims_corner_downleft[4]=new Anim("res/pdr4c/levels/djbeat/corners/downleft/corner_dl_green.png", OSL_PF_8888);
	m_anims_corner_downleft[5]=new Anim(2, "corner_dl_cyan", "res/pdr4c/levels/djbeat/corners/downleft/", 2, OSL_PF_8888);
	
	m_anims_corner_downright[0]=new Anim(2, "corner_dr_pink", "res/pdr4c/levels/djbeat/corners/downright/", 2, OSL_PF_8888);
	m_anims_corner_downright[1]=new Anim("res/pdr4c/levels/djbeat/corners/downright/corner_dr_red.png", OSL_PF_8888);
	m_anims_corner_downright[2]=new Anim(2, "corner_dr_yellow", "res/pdr4c/levels/djbeat/corners/downright/", 2, OSL_PF_8888);
	m_anims_corner_downright[3]=new Anim("res/pdr4c/levels/djbeat/corners/downright/corner_dr_green.png", OSL_PF_8888);
	m_anims_corner_downright[4]=new Anim(2, "corner_dr_cyan", "res/pdr4c/levels/djbeat/corners/downright/", 2, OSL_PF_8888);
	m_anims_corner_downright[5]=new Anim("res/pdr4c/levels/djbeat/corners/downright/corner_dr_blue.png", OSL_PF_8888);
	
	m_anims_corner_upright[0]=new Anim("res/pdr4c/levels/djbeat/corners/upright/corner_ur_red.png", OSL_PF_8888);
	m_anims_corner_upright[1]=new Anim(2, "corner_ur_yellow", "res/pdr4c/levels/djbeat/corners/upright/", 2, OSL_PF_8888);
	m_anims_corner_upright[2]=new Anim("res/pdr4c/levels/djbeat/corners/upright/corner_ur_green.png", OSL_PF_8888);
	m_anims_corner_upright[3]=new Anim(2, "corner_ur_cyan", "res/pdr4c/levels/djbeat/corners/upright/", 2, OSL_PF_8888);
	m_anims_corner_upright[4]=new Anim("res/pdr4c/levels/djbeat/corners/upright/corner_ur_blue.png", OSL_PF_8888);
	m_anims_corner_upright[5]=new Anim(2, "corner_ur_pink", "res/pdr4c/levels/djbeat/corners/upright/", 2, OSL_PF_8888);
	
	for(int i=0;i<6;i++)
	{
		m_anims_corner_upleft[i]->start();
		m_anims_corner_upleft[i]->setReferencePoint(0,0);
		
		m_anims_corner_downleft[i]->start();
		m_anims_corner_downleft[i]->setReferencePoint(0,45);
		
		m_anims_corner_downright[i]->start();
		m_anims_corner_downright[i]->setReferencePoint(45,45);
		
		m_anims_corner_upright[i]->start();
		m_anims_corner_upright[i]->setReferencePoint(45,0);
	}
	
	
}

TranslationEffect *PDR4CLevelDjbeat::nextTranslationEffect(TranslationEffect *old)
{
	int xfrom, yfrom; //départ de la translation
	int xto=0, yto=0; //arrivée de la translation
	if(old==NULL)
	{
		xfrom=240;
		yfrom=136;
	}
	else
	{
		Pos p=old->getEndPos();
		xfrom=p.x;
		yfrom=p.y;
	}
	
	int cellfrom;
	//l'ecran est divisé en 9 cases, on cherche ou on est
	if(xfrom<160)
	{
		if(yfrom<90)
		{
			cellfrom=1;
		}
		else if(yfrom<180)
		{
			cellfrom=2;
		}
		else
		{
			cellfrom=3;
		}
	}
	else if(xfrom<320)
	{
		if(yfrom<90)
		{
			cellfrom=4;
		}
		else if(yfrom<180)
		{
			cellfrom=5;
		}
		else
		{
			cellfrom=6;
		}
	}
	else //entre 320 et 480
	{
		if(yfrom<90)
		{
			cellfrom=7;
		}
		else if(yfrom<180)
		{
			cellfrom=8;
		}
		else
		{
			cellfrom=9;
		}
	}
	
	//on va dans une autre case, sauf celle du milieu (5)
	int cellto=random(1,9);
	while(cellto==5 || cellto==cellfrom)
	{
		cellto=random(1,9);
	}
	
	if(cellto==1)
	{
		xto=random(0, 159);
		yto=random(0, 89);
	}
	else if(cellto==2)
	{
		xto=random(160, 319);
		yto=random(0, 89);
	}
	else if(cellto==3)
	{
		xto=random(320, 479);
		yto=random(0, 89);
	}
	else if(cellto==4)
	{
		xto=random(0, 159);
		yto=random(90, 179);
	}
	else if(cellto==6)
	{
		xto=random(320, 479);
		yto=random(90, 179);
	}
	else if(cellto==7)
	{
		xto=random(0, 159);
		yto=random(180, 271);
	}
	else if(cellto==8)
	{
		xto=random(160, 319);
		yto=random(180, 271);
	}
	else if(cellto==9)
	{
		xto=random(320, 479);
		yto=random(180, 271);
	}
	
	int time=calculateTranslationTime(xfrom, yfrom, xto, yto);
		
	return new TranslationEffect(xfrom, yfrom, xto, yto, time, time, true, 1);
}

int PDR4CLevelDjbeat::calculateTranslationTime(int x0, int y0, int x1, int y1)
{
	//calcul de la distance qu'elle va parcourir
	float dist=pow(float(x1-x0),2.0f)+pow(float(y1-y0),2.0f);
	dist=sqrt(dist);
	
	float time;
	if(m_translation_fast)
	{
		time=dist*m_speedFast;//*0.20f;
	}
	else
	{
		time=dist*m_speedSlow;//*0.60f;
	}
	
	if(time<2.0f)
		time=2.0f;
	
	return int(time);
}

void PDR4CLevelDjbeat::drawCircles()
{
	//les cercles
	if(m_circles_big)
	{
		//if(m_frame_even)
		//{
			fx::draw(m_img_circle_blue_big, m_fx_circle_blue);
			fx::draw(m_img_circle_red_big, m_fx_circle_red);
		//}
		//else
			fx::draw(m_img_circle_green_big, m_fx_circle_green);
	}
	else
	{
		//if(m_frame_even)
		//{
			fx::draw(m_img_circle_blue_small, m_fx_circle_blue);
			fx::draw(m_img_circle_red_small, m_fx_circle_red);
		//}
		//else
			fx::draw(m_img_circle_green_small, m_fx_circle_green);
	}
}

void PDR4CLevelDjbeat::changeCirclesSpeed()
{
	Pos start;
	Pos end;
	int time;
	
	//blue
	start=m_fx_circle_blue->getStartPos();
	end=m_fx_circle_blue->getEndPos();
	time=calculateTranslationTime(start.x, start.y, end.x, end.y);
	
	m_fx_circle_blue->setTimeForward(time);
	m_fx_circle_blue->setTimeBackward(time);
	
	//green
	start=m_fx_circle_green->getStartPos();
	end=m_fx_circle_green->getEndPos();
	time=calculateTranslationTime(start.x, start.y, end.x, end.y);
	
	m_fx_circle_green->setTimeForward(time);
	m_fx_circle_green->setTimeBackward(time);
	
	//red
	start=m_fx_circle_red->getStartPos();
	end=m_fx_circle_red->getEndPos();
	time=calculateTranslationTime(start.x, start.y, end.x, end.y);
	
	m_fx_circle_red->setTimeForward(time);
	m_fx_circle_red->setTimeBackward(time);
	
}
































