#include "code/game/levels/impl/musong/LevelMusong.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/game/pieces/impl/PieceCombo.h"
#include "code/game/pieces/impl/PieceShielded.h"
#include "code/framework/Properties.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define LEVEL_MUSONG_PART_TERMINATOR1 1
#define LEVEL_MUSONG_PART_BLACK1 2
#define LEVEL_MUSONG_PART_ISLAND 3
#define LEVEL_MUSONG_PART_BLACK2 4
#define LEVEL_MUSONG_PART_TERMINATOR2 5
#define LEVEL_MUSONG_DURATION 380000.0f

string LevelMusong::getCode()
{
	return "musong";
}

LevelMusong::LevelMusong(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{		
	Properties *props=NULL;
	
	props=new Properties("res/levels/musong/musong.properties");
	m_linepattern_terminator=new LinePattern("res/levels/musong/musong.part1.pattern");
	m_linepattern_island=new LinePattern("res/levels/musong/musong.part2.pattern");
	m_gridImage=fwkLoadImageFilePNG("res/levels/musong/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	
	
	m_dropspeed=props->getIntValue("normal.dropspeed.start");
	m_upspeed=props->getIntValue("normal.upspeed.start");
		
		
	m_dropspeed_terminator=props->getIntValue("normal.dropspeed.terminator");
	m_upspeed_terminator=props->getIntValue("normal.upspeed.terminator");
	m_dropspeed_island=props->getIntValue("normal.dropspeed.island");
	m_upspeed_island=props->getIntValue("normal.upspeed.island");
	
	m_jokerRate=props->getIntValue("normal.rate.joker");
	m_shildedRate=props->getIntValue("normal.rate.shielded");
	m_normalRate=props->getIntValue("normal.rate.normal");
	m_comboRate=props->getIntValue("normal.rate.combo");
	
	m_comboPieceMin=props->getIntValue("normal.piece.combo.min");
	m_comboPieceMax=props->getIntValue("normal.piece.combo.max");
		 
	m_difficulty=difficulty;
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	m_comboDrawer=new ComboNumberDrawer();
	m_scenario=new EventsScenario("res/levels/musong/musong.scenario");
	m_music=oslLoadSoundFileBGM("res/levels/musong/musong.bgm", OSL_FMT_STREAM);
	
	m_addline=3;
	
	m_part=LEVEL_MUSONG_PART_TERMINATOR1;
	
	
	m_img_allblack=oslCreateImage(480,272,OSL_IN_RAM, OSL_PF_4444);
	oslClearImage(m_img_allblack, RGB(0,0,0));
	
	m_fx_darkpause1=new SerialGlobalEffect(1);
	m_fx_darkpause1->addEffect(new ProgressiveAlphaEffect(20, 20, 0, 255, 1)); //apparition du noir
	m_fx_darkpause1->addEffect(new ProgressiveAlphaEffect(60, 60, 255, 255, 1)); //attente dans le noir
	m_fx_darkpause1->addEffect(new ProgressiveAlphaEffect(20, 20, 255, 0, 1)); //disparition du noir
	
	m_fx_darkpause2=new SerialGlobalEffect(1);
	m_fx_darkpause2->addEffect(new ProgressiveAlphaEffect(20, 20, 0, 255, 1)); //apparition du noir
	m_fx_darkpause2->addEffect(new ProgressiveAlphaEffect(60, 60, 255, 255, 1)); //attente dans le noir
	m_fx_darkpause2->addEffect(new ProgressiveAlphaEffect(20, 20, 255, 0, 1)); //disparition du noir
	
	m_delayer_dark=NULL;
	
	m_changeToShieldedRate=props->getIntValue("normal.part3.rate.shielded");
	
	delete props;
	
	
	
}

LevelMusong::~LevelMusong()
{
	oslDeleteImage(m_gridImage);
	
	delete m_linepattern_terminator;
	delete m_linepattern_island;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
		
	oslDeleteSound(m_music);
	oslDeleteImage(m_img_allblack);
	
	delete m_fx_darkpause1;
	delete m_fx_darkpause2;
	
	if(m_delayer_dark!=NULL)
		delete m_delayer_dark;
}


//dessin avec une paire
void LevelMusong::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelMusong::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelMusong::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelMusong::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
	
	if(isPaused())
	{
		//TODO: vrai menu pause
		oslDrawString(220, 130, "pause");
	}
	else
	{
		//dessin du contenu de la grille
		
		
			for(int i=0;i<grid->getNbCols();i++)
			{
				for(int j=0;j<grid->getNbLines();j++)
				{
					MatrixContentObject *mco=grid->getAt(i,j);
					if(mco->getPiece()!=NULL)
					{
						Pos pos=grid->getContentPosition(i, j);
						AbstractPiece *p=mco->getPiece();
											
						p->drawXY(pos.x,pos.y);					
					}
				}	
			}
		
		
	}
	
	
}

void LevelMusong::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	nextPair->drawXY(60, 30, grid, APPLY_FX_ON_NONE);
	oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	
	m_songbanner->draw();
	m_comboDrawer->drawXY(200, 180); //TODO: calculer le placement
	
	if(m_fx_darkpause1->isRunning())
	{
		m_fx_darkpause1->activate();
		oslDrawImage(m_img_allblack);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	else if(m_fx_darkpause2->isRunning())
	{
		m_fx_darkpause2->activate();
		oslDrawImage(m_img_allblack);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
	oslDrawString(10, 200, ("drop: ="+toString(m_dropspeed)).c_str());
	oslDrawString(10, 210, ("up: ="+toString(m_upspeed)).c_str());
	oslDrawString(10, 220, ("part: ="+toString(m_part)).c_str());
	
	
}

void LevelMusong::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

//appelé uniquement si le level n'est pas en pause.
void LevelMusong::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	
	m_scenario->update();
	m_fx_darkpause1->update();
	m_fx_darkpause2->update();
	
	if(m_delayer_dark!=NULL)
		m_delayer_dark->update();
		
	
	if(m_delayer_dark!=NULL && m_delayer_dark->get())
	{
		//changer les pieces
		if(m_part==LEVEL_MUSONG_PART_BLACK1) //on change en pieces simples
		{
			changePiecesToSimple(grid, nextPair, currentPair);
		}
		else if(m_part==LEVEL_MUSONG_PART_BLACK2) //on change en pieces shielded
		{
			changePiecesToShielded(grid, nextPair, currentPair);	
		}
	}
	
		
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //1eres basses tu tu ti tu du ti
	{
		m_dropspeed=m_dropspeed_terminator;
		m_upspeed=m_upspeed_terminator;
	}
	if(m_scenario->hasEvent(2)) //terminator (bruit mettallique lointain ta tata ta tata ta)
	{
		
	}
	if(m_scenario->hasEvent(3)) //fin terminator
	{
		
	}
	
	if(m_scenario->hasEvent(4)) //0 bruit
	{
		if(m_part==LEVEL_MUSONG_PART_TERMINATOR1)
		{
			m_part=LEVEL_MUSONG_PART_BLACK1;
			m_fx_darkpause1->restart();
			if(m_delayer_dark!=NULL)
				delete m_delayer_dark;
			m_delayer_dark=new Delayer(30, 1);
			m_upspeed=-1;
			m_dropspeed=-1;
		}
		else if(m_part==LEVEL_MUSONG_PART_ISLAND)
		{
			m_part=LEVEL_MUSONG_PART_BLACK2;
			m_fx_darkpause2->restart();
			if(m_delayer_dark!=NULL)
				delete m_delayer_dark;
			m_delayer_dark=new Delayer(30, 1);
			m_upspeed=-1;
			m_dropspeed=-1;
		}
	}
	
	if(m_scenario->hasEvent(5)) //debut musique des iles
	{
		m_part=LEVEL_MUSONG_PART_ISLAND;
		m_dropspeed=m_dropspeed_island;
		m_upspeed=m_upspeed_island;
	}
	
	if(m_scenario->hasEvent(8)) // debut musique industrielle
	{
		m_part=LEVEL_MUSONG_PART_TERMINATOR2;
		m_dropspeed=m_dropspeed_terminator;
		m_upspeed=m_upspeed_terminator;
	}
	
	
}

void LevelMusong::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelMusong::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}

list<AbstractPiece *> *LevelMusong::getNextLine()
{
	if(m_part==LEVEL_MUSONG_PART_TERMINATOR1 || m_part==LEVEL_MUSONG_PART_TERMINATOR2)
		return m_linepattern_terminator->getNextLine();
	return m_linepattern_island->getNextLine();
}

bool LevelMusong::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelMusong::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	if(m_part==LEVEL_MUSONG_PART_BLACK1 || m_part==LEVEL_MUSONG_PART_BLACK2)
		return -1;
	return m_dropspeed;
}

int LevelMusong::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	if(m_part==LEVEL_MUSONG_PART_BLACK1 || m_part==LEVEL_MUSONG_PART_BLACK2)
		return -1;
	return m_upspeed;
}

void LevelMusong::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		
	}
}

bool LevelMusong::finished()
{
	if(m_scenario->getTime()>=LEVEL_MUSONG_DURATION)
	{
		return true;
	}
	return false;
}

float LevelMusong::getDuration()
{
	return LEVEL_MUSONG_DURATION;
}

Pair *LevelMusong::getNewPair()
{
	if(m_part==LEVEL_MUSONG_PART_ISLAND)
		return AbstractLevel::getNewPair();
	
	//else (partie terminator)
	AbstractPiece *p1=getRandomPiece(), *p2=getRandomPiece();
	
	
	Pair *p=new Pair(p1,p2);
	
	int posX=int(m_nbCols/2)-1;
	int posY=-1; //au dessus de tout en haut
	p->goTo(posX, posY);
	
	return p;
}

AbstractPiece *LevelMusong::getRandomPiece()
{
	int r=random(1, 100);
	
	if(r<=m_normalRate)
	{
		return new PieceSimple();
	}
	else if(r<=m_normalRate+m_shildedRate)
	{
		return new PieceShielded();
	}
	else if(r<=m_normalRate+m_shildedRate+m_comboRate)
	{
		return new PieceCombo(random(m_comboPieceMin, m_comboPieceMax));
	}
	//else
	return new PieceJoker();
}



void LevelMusong::changePiecesToSimple(GameGrid *grid, Pair *nextPair, Pair *currentPair)
{
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			if(mco->getPiece()!=NULL)
			{
				AbstractPiece *p=mco->getPiece();
				if(p->getType()==PIECE_TYPE_SHIELDED)
				{
					mco->setPiece(new PieceSimple(p->getShape(), p->getColor()));
				}				
			}
		}	
	}
	
	//next pair
	changePairToSimple(nextPair);
	
	//currentpair
	changePairToSimple(currentPair);
}

void LevelMusong::changePairToSimple(Pair *pair)
{
	for(int i=0;i<=1;i++)
	{
		AbstractPiece *oldp=pair->getPiece(i);
		
		if(oldp->getType()==PIECE_TYPE_SHIELDED)
		{
			if(i==0)
				pair->setPiece0(new PieceSimple(oldp->getShape(), oldp->getColor()));
			else
				pair->setPiece1(new PieceSimple(oldp->getShape(), oldp->getColor()));
		}
		else if(oldp->getType()==PIECE_TYPE_COMBO)
		{
			int nbCombos=oldp->specialEvent(PIECE_EVENT_COMBO_GET);
			for(int c=0; c<nbCombos-1; c++)
			{
				oldp->specialEvent(PIECE_EVENT_COMBO_REDUCE); //on va se retrouver a 1
			}
		}
	}
}

void LevelMusong::changePiecesToShielded(GameGrid *grid, Pair *nextPair, Pair *currentPair)
{
	//grille
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			if(mco->getPiece()!=NULL)
			{
				AbstractPiece *p=mco->getPiece();
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					int r=random(1,100);
					if(r<=m_changeToShieldedRate)
						mco->setPiece(new PieceShielded(p->getShape(), p->getColor()));
				}				
			}
		}	
	}
	
	//next pair
	changePairToShielded(nextPair);
	
	//current pair
	changePairToShielded(currentPair);
	
}

void LevelMusong::changePairToShielded(Pair *pair)
{
	for(int i=0;i<=1;i++)
	{
		int r=random(1,100);
		if(r<=m_changeToShieldedRate)
		{
			AbstractPiece *p=new PieceShielded(pair->getPiece(i)->getShape(), pair->getPiece(i)->getColor());
			if(i==0)
				pair->setPiece0(p);
			else
				pair->setPiece1(p);
		}
	}
}























