#include "code/game/levels/impl/ssoup/LevelSsoup.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomShake.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/Properties.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomPosition.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomColor.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomProportionMultiTint.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomAngleTranslation.h"
#include <math.h>
#include "code/framework/check/loaders.h"

using namespace utils;

//#define LEVEL_SSOUP_DROPSPEED 30
//#define LEVEL_SSOUP_DROPSPEED_FAST 20
//#define LEVEL_SSOUP_DROPSPEED_SLOW 60
//#define LEVEL_SSOUP_UPSPEED 450
//#define LEVEL_SSOUP_UPSPEED_FAST 250
//#define LEVEL_SSOUP_UPSPEED_SLOW 700

#define LEVEL_SSOUP_DURATION 415000.0f

#define LEVEL_SSOUP_COLOR_BLACK RGB(0,0,0)
#define LEVEL_SSOUP_COLOR_WHITE RGB(255,255,255)
#define LEVEL_SSOUP_COLOR_RED RGB(255,0,0)
#define LEVEL_SSOUP_COLOR_YELLOW RGB(254,214,13)

#define LEVEL_SSOUP_NUKEMED_POS_X 232
#define LEVEL_SSOUP_NUKEMED_POS_Y 137

#define LEVEL_SSOUP_FIRE_POS_X 237
#define LEVEL_SSOUP_FIRE_POS_Y 280

string LevelSsoup::getCode()
{
	return "ssoup";
}

LevelSsoup::LevelSsoup(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{	
	Properties *props=new Properties("res/levels/ssoup/ssoup.properties");
	
	if(difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_linepattern=new LinePattern("res/levels/ssoup/ssoup_hard.pattern");
	}
	else if(difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_linepattern=new LinePattern("res/levels/ssoup/ssoup.pattern");
	}
	else
	{
		fatalError("no such difficulty level in S Soup");
	}
	
	m_addline=0;
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	
	m_comboDrawer=new ComboNumberDrawer();
	
	m_scenario=new EventsScenario("res/levels/ssoup/ssoup.scenario");
	
	m_music=fwkLoadSoundFileMP3("res/levels/ssoup/ssoup.mp3", OSL_FMT_STREAM);
	
	m_fx_shake1=new ShakeEffect(4, 0, 3, true, 20);
	m_fx_shake2=new ShakeEffect(5, 0, 4, false, 15);
	m_fx_shake3=new ShakeEffect(8, 0, 3, true, 20);
	m_fx_shake4=new ShakeEffect(9, 0, 2, false, 30);
	
	
	
	m_fxmgr_wave=new LocalEffectManager();
	m_fx_wave=new AbstractLocalEffect *[m_nbLines];
	for(int i=0;i<m_nbLines;i++)
	{
		if(i==0)
		{
			ShakeEffect *shakefx=new ShakeEffect(20, 0, 15, false, 3);
			m_fxmgr_wave->addEffect(shakefx);
			m_fx_wave[0]=shakefx;
		}
		else
		{
			SerialLocalEffect *serialfx=new SerialLocalEffect(1);
			WaitLocalEffect *waitfx=new WaitLocalEffect(4*i);
			ShakeEffect *shakefx=new ShakeEffect(20, 0, 15, false, 3);
			serialfx->addEffect(waitfx);
			serialfx->addEffect(shakefx);
			m_fxmgr_wave->addEffect(serialfx);
			m_fx_wave[i]=serialfx;
		}
	}
	
	
	m_fx_click_size=new SizeWowEffect(0, 150, 0, 150, 8, 8, true, 1);
	m_fx_click_alpha=new ProgressiveAlphaEffect(8, 8, 255, 150, 1);
	
	m_alphastate=new AlphaParamsState();
	
	
	m_dropspeed_normal=props->getIntValue("dropspeed.normal");
	m_dropspeed_slow=props->getIntValue("dropspeed.slow");
	m_dropspeed_fast=props->getIntValue("dropspeed.fast");
	m_upspeed_normal=props->getIntValue("upspeed.normal");
	m_upspeed_slow=props->getIntValue("upspeed.slow");
	m_upspeed_fast=props->getIntValue("upspeed.fast");
	
	m_dropspeed=m_dropspeed_normal;
	m_upspeed=-1;
	
	/////DECORATION
	//BackGround
	m_img_bg=oslCreateImage(1,1,OSL_IN_RAM, OSL_PF_8888);
	oslClearImage(m_img_bg, RGB(255,255,255));
	m_img_bg->stretchX=480;
	m_img_bg->stretchY=272;
	oslSwizzleImage(m_img_bg);
		
	m_bg_speed_min=props->getIntValue("fx.bg.time.min");
	m_bg_speed_max=props->getIntValue("fx.bg.time.max");
	m_bg_time_total_min=props->getFloatValue("fx.bg.time.total.min");
	m_bg_time_total_max=props->getFloatValue("fx.bg.time.total.max");
	
	m_bg_frame_counter=0;
	m_bg_fx_time=0;
	
	m_fx_bg=new ProgressiveTintEffect(1, 1, LEVEL_SSOUP_COLOR_BLACK, LEVEL_SSOUP_COLOR_RED, 2);
	updateBG(true);
	
	//grille
	m_gridImage=new OSL_IMAGE *[3];
	m_gridImage[0]=fwkLoadImageFilePNG("res/levels/ssoup/grid1.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	m_gridImage[1]=fwkLoadImageFilePNG("res/levels/ssoup/grid2.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	m_gridImage[2]=fwkLoadImageFilePNG("res/levels/ssoup/grid3.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	
	
	m_current_theme=utils::random(0,2);
	
	//fire
	int firelife=props->getIntValue("partgen.fire.life");
	int ptmin=props->getIntValue("partgen.fire.particle.color.transition.minimum.time");
	int ptfinaltransp=props->getIntValue("partgen.fire.particle.color.final.transparency");
	
	RandomAnimationFactory *raf=new RandomAnimationFactory();
	Anim *animptr=new Anim("res/levels/ssoup/particle.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/ssoup/particle2.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/ssoup/particle3.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	m_partgen=new ParticleGeneratorSegment(	LEVEL_SSOUP_FIRE_POS_X, LEVEL_SSOUP_FIRE_POS_Y, 
											90, props->getIntValue("partgen.fire.width"), 
											props->getIntValue("partgen.fire.throughput.particles"), props->getIntValue("partgen.fire.throughput.frames"), 
											firelife, props->getIntValue("partgen.fire.distance.max"), firelife, 
											raf);
	m_partgen->setLocalEffect1(new PFXFRandomTranslation(props->getIntValue("partgen.fire.distance.min"), props->getIntValue("partgen.fire.distance.max")));
	PFXFRandomProportionMultiTint *colorfx=new PFXFRandomProportionMultiTint(firelife, ptmin);
	colorfx->addColor(LEVEL_SSOUP_COLOR_YELLOW);
	colorfx->addColor(LEVEL_SSOUP_COLOR_RED);
	colorfx->addColor(RGBA(0,0,0,ptfinaltransp));
	m_partgen->setGlobalEffect1(colorfx);
	
	m_fx_fire_shake=new ShakeEffect(0, props->getIntValue("partgen.fire.shake.size"), props->getIntValue("partgen.fire.shake.time"));
		
	//usines et leur fumee
	float resize_factor=props->getFloatValue("img.factory.size.mult");
	int fxsmoketime=props->getIntValue("fx.factory.smoke.color.time");
	m_img_factory=fwkLoadImageFilePNG("res/levels/ssoup/factory.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_factory_smoke=fwkLoadImageFilePNG("res/levels/ssoup/factory_smoke.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_tree_front=fwkLoadImageFilePNG("res/levels/ssoup/tree_fg.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_tree_back=fwkLoadImageFilePNG("res/levels/ssoup/tree_bg.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_img_factory);
	oslImageSetRotCenter(m_img_factory_smoke);
	oslImageSetPosition(m_img_factory, props->getIntValue("img.factory.pos.x"), props->getIntValue("img.factory.pos.y"));
	oslImageSetPosition(m_img_factory_smoke, props->getIntValue("img.factory.smoke.pos.x"), props->getIntValue("img.factory.smoke.pos.y"));
	m_img_factory->stretchX=int(m_img_factory->sizeX*resize_factor);
	m_img_factory->stretchY=int(m_img_factory->sizeY*resize_factor);
	m_img_factory_smoke->stretchX=int(m_img_factory_smoke->sizeX*resize_factor);
	m_img_factory_smoke->stretchY=int(m_img_factory_smoke->sizeY*resize_factor);
	m_fx_factory_smoke_turn=new RotateBackAndForthEffect(
									props->getIntValue("fx.factory.smoke.rotate.time"), 
									props->getIntValue("fx.factory.smoke.rotate.from"), 
									props->getIntValue("fx.factory.smoke.rotate.to")
								);
	
	m_fx_factory_smoke_color=new SerialGlobalEffect();
	m_fx_factory_smoke_color->addEffect(new ProgressiveTintEffect(fxsmoketime, fxsmoketime, LEVEL_SSOUP_COLOR_BLACK, LEVEL_SSOUP_COLOR_RED, 1));
	m_fx_factory_smoke_color->addEffect(new ProgressiveTintEffect(fxsmoketime, fxsmoketime, LEVEL_SSOUP_COLOR_RED, LEVEL_SSOUP_COLOR_WHITE, 1));
	m_fx_factory_smoke_color->addEffect(new ProgressiveTintEffect(fxsmoketime, fxsmoketime, LEVEL_SSOUP_COLOR_WHITE, LEVEL_SSOUP_COLOR_RED, 1));
	m_fx_factory_smoke_color->addEffect(new ProgressiveTintEffect(fxsmoketime, fxsmoketime, LEVEL_SSOUP_COLOR_RED, LEVEL_SSOUP_COLOR_BLACK, 1));
	
	m_fx_factory_appear=new ProgressiveAlphaEffect(
								props->getIntValue("fx.factory.appear.time"), 
								props->getIntValue("fx.factory.appear.time"), 
								0, 255, 1);
	
	
	//jauge
	int gaugetime=props->getIntValue("fx.gauge.time");
	m_img_gauge=oslCreateImage(1,1,OSL_IN_RAM,OSL_PF_8888);
	m_img_gauge->stretchX=3;
	oslClearImage(m_img_gauge, RGB(255,255,255));
	oslSwizzleImage(m_img_gauge);
	m_fx_gauge=new ProgressiveTintEffect(gaugetime, gaugetime, LEVEL_SSOUP_COLOR_RED, LEVEL_SSOUP_COLOR_WHITE);
	m_fx_gauge->start();
	
	//generateur nukes
	int life=props->getIntValue("partgen.nukes.life");
	int nbPart=props->getIntValue("partgen.nukes.throughput.particles");
	int nbFrames=props->getIntValue("partgen.nukes.throughput.frames");
	
	m_gen_nukes=new ParticleGeneratorSegment(	240, 300, 90, 480, nbPart, nbFrames, life, 330, life, 
												"res/levels/ssoup/nuke_small.png", OSL_PF_8888);
	int fade=props->getIntValue("partgen.nukes.fade.time");
												
	m_gen_nukes->setLocalEffect1(new PFXFRandomPosition());
	SerialGlobalEffect *fx_appearwaitvanish=new SerialGlobalEffect(1);
	fx_appearwaitvanish->addEffect(new ProgressiveAlphaEffect(fade, fade, 0, 255, 1));
	fx_appearwaitvanish->addEffect(new WaitGlobalEffect(life-(2*fade)));
	fx_appearwaitvanish->addEffect(new ProgressiveAlphaEffect(fade, fade, 255, 0, 1));
	m_gen_nukes->setGlobalEffect1(fx_appearwaitvanish);
	
	
	//flash
	m_flash_duration=props->getIntValue("flash.duration");
	m_flash_frame_counter=0;
	m_flash_color=0;
	
	
	//gros nuke qui tourne
	m_img_nuke_med=fwkLoadImageFilePNG("res/levels/ssoup/nuke_med.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_img_nuke_med);
	m_fx_nukemed_rot=new RotationEffect(props->getIntValue("nukemed.turn.time"));
	m_fx_nukemed_transl=new TranslationEffect(LEVEL_SSOUP_NUKEMED_POS_X, LEVEL_SSOUP_NUKEMED_POS_Y, LEVEL_SSOUP_NUKEMED_POS_X, LEVEL_SSOUP_NUKEMED_POS_Y, 1, 1, true, 1);
	m_nukemed_turning=false;
	m_nukemed_speed=props->getFloatValue("nukemed.speed");
	//son anim:
	m_anim_nuke=new Anim(6, "nuke_med", "res/levels/ssoup/nukeanim/", props->getFloatValue("anim.nuke.time"), OSL_PF_8888, 1);
	m_anim_nuke->setX(LEVEL_SSOUP_NUKEMED_POS_X);
	m_anim_nuke->setY(LEVEL_SSOUP_NUKEMED_POS_Y);
	m_anim_nuke->setReferencePointCenter();
	//son générateur
	int nukemedgenparticles=props->getIntValue("partgen.nukemed.throughput.particles");
	int nukemedgenframes=props->getIntValue("partgen.nukemed.throughput.frames");
	int nukemedgenfxwait=props->getIntValue("partgen.nukemed.fx.wait");
	int nukemedgenfxvanish=props->getIntValue("partgen.nukemed.fx.vanish");
	int nukemedgenlifespan=nukemedgenfxwait+nukemedgenfxvanish;
	m_partgen_nukemed=new ParticleGeneratorPoint(	0, 0, 0, 0, nukemedgenparticles, nukemedgenframes, 0, 0, nukemedgenlifespan, 
													"res/levels/ssoup/nuke_small.png", OSL_PF_8888);
	SerialGlobalEffect *nukemedgenfx=new SerialGlobalEffect(1);
	nukemedgenfx->addEffect(new WaitGlobalEffect(nukemedgenfxwait));
	nukemedgenfx->addEffect(
		new ProgressiveAlphaEffect(nukemedgenfxvanish, nukemedgenfxvanish, 255, 0, 1)
		//new ProgressiveTintEffect(nukemedgenfxvanish, nukemedgenfxvanish, LEVEL_SSOUP_COLOR_YELLOW, RGBA(255,0,0,0), 1)
	);
	m_partgen_nukemed->setGlobalEffect1(nukemedgenfx);
	
	//nukeboarder
	initNukeBoarder(props);
	
	//no nuke, more trees
	m_img_nonuke=fwkLoadImageFilePNG("res/levels/ssoup/nonuke.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_nonuke, props->getIntValue("nonuke.x"), props->getIntValue("nonuke.y"));
	m_img_moretrees=fwkLoadImageFilePNG("res/levels/ssoup/moretrees.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_moretrees, props->getIntValue("moretrees.x"), props->getIntValue("moretrees.y"));
	m_nonuke_delayer=new Repeater(props->getIntValue("nonuke.display.time"), 1);
	m_bool_nonuke=true;
	
	//rain
	raf=new RandomAnimationFactory();
	animptr=new Anim("res/levels/ssoup/rainp.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/ssoup/rainp2.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/ssoup/rainp3.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	m_partgen_rain= new ParticleGeneratorSegment(	props->getIntValue("partgen.rain.pos.x"), 
													props->getIntValue("partgen.rain.pos.y"), 
													props->getIntValue("partgen.rain.direction"), 
													props->getIntValue("partgen.rain.scale"), 
													props->getIntValue("partgen.rain.throughput.particles"), 
													props->getIntValue("partgen.rain.throughput.frames"), 
													props->getIntValue("partgen.rain.time"), 
													props->getIntValue("partgen.rain.distance"), 
													props->getIntValue("partgen.rain.time"), 
													raf);
	m_partgen_rain->setLocalEffect1(new PFXFRandomAngleTranslation(
													props->getIntValue("partgen.rain.angle.min"),
													props->getIntValue("partgen.rain.angle.max"))
									);
	m_partgen_rain->setLocalEffect2(
									new RotationEffect(
													props->getIntValue("partgen.rain.particles.rotation.time"), 
													false)
									);
	PFXFRandomColor *fxraincolor=new PFXFRandomColor(LEVEL_SSOUP_COLOR_BLACK);
	fxraincolor->addColor(LEVEL_SSOUP_COLOR_RED);
	fxraincolor->addColor(LEVEL_SSOUP_COLOR_WHITE);
	fxraincolor->addColor(LEVEL_SSOUP_COLOR_YELLOW);
	m_partgen_rain->setGlobalEffect1(fxraincolor);
	
	m_fx_rain_shake=new ShakeEffect(	props->getIntValue("partgen.rain.particles.shake.width"), 
										props->getIntValue("partgen.rain.particles.shake.height"), 
										props->getIntValue("partgen.rain.particles.shake.time"));
	m_fx_rain_shake2=new SerialLocalEffect();
	m_fx_rain_shake2->addEffect(new WaitLocalEffect(props->getIntValue("partgen.rain.particles.shake2.waittime")));
	m_fx_rain_shake2->addEffect(
									new ShakeEffect(
										props->getIntValue("partgen.rain.particles.shake2.width"), 
										props->getIntValue("partgen.rain.particles.shake2.height"), 
										props->getIntValue("partgen.rain.particles.shake2.time"), 
										true,
										props->getIntValue("partgen.rain.particles.shake2.loops"))
								);
								
	m_img_pause=fwkLoadImageFilePNG("res/levels/ssoup/stop"+toString(random(1,3))+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	//oslImageSetRotCenter(m_img_pause);
	oslImageCenterScreen(m_img_pause);
	
	delete props;
}

void LevelSsoup::reset()
{
	m_linepattern->reset();
	m_addline=0;
	m_songbanner->reset();
	m_comboDrawer->reset();
	m_scenario->reset();
	oslStopSound(m_music);
	m_dropspeed=m_dropspeed_normal;
	m_upspeed=-1;
	m_current_theme=utils::random(0,2);
	m_partgen->generationOff();
	m_fx_nukeboarder_end_move->stop();
	m_fx_nukeboarder_end_resize->stop();
	m_fx_nukeboarder_start_move->stop();
	m_fx_nukeboarder_start_resize->stop();
	m_fx_nukeboarder_end_move->reset();
	m_fx_nukeboarder_end_resize->reset();
	m_fx_nukeboarder_start_move->reset();
	m_fx_nukeboarder_start_resize->reset();
	m_fx_fire_shake->stop();
	m_nukeboarder_delayer->stop();
	m_nukeboarder_delayer->reset();
	m_fx_factory_smoke_turn->stop();
	m_fx_factory_smoke_color->stop();
	m_fx_factory_appear->stop();
	m_fx_factory_smoke_turn->reset();
	m_fx_factory_smoke_color->reset();
	m_fx_factory_appear->reset();
	m_gen_nukes->generationOff();
	m_nukemed_turning=false;
	m_partgen_nukemed->generationOff();
	m_partgen_nukemed->stop();
	m_partgen_rain->generationOff();
	m_fx_rain_shake->stop();
	m_fx_rain_shake2->stop();
}

void LevelSsoup::initNukeBoarder(Properties *props)
{
	m_img_nukeboarder=fwkLoadImageFilePNG("res/levels/ssoup/nukeboarder.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	Pos nb_pos_start(props->getIntValue("nukeboarder.startpos.x"), props->getIntValue("nukeboarder.startpos.y"));
	Pos nb_pos_middle(props->getIntValue("nukeboarder.middlepos.x"), props->getIntValue("nukeboarder.middlepos.y"));
	Pos nb_pos_end(props->getIntValue("nukeboarder.endpos.x"), props->getIntValue("nukeboarder.endpos.y"));
	
	oslImageSetRotCenter(m_img_nukeboarder);
	oslImageSetPosition(m_img_nukeboarder, nb_pos_middle.x, nb_pos_middle.y);
	
	int nb_comeTime=props->getIntValue("nukeboarder.come.time");
	
	m_fx_nukeboarder_start_move=new SerialLocalEffect(1);	
	m_fx_nukeboarder_start_move->addEffect(
		new TranslationEffect(nb_pos_start.x-nb_pos_middle.x, nb_pos_start.y-nb_pos_middle.y, 0, 0, nb_comeTime, nb_comeTime, true, 1)
	);
	
	int nb_shakeSize=props->getIntValue("nukeboarder.shake.size");
	int nb_shakeTime=props->getIntValue("nukeboarder.shake.time");
	m_fx_nukeboarder_start_move->addEffect(
		new ShakeEffect(0, nb_shakeSize, nb_shakeTime)
	);	
	
	int nb_startSize=props->getIntValue("nukeboarder.size.start");
	int nb_middleSize=props->getIntValue("nukeboarder.size.middle");
	int nb_endSize=props->getIntValue("nukeboarder.size.end");
	int nb_startGrowTime=props->getIntValue("nukeboarder.come.grow.time");
//	shrinkPercentX, growPercentX, hrinkPercentY, growPercentY, growTime, shrinkTime, startSmall=true, nbLoops=-1
	m_fx_nukeboarder_start_resize=new SizeWowEffect(100-nb_startSize, nb_middleSize-100, 100-nb_startSize, nb_middleSize-100, nb_startGrowTime, nb_startGrowTime, true, 1);
	
	int nb_goAwayTime=props->getIntValue("nukeboarder.goaway.time");
	m_fx_nukeboarder_end_move=new TranslationEffect(0, 0, nb_pos_end.x-nb_pos_middle.x, nb_pos_end.y-nb_pos_middle.y, nb_goAwayTime, nb_goAwayTime, true, 1);
//	shrinkPercentX, growPercentX, hrinkPercentY, growPercentY, growTime, shrinkTime, startSmall=true, nbLoops=-1
	m_fx_nukeboarder_end_resize=new SizeWowEffect(100-nb_endSize, nb_middleSize-100, 100-nb_endSize, nb_middleSize-100, nb_goAwayTime, nb_goAwayTime, false, 1);	
	
	m_nukeboarder_delayer=new Repeater(props->getIntValue("nukeboarder.delay"), 1);
}

LevelSsoup::~LevelSsoup()
{	
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	delete m_partgen;
	
	fwkDeleteSound(m_music);
	
	delete m_fx_shake1;
	delete m_fx_shake2;
	delete m_fx_shake3;
	delete m_fx_shake4;
	
	m_fxmgr_wave->deleteAll();
	delete m_fxmgr_wave;
	delete[] m_fx_wave;	
	
	delete m_fx_click_size;
	delete m_fx_click_alpha;
	
	delete m_alphastate;
	
	//DECORATION
	delete m_fx_bg;
	oslDeleteImage(m_img_bg);
	
	delete m_fx_factory_smoke_color;
	oslDeleteImage(m_img_factory);
	oslDeleteImage(m_img_factory_smoke);
	
	oslDeleteImage(m_img_gauge);
	delete m_fx_gauge;
	
	delete m_gen_nukes;
	
	oslDeleteImage(m_gridImage[0]);
	oslDeleteImage(m_gridImage[1]);
	oslDeleteImage(m_gridImage[2]);
	delete[] m_gridImage;
	
	delete m_anim_nuke;
	
	oslDeleteImage(m_img_nukeboarder);
	delete m_nukeboarder_delayer;
	delete m_fx_nukeboarder_start_move;
	delete m_fx_nukeboarder_start_resize;
	delete m_fx_nukeboarder_end_move;
	delete m_fx_nukeboarder_end_resize;
	
	oslDeleteImage(m_img_nuke_med);
	delete m_fx_nukemed_rot;
	delete m_fx_nukemed_transl;
	
	//no nuke, more trees
	oslDeleteImage(m_img_nonuke);
	oslDeleteImage(m_img_moretrees);
	delete m_nonuke_delayer;
	
	delete m_fx_fire_shake;
	
	delete m_partgen_nukemed;
	
	delete m_fx_factory_smoke_turn;
	delete m_fx_factory_appear;
	
	delete m_partgen_rain;
	delete m_fx_rain_shake;
	delete m_fx_rain_shake2;
	
	oslDeleteImage(m_img_tree_front);
	oslDeleteImage(m_img_tree_back);
	
	oslDeleteImage(m_img_pause);
}


//dessin avec une paire
void LevelSsoup::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused()) //TODO: on a besoin de ca?
	{
		Pos pixPos0=grid->getContentPosition(currentPair->getGridPos(0).x, currentPair->getGridPos(0).y);
		//Pos *pixPos1=grid->getContentPosition(currentPair->getGridPos(1).x, currentPair->getGridPos(1).y);
		if(m_fx_click_size->isRunning())
		{
			m_alphastate->saveState();
			m_fx_click_alpha->activate();
			currentPair->drawXY(pixPos0.x-7, pixPos0.y, grid, fx_trans, APPLY_FX_ON_BOTH, m_fx_click_size);
			m_alphastate->restoreState();
		}
		else
		{
			currentPair->drawXY(pixPos0.x-7, pixPos0.y, grid, fx_trans, APPLY_FX_ON_NONE);	
		}
		
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelSsoup::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	if(m_fx_click_size->isRunning())
	{
		m_alphastate->saveState();
		m_fx_click_alpha->activate();
		pieceConnected->drawXY(pixPosConnected.x-7, pixPosConnected.y, m_fx_click_size);
		pieceUnconnected->drawXY(pixPosUnconnected.x-7, pixPosUnconnected.y, fx_trans, m_fx_click_size);
		m_alphastate->restoreState();
	}
	else
	{
		pieceConnected->drawXY(pixPosConnected.x-7, pixPosConnected.y);
		pieceUnconnected->drawXY(pixPosUnconnected.x-7, pixPosUnconnected.y, fx_trans);
	}
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelSsoup::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelSsoup::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//bg
	if(m_flash_frame_counter<m_flash_duration) //on est dans un flash
		oslDrawFillRect(0,0,480,272,m_flash_color);
	else
	{
		m_fx_bg->activate();
		oslDrawImage(m_img_bg);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	//dessin nukemed
	m_partgen_nukemed->draw();
	if(m_nukemed_turning)
	{
		fx::draw(m_img_nuke_med, m_fx_nukemed_rot, m_fx_nukemed_transl);
	}
	else
	{
		oslDrawImageXY(m_img_nuke_med, LEVEL_SSOUP_NUKEMED_POS_X, LEVEL_SSOUP_NUKEMED_POS_Y);
	}
	
	if(m_anim_nuke->isRunning())
	{
		m_fx_gauge->activate();
		m_anim_nuke->draw();
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	m_partgen_rain->draw(m_fx_rain_shake, m_fx_rain_shake2);
	
	//cheminees
	if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
	{
		if(m_fx_factory_appear->isRunning())
			m_fx_factory_appear->activate();
			
		//fumee
		m_fx_factory_smoke_color->activate();
		//oslDrawImageXY(m_img_factory_smoke, 55, 185);
		//oslDrawImageXY(m_img_factory_smoke, 445, 185);
		fx::draw(m_img_factory_smoke, m_fx_factory_smoke_turn);
		//fx::drawXY(m_img_factory_smoke, 440, 185, m_fx_factory_smoke_turn);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
		
		if(m_fx_factory_appear->isRunning())
			m_fx_factory_appear->activate();
		
		//cheminee
		oslDrawImage(m_img_factory);
		//oslDrawImageXY(m_img_factory, 435, 230);
		
		//arbre fond
		oslDrawImageXY(m_img_tree_back, 366, 111);
		
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	
	//dessin de la grille
	if(m_fx_shake4->isRunning())
	{
		fx::drawXY(m_gridImage[m_current_theme], 47, 6, m_fx_shake4);
	}
	else
	{
		oslDrawImageXY(m_gridImage[m_current_theme], 47, 6);
	}
	
	
	
	
	
	if(!isPaused())
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					//Pos pos(grid->getContentPosition(i, j)->x-7, grid->getContentPosition(i, j)->y);
					Pos pos=grid->getContentPosition(i, j);
					pos.x-=7;
					
					AbstractPiece *p=mco->getPiece();
					if(m_fx_shake1->isRunning() && (p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED) && 
						(p->getShape()==PIECE_SHAPE_CIRCLE || p->getShape()==PIECE_SHAPE_SQUARE))
					{
						p->drawXY(pos.x,pos.y, m_fx_shake1);
					}
					else if(m_fx_shake2->isRunning() && (p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED) && 
						(p->getShape()==PIECE_SHAPE_CROSS || p->getShape()==PIECE_SHAPE_DIAMOND))
					{
						p->drawXY(pos.x,pos.y, m_fx_shake2);
					}
					else if(m_fx_shake3->isRunning() && (p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED) && 
						(p->getShape()==PIECE_SHAPE_STAR || p->getShape()==PIECE_SHAPE_TRIANGLE))
					{
						p->drawXY(pos.x,pos.y, m_fx_shake3);
					}
					else if(m_fx_shake4->isRunning() && p->getType()!=PIECE_TYPE_NORMAL && p->getType()!=PIECE_TYPE_SHIELDED)
					{
						p->drawXY(pos.x,pos.y, m_fx_shake4);
					}
					else
					{
						mco->getPiece()->drawXY(pos.x,pos.y,m_fx_wave[grid->getNbLines()-j-1]);
					}
				}
			}	
		}
	}
}

void LevelSsoup::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//usines + fumée
	if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
	{
		if(m_fx_factory_appear->isRunning())
			m_fx_factory_appear->activate();
		
		oslDrawImageXY(m_img_tree_front, 341, 249);
		
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	/*m_fx_factory_smoke_color->activate();
	oslDrawImageXY(m_img_factory_smoke, 55, 185);
	oslDrawImageXY(m_img_factory_smoke, 445, 185);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	oslDrawImageXY(m_img_factory, 45, 230);
	oslDrawImageXY(m_img_factory, 435, 230);
	*/
	
	if(!isPaused())
		nextPair->drawXY(61, 30, grid, APPLY_FX_ON_NONE);
	//oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	//sizeY=252
	int stretch=int((252.0f*cc->getTime())/cc->getTimeTotalAvailable());
	if(stretch>0)
	{
		m_img_gauge->stretchY=stretch;
		m_fx_gauge->activate();
		oslDrawImageXY(m_img_gauge, 367, 262-stretch);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	//fire
	m_partgen->draw();
	
	//nukeboarder
	if(m_fx_nukeboarder_end_move->isRunning())
	{
		fx::draw(m_img_nukeboarder, m_fx_nukeboarder_end_move, m_fx_nukeboarder_end_resize);
	}
	else if(m_fx_nukeboarder_start_move->isRunning())
	{
		fx::draw(m_img_nukeboarder, m_fx_nukeboarder_start_move, m_fx_nukeboarder_start_resize);
	}
	//else: don't draw it
	
	//no nuke, more trees
	if(m_nonuke_delayer->isRunning())
	{
		if(m_bool_nonuke)
		{
			oslDrawImage(m_img_nonuke);
		}
		else
		{
			oslDrawImage(m_img_moretrees);
		}
	}
	
	//nukes
	m_gen_nukes->draw();
	
	m_songbanner->draw();
	m_comboDrawer->drawXY(240, 180);
	
	if(isPaused())
	{
		//TODO: vrai menu pause
		//oslDrawString(220, 130, "pause");
		oslDrawFillRect(0,0,480,272,RGBA(255,0,0,128));
		oslDrawImageXY(m_img_pause, 240, 136-14);
	}
	
	
	
	
	//oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
	//oslDrawString(10, 180, m_test.c_str());
}

void LevelSsoup::start_specific()
{
	//enableDebug();
	//debug("start level");
	//enableDebug(false);
	
	oslPlaySound(m_music, 0);
	m_scenario->restart();
	//m_partgen
}

void LevelSsoup::updateNukeBoarder()
{
	//if(m_nukeboarder_delayer->isRunning())
		m_nukeboarder_delayer->update();
	if(m_nukeboarder_delayer->isEvent())
	{
		m_fx_nukeboarder_start_move->start();
		m_fx_nukeboarder_start_resize->start();
	}
	else
	{
		m_fx_nukeboarder_start_move->update();
		m_fx_nukeboarder_start_resize->update();
		m_fx_nukeboarder_end_move->update();
		m_fx_nukeboarder_end_resize->update();
	}
}

void LevelSsoup::updateBG(bool force)
{
	if(force || m_fx_bg->finished())
	{
		float randomspeed=utils::randomf(float(m_bg_speed_min), float(m_bg_speed_max));
		int minloops=int((m_bg_time_total_min/2)/randomspeed)+1;
		int maxloops=int((m_bg_time_total_max/2)/randomspeed);
		int randomloops=utils::random(minloops, maxloops)*2;
		m_bg_fx_time=int(randomspeed);
		m_bg_frame_counter=0;
		
		m_fx_bg->setTimeBackward(m_bg_fx_time);
		m_fx_bg->setTimeForward(m_bg_fx_time);
		m_fx_bg->setNbLoops(randomloops);
		m_fx_bg->restart();
	}
	m_fx_bg->update();
	m_bg_frame_counter++;
	/*if(bgIsBlack())
	{
		m_test="BLACK";
	}
	else
	{
		m_test="RED";
	}*/
}

bool LevelSsoup::bgIsBlack()
{
	int times=int(float(m_bg_frame_counter)/float(m_bg_fx_time));
	bool even=true;
	if(times%2==1)
		even=false;
	
	int rest=m_bg_frame_counter-times*m_bg_fx_time;
	
	if(rest<int(float(m_bg_fx_time)/2.0f))
	{
		return even?true:false;
	}
	//else
	return even?false:true;
}

void LevelSsoup::startFlash(bool super)
{
	if(super) //on change le decor
	{
		int r=utils::random(0,2);
		if(r==m_current_theme)
		{
			r++;
			if(r>2)
				r=0;
		}
		m_current_theme=r;
		//m_anim_nuke->restart();
	}
	else
	{
		m_flash_frame_counter=0;
		int r=utils::random(0,3);
		if(r==0)
			m_flash_color=LEVEL_SSOUP_COLOR_WHITE;
		else if(r==1)
			m_flash_color=LEVEL_SSOUP_COLOR_YELLOW;
		else
		{
			if(bgIsBlack())
			{
				m_flash_color=LEVEL_SSOUP_COLOR_RED;
			}
			else
			{
				m_flash_color=LEVEL_SSOUP_COLOR_BLACK;
			}
		}
		//m_anim_nuke->restart();
	}
}

void LevelSsoup::startNukeMed()
{
	m_nukemed_turning=true;
	updateNukeMed();
	m_fx_nukemed_rot->restart();
	oslImageSetPosition(m_img_nuke_med, 0, 0);
	m_partgen_nukemed->start();
	m_partgen_nukemed->generationOn();
}

void LevelSsoup::updateNukeMed()
{
	//x: 103-376
	int outscreen=70;
	if(m_nukemed_turning)
	{
		if(!m_fx_nukemed_transl->isRunning())
		{
			//on crée une nouvelle translation			
			Pos startpos(m_fx_nukemed_transl->getEndPos());
			int endx=utils::random(-outscreen, 480+outscreen);
			int endy;
			if(endx>103 && endx<376)
			{
				int corner=utils::random(0,1);
				if(corner==0)
				{
					endy=utils::random(-outscreen, 80);
				}
				else
				{
					endy=utils::random(190, 272+outscreen);
				}
			}
			else
			{
				endy=utils::random(-outscreen, 272+outscreen);
			}
			
			//calcul de la distance qu'elle va parcourir
			float dist=pow(float(endx-startpos.x),2.0f)+pow(float(endy-startpos.y),2.0f);
			dist=sqrt(dist);
			
			int time=int(dist*m_nukemed_speed);
			if(time<2)
				time=2;
			
			delete m_fx_nukemed_transl;
			m_fx_nukemed_transl=new TranslationEffect(startpos.x, startpos.y, endx, endy, time, time, true, 1);
			m_fx_nukemed_transl->start();
			
		}
		else
		{
			m_fx_nukemed_transl->update();
		}
		m_fx_nukemed_rot->update();
		
		//non mis a jour si le nukemed ne tourne plus
		m_fx_nukemed_transl->applyOnPositionable(m_partgen_nukemed);
		m_partgen_nukemed->update();
		m_partgen_nukemed->setX(0);
		m_partgen_nukemed->setY(0);
	}
	
	//mis a jour meme si le nukemed ne tourne plus
//	m_fx_nukemed_transl->applyOn(m_partgen_nukemed);
//	m_partgen_nukemed->update();
//	m_partgen_nukemed->setX(0);
//	m_partgen_nukemed->setY(0);
}

void LevelSsoup::stopNukeMed()
{
	m_nukemed_turning=false;
	delete m_fx_nukemed_transl;
	m_fx_nukemed_transl=new TranslationEffect(LEVEL_SSOUP_NUKEMED_POS_X, LEVEL_SSOUP_NUKEMED_POS_Y, LEVEL_SSOUP_NUKEMED_POS_X, LEVEL_SSOUP_NUKEMED_POS_Y, 1, 1, true, 1);
	m_partgen_nukemed->generationOff();
}


//appelé uniquement si le level n'est pas en pause.
void LevelSsoup::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{	
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	
	m_fx_shake1->update();
	m_fx_shake2->update();
	m_fx_shake3->update();
	m_fx_shake4->update();
	
	m_fxmgr_wave->update();
	
	m_fx_click_size->update();
	m_fx_click_alpha->update();
	
		//enableDebug();
	
	//évènements du scénario
	if(m_scenario->hasEvent(1)) //petit clic
	{
		m_fx_click_size->restart();
		m_fx_click_alpha->restart();
		//flash
		startFlash();
	}
	if(m_scenario->hasEvent(2)) //1er toc
	{
		m_addline+=3;
		//super flash (nuclear waste)
		startFlash(true);
	}
	if(m_scenario->hasEvent(3)) //prouit
	{
		m_addline++;
		//flash
		//startFlash();
		m_anim_nuke->restart();
	}
	if(m_scenario->hasEvent(4)) //truc metal qui tombe
	{
		m_addline++;
		//flash chg + flash
		//flash chg=changer les couleurs du decor
		startFlash(true);
	}
	if(m_scenario->hasEvent(5)) //dinguelinguelingueling
	{
		//mini nuclear signs random partout
		m_gen_nukes->generationOn();
		m_gen_nukes->start();
	}
	if(m_scenario->hasEvent(9)) //fin dinguelinguelingueling
	{
		//mini nuclear signs random partout
		m_gen_nukes->generationOff();
	}
	if(m_scenario->hasEvent(6)) //thirteen
	{
		//no nukes
		m_nonuke_delayer->restart();
		m_bool_nonuke=true;
	}
	if(m_scenario->hasEvent(7)) //dreuwa
	{
		//more trees
		m_nonuke_delayer->restart();
		m_bool_nonuke=false;
	}
	if(m_scenario->hasEvent(8)) //wiwiwihouuuu
	{
		m_fxmgr_wave->restart();
	}
	if(m_scenario->hasEvent(10)) //ligne basse ++ (120000)
	{
		m_dropspeed=m_dropspeed_normal;
		m_upspeed=m_upspeed_normal;
	}
	if(m_scenario->hasEvent(11)) //debut feu
	{
		m_dropspeed=m_dropspeed_slow;
		m_upspeed=m_upspeed_slow;
		m_partgen->start();
		m_partgen->generationOn();
		m_nukeboarder_delayer->start();
		m_fx_fire_shake->start();
	}
	if(m_scenario->hasEvent(12)) //fin feu
	{
		m_partgen->generationOff();
		m_upspeed=m_upspeed_normal;
		m_dropspeed=m_dropspeed_normal;
		m_fx_nukeboarder_end_move->start();
		m_fx_nukeboarder_end_resize->start();
		m_fx_nukeboarder_start_move->stop();
		m_fx_nukeboarder_start_resize->stop();
		m_fx_fire_shake->stop();
	}
	if(m_scenario->hasEvent(13)) //wiwou - wiwiwou long jusqu'a la fin de la chanson
	{
		m_dropspeed=m_dropspeed_slow;
		m_upspeed=m_upspeed_slow;
		m_fx_factory_appear->start();
	}
	if(m_scenario->hasEvent(14)) //charabia1
	{
		
	}
//	if(m_scenario->hasEvent(15)) //frafrafrafrafrafrafrafra
//	{
//		
//	}
	if(m_scenario->hasEvent(16)) //prrrrrrrrrrrrrrrrrrrrrrrrr
	{
		m_fx_shake1->restart();
		m_fx_shake2->restart();
		m_fx_shake3->restart();
		m_fx_shake4->restart();
	}
	if(m_scenario->hasEvent(17)) //debut pinpon
	{
		//gros nuclear sign qui tourne et bouge dans l'ecran
		startNukeMed();
	}
	if(m_scenario->hasEvent(18)) //fin pinpon
	{
		//gros nuclear sign qui tourne et bouge dans l'ecran
		stopNukeMed();
	}
	if(m_scenario->hasEvent(19)) //long charabia
	{
		
	}
	if(m_scenario->hasEvent(20)) //charabia2
	{
		
	}
	if(m_scenario->hasEvent(21)) //charabia3
	{
		
	}
	if(m_scenario->hasEvent(22)) //reprise basses
	{
		//if(m_dropspeed==LEVEL_SSOUP_DROPSPEED_SLOW)
		//	m_dropspeed=LEVEL_SSOUP_DROPSPEED;
		//else if(m_dropspeed==LEVEL_SSOUP_DROPSPEED)
			m_dropspeed=m_dropspeed_fast;
		if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
			m_fx_factory_smoke_turn->start();
	}
	if(m_scenario->hasEvent(23)) //reprise percussions
	{
		//if(m_upspeed==LEVEL_SSOUP_UPSPEED_SLOW)
		//	m_upspeed=LEVEL_SSOUP_UPSPEED;
		//else if(m_upspeed==LEVEL_SSOUP_UPSPEED)
			m_upspeed=m_upspeed_fast;
		if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
		{
			m_fx_factory_smoke_color->start();
			m_partgen_rain->start();
			m_partgen_rain->generationOn();
		}
	}
	if(m_scenario->hasEvent(24)) //stop basses
	{
		//if(m_dropspeed==LEVEL_SSOUP_DROPSPEED)
			m_dropspeed=m_dropspeed_slow;
		//else if(m_dropspeed==LEVEL_SSOUP_DROPSPEED_FAST)
		//	m_dropspeed=LEVEL_SSOUP_DROPSPEED;
		if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
			m_fx_factory_smoke_turn->stop();
	}
	if(m_scenario->hasEvent(25)) //arret percussions
	{
		//if(m_dropspeed==LEVEL_SSOUP_UPSPEED)
			m_upspeed=m_upspeed_slow;
		//else if(m_dropspeed==LEVEL_SSOUP_UPSPEED_FAST)
		//	m_dropspeed=LEVEL_SSOUP_UPSPEED;
		if(m_fx_factory_appear->isRunning() || m_fx_factory_appear->finished())
			m_fx_factory_smoke_color->stop();
	}
	if(m_scenario->hasEvent(26)) //reduction bruit feu
	{
		m_dropspeed=m_dropspeed_normal;
		m_upspeed=m_upspeed_normal;
		m_partgen->setThroughput(4, 1);
	}	
	if(m_scenario->hasEvent(27)) //krrrrrrrrrrrrrrrr
	{
		m_fx_rain_shake->start();
		m_fx_rain_shake2->start();
	}
	if(m_scenario->hasEvent(28)) //debut trill trill trill trilllll
	{
		m_anim_nuke->setNbLoops(-1);
		m_anim_nuke->restart();
		m_anim_nuke->setDelayTotal(20.0f);
	}
	if(m_scenario->hasEvent(29)) //fin trill trill trill trilllll
	{
		m_anim_nuke->stop();
	}
	if(m_scenario->hasEvent(30)) //fin zik
	{
		m_fx_rain_shake->stop();
		m_fx_rain_shake2->stop();
		m_partgen_rain->setThroughput(1,25);
	}
	
	
		
		
	//DECORATION
	m_fx_factory_smoke_color->update();
	m_fx_factory_smoke_turn->update();
	m_fx_factory_appear->update();
	updateBG();
	m_fx_gauge->update();
	m_gen_nukes->update();
	m_flash_frame_counter++;
	m_anim_nuke->update();
	updateNukeBoarder();
	updateNukeMed();
	m_nonuke_delayer->update();
	m_fx_fire_shake->update();
	
	m_fx_fire_shake->applyOnPositionable(m_partgen);
	m_partgen->update();
	m_partgen->setX(LEVEL_SSOUP_FIRE_POS_X);
	m_partgen->setY(LEVEL_SSOUP_FIRE_POS_Y);
	
	m_partgen_rain->update();
//	int x=m_partgen_rain->getX();
//	int y=m_partgen_rain->getY();
//	m_fx_rain_shake->applyOn(m_partgen_rain);
	m_fx_rain_shake->update();
	m_fx_rain_shake2->update();
//	m_partgen_rain->setX(x);
//	m_partgen_rain->setY(y);
}

void LevelSsoup::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelSsoup::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}

list<AbstractPiece *> *LevelSsoup::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelSsoup::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelSsoup::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelSsoup::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelSsoup::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
		//m_partgen->start();
		//m_dropspeed=LEVEL_SSOUP_DROPSPEED_SLOW;
		//m_upspeed=LEVEL_SSOUP_UPSPEED_SLOW;
		/*enableDebug();
		debug("drop="+toString(m_dropSpeed)+" up="+toString(m_upSpeed));
		enableDebug(false);*/
		
		/*
		m_fx_shake1->restart();
		m_fx_shake2->restart();
		m_fx_shake3->restart();
		m_fx_shake4->restart();
		*/
		
		//m_fxmgr_wave->reset();
		
//		
//		m_partgen->start();
//		m_nukeboarder_delayer->start();
//		m_fx_fire_shake->start();

		//m_fx_factory_appear->start();
		//m_partgen_rain->start();
		//m_fx_rain_shake->start();
		//m_fx_rain_shake2->start();

		//startNukeMed();
	}
	else if(key==OSL_KEY_SQUARE)
	{
		//m_partgen->setThroughput(7, 1);
		//m_upSpeed=LEVEL_SSOUP_UPSPEED_FAST;
		//m_dropSpeed=LEVEL_SSOUP_DROPSPEED_FAST;
		//enableDebug();
		//debug("carre!");
		//enableDebug(false);
		/*
		m_partgen->start();
		m_partgen->generationSwitchOnOff();
		*/
		//m_fxmgr_wave->restart();
		//startNukeMed();
		
		//stopNukeMed();
	}
}

bool LevelSsoup::finished()
{
	if(m_scenario->getTime()>=LEVEL_SSOUP_DURATION)
	{
		return true;
	}
	return false;
}

float LevelSsoup::getDuration()
{
	return LEVEL_SSOUP_DURATION;
}

