#ifndef LEVELSSOUP_H_
#define LEVELSSOUP_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/framework/time/Repeater.h"
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorPoint.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/Properties.h"
#include <string>


using namespace std;

class LevelSsoup : public AbstractLevel
{
	private:
		
		//fond
		OSL_IMAGE *m_img_bg;
		ProgressiveTintEffect *m_fx_bg;
		int m_bg_speed_min;
		int m_bg_speed_max;
		float m_bg_time_total_min;
		float m_bg_time_total_max;
		int m_bg_frame_counter;
		int m_bg_fx_time;
		
		//usines et leurs nuages
		OSL_IMAGE *m_img_factory;
		OSL_IMAGE *m_img_factory_smoke;
		SerialGlobalEffect *m_fx_factory_smoke_color;
		RotateBackAndForthEffect *m_fx_factory_smoke_turn;
		ProgressiveAlphaEffect *m_fx_factory_appear;
		OSL_IMAGE *m_img_tree_front;
		OSL_IMAGE *m_img_tree_back;
				
		//jauge
		OSL_IMAGE *m_img_gauge;
		ProgressiveTintEffect *m_fx_gauge;
				
		//generateur de nuke signs
		ParticleGeneratorSegment *m_gen_nukes;
		
		//grille
		OSL_IMAGE **m_gridImage; //tableau de pointeurs, taille = 3
		int m_current_theme;
		
		//flash
		int m_flash_duration;
		int m_flash_frame_counter;
		int m_flash_color;
		
		//gros nuke qui tourne
		OSL_IMAGE *m_img_nuke_med;
		RotationEffect *m_fx_nukemed_rot;
		TranslationEffect *m_fx_nukemed_transl;
		float m_nukemed_speed;
		bool m_nukemed_turning;
		void startNukeMed();
		void updateNukeMed();
		void stopNukeMed();
		//et son générateur
		ParticleGeneratorPoint *m_partgen_nukemed;
		
		//anim nuke
		Anim *m_anim_nuke;
		
		//nukeboarder
		OSL_IMAGE *m_img_nukeboarder;
		SerialLocalEffect *m_fx_nukeboarder_start_move;//+shake
		SizeWowEffect *m_fx_nukeboarder_start_resize;
		TranslationEffect *m_fx_nukeboarder_end_move;
		SizeWowEffect *m_fx_nukeboarder_end_resize;
		Repeater *m_nukeboarder_delayer;
		
		void initNukeBoarder(Properties *props);
		void updateNukeBoarder();
		
		//no nuke, more trees
		OSL_IMAGE *m_img_nonuke;
		OSL_IMAGE *m_img_moretrees;
		Repeater *m_nonuke_delayer;
		bool m_bool_nonuke;
		
		//fire
		ParticleGeneratorSegment * m_partgen;
		ShakeEffect *m_fx_fire_shake;
		
		//rain
		ParticleGeneratorSegment *m_partgen_rain;
		ShakeEffect *m_fx_rain_shake;
		SerialLocalEffect *m_fx_rain_shake2;
		
		//pause
		OSL_IMAGE *m_img_pause;
		
		LinePattern *m_linepattern;
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
	
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); //pause, pour par exemple utiliser spliff politics
		void resume_specific();
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		SongBanner *m_songbanner;
		
		ComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed;
		int m_upspeed;
		int m_dropspeed_normal;
		int m_dropspeed_slow;
		int m_dropspeed_fast;
		int m_upspeed_normal;
		int m_upspeed_slow;
		int m_upspeed_fast;
		
		ShakeEffect *m_fx_shake1;
		ShakeEffect *m_fx_shake2;
		ShakeEffect *m_fx_shake3;
		ShakeEffect *m_fx_shake4;
		
		LocalEffectManager *m_fxmgr_wave;
		AbstractLocalEffect ** m_fx_wave; //tableau de pointeurs vers des effets
		
		SizeWowEffect *m_fx_click_size;
		ProgressiveAlphaEffect *m_fx_click_alpha;
		
		AlphaParamsState *m_alphastate;
		
		void updateBG(bool force=false);
		bool bgIsBlack();
		void startFlash(bool super=false);
		
	public:
		LevelSsoup(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelSsoup();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		float getDuration();
		
		void reset();
};




#endif /*LEVELSSOUP_H_*/
