#ifndef LEVELTEST_H_
#define LEVELTEST_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/framework/time/Delayer.h"
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"

using namespace std;

class LevelTest : public AbstractLevel
{
	private:
		int m_dropSpeed;
		int m_upSpeed;
		OSL_IMAGE *m_gridImage;
		OSL_IMAGE *m_bgImage;
		LinePattern *m_linepattern;
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
	
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); //pause, pour par exemple utiliser spliff politics
		void resume_specific();
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		int m_linecounter; //basé la dessus pour dire si on a gagné: des que toutes les lignes ont apparu c gagné.
		
		Delayer *m_delayer; //va nous aider à faire monter 4 lignes au debut, mais progressivement
		
		SongBanner *m_songbanner;
		
		ComboNumberDrawer * m_comboDrawer;
		
		bool m_putJoker;
		
		bool m_addspecialexplosion;
		
	public:
		LevelTest(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelTest();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		float getDuration();
		
		int getNbSpecialExplosions();
		
		void reset();
};


#endif /*LEVELTEST_H_*/
