#include "code/game/levels/impl/turnandburn/LevelTurnAndBurn.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/Properties.h"
#include "code/game/levels/impl/turnandburn/TurnAndBurnMCOContent.h"
#include "code/framework/check/loaders.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomColor.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomProportionMultiTint.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"

using namespace utils;

#define LEVEL_TURNANDBURN_DURATION 176000.0f

string LevelTurnAndBurn::getCode()
{
	return "turnandburn";
}

LevelTurnAndBurn::LevelTurnAndBurn(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{	
	//enableDebug();
	//debug("debut turn and burn");
	
	Properties *props=new Properties("res/levels/turnandburn/turnandburn.properties");
	//debug("properties ok");
	
	m_dropspeedNormal=props->getIntValue("dropspeed.normal");
	m_dropspeedSlow=props->getIntValue("dropspeed.slow");
	m_dropspeedFast=props->getIntValue("dropspeed.fast");
	
	m_gridImage=fwkLoadImageFilePNG("res/levels/turnandburn/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	
	if(difficulty==GAMEDEF_DIFFICULTY_HARD)	
		m_linepattern=new LinePattern("res/levels/turnandburn/turnandburn_hard.pattern");
	else
	{
		m_linepattern=new LinePattern("res/levels/turnandburn/turnandburn.pattern");
		if(difficulty==GAMEDEF_DIFFICULTY_EASY)
		{
			oslWarning("turn and burn should not exist in easy");
		}
	}
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	
	m_comboDrawer=new ComboNumberDrawer();
	
	m_scenario=new EventsScenario("res/levels/turnandburn/turnandburn.scenario");
	m_music=fwkLoadSoundFileMP3("res/levels/turnandburn/turnandburn.mp3", OSL_FMT_STREAM);
	
	m_dropspeed=m_dropspeedSlow;
	
	
	m_addline=2;
	
	m_fx_rotation1=new RotationEffect(props->getIntValue("fx.rotation1.time"), true, props->getIntValue("fx.rotation1.loops"));
	m_fx_rotation2=new SerialLocalEffect(1);
	m_fx_rotation2->addEffect(new WaitLocalEffect(props->getIntValue("fx.rotation2.delay")));
	m_fx_rotation2->addEffect(new RotationEffect(props->getIntValue("fx.rotation2.time"), true, props->getIntValue("fx.rotation2.loops")));
	
	m_burning=false;
	
	
	
	
	
	//generateurs de particules FIRE
	int firelife=props->getIntValue("partgen.fire.life");
	
	RandomAnimationFactory *raf=new RandomAnimationFactory();
	Anim *animptr;
	animptr=new Anim("res/levels/turnandburn/particle.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/turnandburn/particle2.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	animptr=new Anim("res/levels/turnandburn/particle3.png");
	animptr->setReferencePointCenter();
	raf->addAnimation(animptr);
	m_generator_model=new ParticleGeneratorPoint(	
											0, 0, 0, 360, 
											props->getIntValue("partgen.fire.throughput.particles"), 
											props->getIntValue("partgen.fire.throughput.frames"), 
											firelife, props->getIntValue("partgen.fire.distance.max"), firelife, 
											raf);
	m_generator_model->setLocalEffect1(new PFXFRandomTranslation(props->getIntValue("partgen.fire.distance.min"), props->getIntValue("partgen.fire.distance.max")));
	
	int ptmin=props->getIntValue("partgen.fire.particle.color.transition.minimum.time");
	PFXFRandomProportionMultiTint *colorfx=new PFXFRandomProportionMultiTint(firelife, ptmin);
	colorfx->addColor(props->getColorValue("partgen.fire.color1"));
	colorfx->addColor(props->getColorValue("partgen.fire.color2"));
	colorfx->addColor(props->getColorValue("partgen.fire.color3"));
	m_generator_model->setGlobalEffect1(colorfx);
	
	
	
	
	
	
	m_burnDelay=props->getIntValue("burn.delay");
	
	delete props;
	
	m_round=0;
	m_difficulty=difficulty;
	
	m_delayer=NULL;
}

LevelTurnAndBurn::~LevelTurnAndBurn()
{
	oslDeleteImage(m_gridImage);
	
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	
	oslDeleteSound(m_music);
	
	if(m_delayer!=NULL)
	{
		delete m_delayer;
	}
	
	delete m_fx_rotation1;
	delete m_fx_rotation2;
	
	delete m_generator_model;
}


//dessin avec une paire
void LevelTurnAndBurn::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelTurnAndBurn::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelTurnAndBurn::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelTurnAndBurn::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
		
	if(isPaused())
	{
		//TODO: vrai menu pause
		oslDrawString(220, 130, "pause");
	}
	else
	{
		//dessin du contenu de la grille (uniquement pieces non combo)
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
					{
						if(m_fx_rotation2->isRunning())
						{
							if(p->getShape()==PIECE_SHAPE_STAR || p->getShape()==PIECE_SHAPE_CROSS)
								p->drawXY(pos.x,pos.y,m_fx_rotation1);
							else if(p->getShape()==PIECE_SHAPE_TRIANGLE || p->getShape()==PIECE_SHAPE_DIAMOND)
								p->drawXY(pos.x,pos.y,m_fx_rotation2);
							else
								p->drawXY(pos.x,pos.y);
						}
						else
							p->drawXY(pos.x,pos.y);
					}
					
				}
			}	
		}
		
		//dessin des generateurs
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getMCOContent()!=NULL)
				{
					mco->getMCOContent()->draw();
				}
			}
		}
		
		//dessin des pieces combo
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				Pos pos=grid->getContentPosition(i, j);
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL && p->getType()==PIECE_TYPE_COMBO)
				{
					p->drawXY(pos.x,pos.y);
				}
			}
		}
	}
	
	
}

void LevelTurnAndBurn::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	nextPair->drawXY(60, 30, grid, APPLY_FX_ON_NONE);
	oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	
	m_songbanner->draw();
	m_comboDrawer->drawXY(240, 180);
	oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
}

void LevelTurnAndBurn::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

//appelé uniquement si le level n'est pas en pause.
void LevelTurnAndBurn::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_rotation1->update();
	m_fx_rotation2->update();
	
	//faut redemarrer la zik en hard
	if(m_difficulty==GAMEDEF_DIFFICULTY_HARD && m_round==0 && m_scenario->getTime()>176000.0f)
	{
		m_round++;
		oslPlaySound(m_music, 0);
		m_scenario->restart();
	}
		
	//si des lignes montent, les nouvelles pieces combo n'ont pas de mcocontent
	//il faut leur créer (uniquement si burn)
	//2 en 1: mise a jour des mco contents
	
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				//au cas ou les pieces tombent, on remet la position a jour
				Pos pos=grid->getContentPosition(i,j);
				mco->getMCOContent()->setX(pos.x);
				mco->getMCOContent()->setY(pos.y);
				mco->getMCOContent()->update();
			}
			
			
			if(mco->getPiece()!=NULL && mco->getPiece()->getType()==PIECE_TYPE_COMBO)
			{
				//on cree si une piece combo n'en a pas
				if(mco->getMCOContent()==NULL)
				{
					mco->setMCOContent(new TurnAndBurnMCOContent(m_generator_model->clone()));
					mco->getMCOContent()->start();
				}
				
				//on allume quand on est en phase burn
				if(m_burning)
				{
					mco->getMCOContent()->specialEvent(LEVEL_TURNANDBURN_MCO_EVENT_GENERATIONON);
				}
				//on eteint en phase non burn
				else
					mco->getMCOContent()->specialEvent(LEVEL_TURNANDBURN_MCO_EVENT_GENERATIONOFF);
				
			}
			else //ce n'est pas une piece combo, ou alors il n'y a pas de piece
			{
				if(mco->getMCOContent()!=NULL) //ca arrive quand les pieces combo ont explosé
				{
					//on detruit si il n'y a plus de particules
					if(mco->getMCOContent()->specialEvent(LEVEL_TURNANDBURN_MCO_EVENT_GETNBPARTICLES)==0)
					{
						mco->deleteMCOContent();
					}
					//sinon on éteint
					else
					{
						mco->getMCOContent()->specialEvent(LEVEL_TURNANDBURN_MCO_EVENT_GENERATIONOFF);
					}						
				}
			}
			
		}
	}
	
	if(m_delayer!=NULL)
		m_delayer->update();
		
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //turn (ta-taaaa) 1er ta: etoile et croix, 2eme triangle et lozange, 1 seul tour.
	{
		m_fx_rotation1->restart();
		m_fx_rotation2->restart();
	}
	
	if(m_scenario->hasEvent(2)) //debut burn
	{
		m_burning=true;
		m_addline+=3;
		//commencer anim, mettre en route timer: au bout de 1s, changer les pieces vertes, bleues et mauves 
		//aleatoirement en rouge, jaune ou orange (sauf les shielded!)
		
		m_delayer=new Delayer(m_burnDelay, 1);
	}
	
	if(m_delayer!=NULL && m_delayer->get()) //changer les couleurs
	{
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				
				//changer les couleurs
				if(mco->getPiece()!=NULL)
				{
					//Pos *pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL)
					{
						int c=p->getColor();
						if(c==PIECE_COLOR_BLUE || c==PIECE_COLOR_GREEN || c==PIECE_COLOR_PURPLE)
						{
							p->setColor(getRandomColor());
						}
					}
				}
				
			}	
		}
		
		delete m_delayer;
		m_delayer=NULL;
	}
	
	if(m_scenario->hasEvent(3)) //fin burn
	{
		//arreter l'anim
		m_burning=false;
	}
	
	if(m_scenario->hasEvent(4)) //debut de musique de fond
	{
		m_dropspeed=m_dropspeedFast;
	}
	
	if(m_scenario->hasEvent(5)) //fin musique de fond
	{
		m_dropspeed=m_dropspeedNormal;
	}
	
	if(m_scenario->hasEvent(6)) //quasi pu d'instruments
	{
		m_dropspeed=m_dropspeedSlow;
	}
	
}



void LevelTurnAndBurn::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelTurnAndBurn::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}

list<AbstractPiece *> *LevelTurnAndBurn::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelTurnAndBurn::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelTurnAndBurn::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelTurnAndBurn::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return -1;
}

void LevelTurnAndBurn::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		
	}
}

bool LevelTurnAndBurn::finished()
{
	if(m_difficulty==GAMEDEF_DIFFICULTY_HARD && m_scenario->getTime()>=LEVEL_TURNANDBURN_DURATION && m_round==1) //2e round
	{
		return true;
	}
	if(m_difficulty!=GAMEDEF_DIFFICULTY_HARD && m_scenario->getTime()>=LEVEL_TURNANDBURN_DURATION)
	{
		return true;
	}
	return false;
}

float LevelTurnAndBurn::getDuration()
{
	if	(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
		return LEVEL_TURNANDBURN_DURATION*2;
	return LEVEL_TURNANDBURN_DURATION;
}

int LevelTurnAndBurn::getRandomColor() //rouge, orange ou jaune
{
	int r=random(0,2);
	switch(r)
	{
		case 0:
		{
			return PIECE_COLOR_ORANGE;
			break;
		}
		case 1:
		{
			return PIECE_COLOR_RED;
			break;
		}
	}
	
	return PIECE_COLOR_YELLOW;
}








