#include "code/game/levels/impl/voltigeur/LevelVoltigeur.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/Properties.h"
#include "code/framework/check/loaders.h"
using namespace utils;

#define LEVEL_VOLTIGEUR_DURATION 294000.0f

string LevelVoltigeur::getCode()
{
	return "voltigeur";
}

LevelVoltigeur::LevelVoltigeur(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{		
	Properties *props=new Properties("res/levels/voltigeur/voltigeur.properties");
	
	if(difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/voltigeur/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
		m_linepattern=new LinePattern("res/levels/voltigeur/voltigeur_hard.pattern");
		m_scenario=new EventsScenario("res/levels/voltigeur/voltigeur_hard.scenario");
		//m_nbCols=14;
		m_dropspeedNormal=props->getIntValue("hard.dropspeed.normal");
		m_dropSpeedSlow=props->getIntValue("hard.dropspeed.slow");
	}
	else if(difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/voltigeur/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
		m_linepattern=new LinePattern("res/levels/voltigeur/voltigeur_normal.pattern");
		m_scenario=new EventsScenario("res/levels/voltigeur/voltigeur_normal.scenario");
		m_dropspeedNormal=props->getIntValue("normal.dropspeed.normal");
		m_dropSpeedSlow=props->getIntValue("normal.dropspeed.slow");
	}
	else //easy
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/voltigeur/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
		m_linepattern=new LinePattern("res/levels/voltigeur/voltigeur_easy.pattern");
		m_scenario=new EventsScenario("res/levels/voltigeur/voltigeur_easy.scenario");
		m_dropspeedNormal=props->getIntValue("easy.dropspeed.normal");
		m_dropSpeedSlow=props->getIntValue("easy.dropspeed.slow");
	}
	
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	
	m_comboDrawer=new ComboNumberDrawer();
	
	m_music=fwkLoadSoundFileMP3("res/levels/voltigeur/voltigeur.mp3", OSL_FMT_STREAM);
	
	m_dropspeed=m_dropSpeedSlow;
	m_upspeed=-1;
	
	m_addline=0;
	
	m_fx_rot=new RotationEffect(props->getIntValue("fx.blocks.rotation.speed"));
	
	m_fx_move_hor = new SerialLocalEffect();
	m_fx_move_vert = new SerialLocalEffect();
	
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part1.from"), 0, props->getIntValue("fx.movement.hor.part1.to"), 0, props->getIntValue("fx.movement.hor.part1.time"), props->getIntValue("fx.movement.hor.part1.time"), true, 1)); //0 a 10
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part2.from"), 0, props->getIntValue("fx.movement.hor.part2.to"), 0, props->getIntValue("fx.movement.hor.part2.time"), props->getIntValue("fx.movement.hor.part2.time"), true, 1)); //10 a 3
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part3.from"), 0, props->getIntValue("fx.movement.hor.part3.to"), 0, props->getIntValue("fx.movement.hor.part3.time"), props->getIntValue("fx.movement.hor.part3.time"), true, 1)); //3 a -5
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part4.from"), 0, props->getIntValue("fx.movement.hor.part4.to"), 0, props->getIntValue("fx.movement.hor.part4.time"), props->getIntValue("fx.movement.hor.part4.time"), true, 1)); //-5 a 8
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part5.from"), 0, props->getIntValue("fx.movement.hor.part5.to"), 0, props->getIntValue("fx.movement.hor.part5.time"), props->getIntValue("fx.movement.hor.part5.time"), true, 1)); //8 a -10
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part6.from"), 0, props->getIntValue("fx.movement.hor.part6.to"), 0, props->getIntValue("fx.movement.hor.part6.time"), props->getIntValue("fx.movement.hor.part6.time"), true, 1)); //-10 a 2
	m_fx_move_hor->addEffect(new TranslationEffect(props->getIntValue("fx.movement.hor.part7.from"), 0, props->getIntValue("fx.movement.hor.part7.to"), 0, props->getIntValue("fx.movement.hor.part7.time"), props->getIntValue("fx.movement.hor.part7.time"), true, 1)); //2 a 0
	
	m_fx_move_vert->addEffect(new TranslationEffect(0, props->getIntValue("fx.movement.vert.part1.from"), 0, props->getIntValue("fx.movement.vert.part1.to"), props->getIntValue("fx.movement.vert.part1.time"), props->getIntValue("fx.movement.vert.part1.time"), true, 1)); //0 a 5
	m_fx_move_vert->addEffect(new TranslationEffect(0, props->getIntValue("fx.movement.vert.part2.from"), 0, props->getIntValue("fx.movement.vert.part2.to"), props->getIntValue("fx.movement.vert.part2.time"), props->getIntValue("fx.movement.vert.part2.time"), true, 1)); //5 a -2
	m_fx_move_vert->addEffect(new TranslationEffect(0, props->getIntValue("fx.movement.vert.part3.from"), 0, props->getIntValue("fx.movement.vert.part3.to"), props->getIntValue("fx.movement.vert.part3.time"), props->getIntValue("fx.movement.vert.part3.time"), true, 1)); //-2 a 3
	m_fx_move_vert->addEffect(new TranslationEffect(0, props->getIntValue("fx.movement.vert.part4.from"), 0, props->getIntValue("fx.movement.vert.part4.to"), props->getIntValue("fx.movement.vert.part4.time"), props->getIntValue("fx.movement.vert.part4.time"), true, 1)); //3 a -5
	m_fx_move_vert->addEffect(new TranslationEffect(0, props->getIntValue("fx.movement.vert.part5.from"), 0, props->getIntValue("fx.movement.vert.part5.to"), props->getIntValue("fx.movement.vert.part5.time"), props->getIntValue("fx.movement.vert.part5.time"), true, 1)); //-5 a 0
	
	m_fx_move_hor_late=new SerialLocalEffect();
	m_fx_move_vert_late=new SerialLocalEffect();
	m_fx_move_hor_late->addEffect(new WaitLocalEffect(props->getIntValue("fx.movement.delay")));
	m_fx_move_vert_late->addEffect(new WaitLocalEffect(props->getIntValue("fx.movement.delay")));
	m_fx_move_hor_late->addEffect(m_fx_move_hor->clone());
	m_fx_move_vert_late->addEffect(m_fx_move_vert->clone());
	
	m_inversion_num=1;
	
	//shake des pieces avant changement
	m_fx_shake=new ShakeEffect(props->getIntValue("fx.shake.width"), props->getIntValue("fx.shake.height"), props->getIntValue("fx.shake.time"));
	
	//shake lent de la piece
	m_fx_shake_pair=new ShakeEffect(props->getIntValue("fx.shake.pair.width"), props->getIntValue("fx.shake.pair.height"), props->getIntValue("fx.shake.pair.time"));
	
	
	m_fx_wow=new SizeWowEffect(props->getIntValue("fx.wow.shrink.x"), props->getIntValue("fx.wow.grow.x"), props->getIntValue("fx.wow.shrink.y"), props->getIntValue("fx.wow.grow.y"), props->getIntValue("fx.wow.grow.time"), props->getIntValue("fx.wow.shrink.time"), false, 2);
	
	delete props;
	
	m_difficulty=difficulty;
}

LevelVoltigeur::~LevelVoltigeur()
{
	oslDeleteImage(m_gridImage);
	
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	
	oslDeleteSound(m_music);
	
	delete m_fx_rot;
	delete m_fx_move_hor;
	delete m_fx_move_vert;
	delete m_fx_move_hor_late;
	delete m_fx_move_vert_late;
	
	delete m_fx_shake;
	
	delete m_fx_wow;
	
	delete m_fx_shake_pair;
}


//dessin avec une paire
void LevelVoltigeur::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		if(m_fx_shake_pair->isRunning() && m_difficulty==GAMEDEF_DIFFICULTY_HARD)
			currentPair->draw(grid, fx_trans, APPLY_FX_ON_BOTH, m_fx_shake_pair);
		else
			currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelVoltigeur::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelVoltigeur::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelVoltigeur::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(m_difficulty==GAMEDEF_DIFFICULTY_HARD && m_fx_rot->isRunning())
	{
		fx::drawXY(m_gridImage, grid->getPosX(), grid->getPosY(), m_fx_move_hor, m_fx_move_vert);
	}
	else
	{
		oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
	}
	
	
	if(isPaused())
	{
		//TODO: vrai menu pause
		oslDrawString(220, 130, "pause");
	}
	else
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					ShakeEffect *fxshake=NULL; //on va le setter si la piece correspond
					if(m_fx_shake->isRunning()/* && p!=NULL*/)
					{
						if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
						{
							switch(m_inversion_num)
							{
								case 1: //rouge <-> bleu
								{
									if(p->getColor()==PIECE_COLOR_BLUE || p->getColor()==PIECE_COLOR_RED)
										fxshake=m_fx_shake;
									break;
								}
								case 2: //losange <-> triangle + une inversion de couleur aleatoire
								{
									if(p->getShape()==PIECE_SHAPE_DIAMOND || p->getShape()==PIECE_SHAPE_TRIANGLE)
										fxshake=m_fx_shake;
									break;
								}
								case 3: //etoile <-> rond
								{
									if(p->getShape()==PIECE_SHAPE_STAR || p->getShape()==PIECE_SHAPE_CIRCLE)
										fxshake=m_fx_shake;
									break;
								}
								case 4: //bleu <-> orange
								{
									if(p->getColor()==PIECE_COLOR_BLUE || p->getColor()==PIECE_COLOR_ORANGE)
										fxshake=m_fx_shake;
									break;
								}
								case 5: //losange <-> étoile
								{
									if(p->getShape()==PIECE_SHAPE_DIAMOND || p->getShape()==PIECE_SHAPE_STAR)
										fxshake=m_fx_shake;
									break;
								}
								case 6: //vert <-> orange
								{
									if(p->getColor()==PIECE_COLOR_GREEN || p->getColor()==PIECE_COLOR_ORANGE)
										fxshake=m_fx_shake;
									break;
								}
								case 7: //croix <-> carré
								{
									if(p->getShape()==PIECE_SHAPE_CROSS || p->getShape()==PIECE_SHAPE_SQUARE)
										fxshake=m_fx_shake;
									break;
								}
								/*default:
								{
									fxshake=m_fx_shake;
									break;
								}*/
							}
						}	
						
						if(m_inversion_num==8) //dernier ou tout tremble
						{
							fxshake=m_fx_shake;
						}
					}
					
					if(m_fx_rot->isRunning())
					{
						p->drawXY(pos.x, pos.y, fxshake, 
									m_fx_rot, 
									m_difficulty!=GAMEDEF_DIFFICULTY_EASY?m_fx_move_hor_late:NULL, 
									m_difficulty!=GAMEDEF_DIFFICULTY_EASY?m_fx_move_vert_late:NULL, 
									m_fx_wow->isRunning()?m_fx_wow:NULL);					
					}
					else
					{
						p->drawXY(pos.x, pos.y, fxshake, m_fx_wow->isRunning()?m_fx_wow:NULL);
					}
				}
			}	
		}
	}
	
	
}

void LevelVoltigeur::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	nextPair->drawXY(60, 30, grid, APPLY_FX_ON_NONE);
	oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	
	m_songbanner->draw();
	m_comboDrawer->drawXY(200, 180); //TODO: calculer le placement
	oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
}

void LevelVoltigeur::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

//appelé uniquement si le level n'est pas en pause.
void LevelVoltigeur::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_rot->update();
	m_fx_shake->update();
	m_fx_move_hor->update();
	m_fx_move_vert->update();
	m_fx_move_hor_late->update();
	m_fx_move_vert_late->update();
	m_fx_wow->update();
	m_fx_shake_pair->update();
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //Inversion
	{
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					AbstractPiece *p=mco->getPiece();
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
					{
						switch(m_inversion_num)
						{
							case 1: //rouge <-> bleu
							{
								if(p->getColor()==PIECE_COLOR_BLUE)
									p->setColor(PIECE_COLOR_RED);
								else if(p->getColor()==PIECE_COLOR_RED)
									p->setColor(PIECE_COLOR_BLUE);
								break;
							}
							case 2: //losange <-> triangle + une inversion de couleur aleatoire
							{
								if(p->getShape()==PIECE_SHAPE_DIAMOND)
									p->setShape(PIECE_SHAPE_TRIANGLE);
								else if(p->getShape()==PIECE_SHAPE_TRIANGLE)
									p->setShape(PIECE_SHAPE_DIAMOND);
								break;
							}
							case 3: //etoile <-> rond
							{
								if(p->getShape()==PIECE_SHAPE_STAR)
									p->setShape(PIECE_SHAPE_CIRCLE);
								else if(p->getShape()==PIECE_SHAPE_CIRCLE)
									p->setShape(PIECE_SHAPE_STAR);
								break;
							}
							case 4: //bleu <-> orange
							{
								if(p->getColor()==PIECE_COLOR_BLUE)
									p->setColor(PIECE_COLOR_ORANGE);
								else if(p->getColor()==PIECE_COLOR_ORANGE)
									p->setColor(PIECE_COLOR_BLUE);
								break;
							}
							case 5: //losange <-> étoile
							{
								if(p->getShape()==PIECE_SHAPE_DIAMOND)
									p->setShape(PIECE_SHAPE_STAR);
								else if(p->getShape()==PIECE_SHAPE_STAR)
									p->setShape(PIECE_SHAPE_DIAMOND);
								break;
							}
							case 6: //vert <-> orange
							{
								if(p->getColor()==PIECE_COLOR_GREEN)
									p->setColor(PIECE_COLOR_ORANGE);
								else if(p->getColor()==PIECE_COLOR_ORANGE)
									p->setColor(PIECE_COLOR_GREEN);
								break;
							}
							case 7: //croix <-> carré
							{
								if(p->getShape()==PIECE_SHAPE_CROSS)
									p->setShape(PIECE_SHAPE_SQUARE);
								else if(p->getShape()==PIECE_SHAPE_SQUARE)
									p->setShape(PIECE_SHAPE_CROSS);
								break;
							}
						}
					}					
				}
			}	
		}
		
		m_inversion_num++;
		m_fx_shake->stop();
	}
	
	if(m_scenario->hasEvent(2)) //Acceleration de la vitesse
	{
		m_dropspeed=m_dropspeedNormal;
	}
	
	if(m_scenario->hasEvent(3)) //Décélération
	{
		m_dropspeed=m_dropSpeedSlow;
	}
	
	if(m_scenario->hasEvent(4)) //Ajout ligne
	{
		m_addline++;
	}
	
	if(m_scenario->hasEvent(5)) //Ajout de 2 lignes
	{
		m_addline+=2;
	}
	
	if(m_scenario->hasEvent(6)) //Piece bonus
	{
		nextPair->setPiece0(new PieceJoker(nextPair->getPiece(0)->getShape(), nextPair->getPiece(0)->getColor()));
	}
	
	if(m_scenario->hasEvent(7)) //Rotation pieces
	{
		m_fx_rot->restart();
		m_fx_move_hor->restart();
		m_fx_move_vert->restart();
		m_fx_move_hor_late->restart();
		m_fx_move_vert_late->restart();
		m_fx_shake_pair->restart();
	}
	
	if(m_scenario->hasEvent(8)) //Fin rotation
	{
		m_fx_rot->stop();
		m_fx_move_hor->stop();
		m_fx_move_vert->stop();
		m_fx_move_hor_late->stop();
		m_fx_move_vert_late->stop();
		m_fx_shake_pair->stop();
	}
	
	if(m_scenario->hasEvent(9)) //Effet d'aspiration des pieces
	{
		m_fx_wow->restart();
	}
	
	if(m_scenario->hasEvent(10)) //ondulation de toutes les pieces pour cacher l'inversion 1x
	{
		
	}
	
	if(m_scenario->hasEvent(11)) //Vibration finale avec tressaillement et inversion de toutes les pieces
	{
		m_fx_shake->restart();
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					AbstractPiece *p=mco->getPiece();
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
					{
						p->setColor((p->getColor()+1)%6);
						p->setShape((p->getShape()+1)%6);
					}
				}
			}
		}
	}
	
	if(m_scenario->hasEvent(12)) //Vibration préalable des inversés= 2 secondes avant un new loop
	{
		m_fx_shake->restart();
	}
}

void LevelVoltigeur::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelVoltigeur::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}


list<AbstractPiece *> *LevelVoltigeur::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelVoltigeur::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelVoltigeur::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelVoltigeur::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelVoltigeur::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		m_fx_wow->restart();
	}
}

bool LevelVoltigeur::finished()
{
	if(m_scenario->getTime()>=LEVEL_VOLTIGEUR_DURATION)
	{
		return true;
	}
	return false;
}

float LevelVoltigeur::getDuration()
{
	return LEVEL_VOLTIGEUR_DURATION;
}




