#include "code/game/menus/CreditsRoll.h"
#include <oslib/oslib.h>
#include <list>
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/utils.h"
#include "code/framework/Properties.h"

using namespace std;
using namespace utils;


CreditsRoll::CreditsRoll(string filename)
{
	m_ydiff=0.0f;
	
	m_list=new list<OSL_IMAGE *>();
	
	ifstream preffile(filename.c_str());
	
	if(preffile.fail())
		fatalError("CreditsRoll : cannot open file " + filename);
	

	string propertiesStr="";
	string line;
	//on prend la partie properties
	while(getline(preffile,line))
	{
		if(line.find("----")==0)
		{
			break;
		}
		else
		{
			propertiesStr.append(line+"\n");
		}
	}
	
	
	istringstream *iss=new istringstream(propertiesStr, istringstream::in);
	Properties *props=new Properties(iss);
	
	m_speed=props->getFloatValue("speed");
	int emptylineheight=props->getIntValue("line.empty.height");
	int linespacing=props->getIntValue("line.spacing");
	int y=props->getIntValue("text.start.y");
	string fontFolder=props->getStrValue("font.folder");
	string fontName=props->getStrValue("font.name");
		
	BmpFont *font=new BmpFont(fontFolder, fontName, OSL_PF_8888);
	
	//multipliers
	float multBig=props->getFloatValue("text.big.multiplier");
	float multBig2=props->getFloatValue("text.big2.multiplier");
	float multBig3=props->getFloatValue("text.big3.multiplier");
	float multSmall=props->getFloatValue("text.small.multiplier");
	float multSmall2=props->getFloatValue("text.small2.multiplier");
	float multSmall3=props->getFloatValue("text.small3.multiplier");
	m_y_stop=props->getIntValue("text.last.stop.y");
	
	delete props;
	delete iss;
	
	while(getline(preffile,line))
	{
		if(line.find("#")!=0 && line.find("//")!=0) //ne pas prendre les commentaires en compte
		{
			line=deleteAll(line, "\r");
			line=deleteAll(line, "\n");
			line=deleteAll(line, "\t");
			if(line.size()==0)
			{
				//m_list->push_back(NULL);
				y+=emptylineheight;
			}
			else
			{
				float mult=1.0f;				
				if(line.find("[BIG3]")==0)
				{
					mult=multBig3;
					line=deleteAll(line, "[BIG3]");
				}
				else if(line.find("[BIG2]")==0)
				{
					mult=multBig2;
					line=deleteAll(line, "[BIG2]");
				}
				else if(line.find("[BIG]")==0)
				{
					mult=multBig;
					line=deleteAll(line, "[BIG]");
				}
				else if(line.find("[SMALL]")==0)
				{
					mult=multSmall;
					line=deleteAll(line, "[SMALL]");
				}
				else if(line.find("[SMALL2]")==0)
				{
					mult=multSmall2;
					line=deleteAll(line, "[SMALL2]");
				}
				else if(line.find("[SMALL3]")==0)
				{
					mult=multSmall3;
					line=deleteAll(line, "[SMALL3]");
				}
				
				
				BmpText *text=new BmpText(line, font);
				
				OSL_IMAGE *image=text->createSwizzledImageCopy();
				
				delete text;
				
				oslImageSetCenter(image, int(image->sizeX/2.0f), 0);
				
				image->stretchX=int(mult * image->sizeX);
				image->stretchY=int(mult * image->sizeY);
				
			
				image->x=int(240);
				image->y=y;
				y+=int(mult * linespacing);
				
				m_list->push_back(image);
				
				
			}
		}
	}
	
	preffile.close();
	delete font;
	
	m_running=true;
}

CreditsRoll::~CreditsRoll()
{
	list<OSL_IMAGE *>::iterator it;
	for (it=m_list->begin() ; it != m_list->end(); it++ )
	{
		if(*it!=NULL)
		{
			oslDeleteImage(*it);
		}
	}
	delete m_list;
}

void CreditsRoll::setSpeed(float speed)
{
	m_speed=speed;
}

float CreditsRoll::getSpeed()
{
	return m_speed;
}

void CreditsRoll::draw()
{
	list<OSL_IMAGE *>::iterator it;
  	for (it=m_list->begin() ; it != m_list->end(); it++ )
    {
    	OSL_IMAGE *image=*it;
    	if(image!=NULL)
    	{
    		int y=image->y;
    		int x=image->x;
    		int drawY=image->y-int(m_ydiff);
    		if(drawY<272 && drawY>-50)
			{
    			oslDrawImageXY(image, x, drawY);
    			image->y=y;
			}
    	}
    	else
    	{
    		//debug("null image, no drawing");
    	}
    }
}

void CreditsRoll::update()
{
	if(m_running)
	{
		m_ydiff+=m_speed;
		
		OSL_IMAGE *last=m_list->back();
		if(last->y-int(m_ydiff)<=m_y_stop)
		{
			m_running=false;
		}
	}
}		


































		
