#include "code/game/menus/Logo.h"
#include "code/framework/resRepo/abstract/AbstractResRepo.h"
#include "code/framework/resRepo/ResRepoFactory.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include <string>
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/Properties.h"
#include "code/framework/check/loaders.h"

using namespace utils;
using namespace std;

Logo::Logo()
{
	AbstractResRepo *res=ResRepoFactory::newInstance("res/menus/logo");
	Properties *props=res->readPropertiesFile("logo.properties");
	props->setFatalError(true);
	
	m_img_excuse=res->readPngFile("excuse.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_wheel=res->readPngFile("wheel.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_bg=res->readPngFile("bg.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_img_excuse);
	oslImageSetRotCenter(m_img_wheel);
	oslImageSetPosition(m_img_excuse, props->getIntValue("img.excuse.pos.x"), props->getIntValue("img.excuse.pos.y"));
	oslImageSetPosition(m_img_wheel, props->getIntValue("img.wheel.pos.x"), props->getIntValue("img.wheel.pos.y"));

	//images de mega asshole portable
	Anim *anim_particle_star=res->readAnim("particles/star.png");
	Anim *anim_particle_heart=res->readAnim("particles/heart.png");
	Anim *anim_particle_spade=res->readAnim("particles/spade.png");
	Anim *anim_particle_diamond=res->readAnim("particles/diamond.png");
	Anim *anim_particle_club=res->readAnim("particles/club.png");
	
	//images de mega drops et dj drops demo
//	Anim *anim_particle_combo=res->readAnim("particles/combo_num_5.png");
//	Anim *anim_particle_diamond1=res->readAnim("particles/diamond1.png");
	Anim *anim_particle_vinyl=res->readAnim("particles/minivinyl.png");
//	Anim *anim_particle_pink=res->readAnim("particles/pink1.png");
//	Anim *anim_particle_roue=res->readAnim("particles/roue1.png");
	
	//on les centre
	anim_particle_star->setReferencePointCenter();
	anim_particle_heart->setReferencePointCenter();
	anim_particle_spade->setReferencePointCenter();
	anim_particle_diamond->setReferencePointCenter();
	anim_particle_club->setReferencePointCenter();
//	anim_particle_combo->setReferencePointCenter();
//	anim_particle_diamond1->setReferencePointCenter();
	anim_particle_vinyl->setReferencePointCenter();
//	anim_particle_pink->setReferencePointCenter();
//	anim_particle_roue->setReferencePointCenter();
	
	RandomAnimationFactory *raf=new RandomAnimationFactory(anim_particle_star);
	raf->addAnimation(anim_particle_heart);
	raf->addAnimation(anim_particle_spade);
	raf->addAnimation(anim_particle_diamond);
	raf->addAnimation(anim_particle_club);
//	raf->addAnimation(anim_particle_combo);
//	raf->addAnimation(anim_particle_diamond1);
	raf->addAnimation(anim_particle_vinyl);
//	raf->addAnimation(anim_particle_pink);
//	raf->addAnimation(anim_particle_roue);
	
	m_test=props->getIntValue("partgen.direction");
	m_partgen_scale1=props->getIntValue("partgen.scale1");
	m_partgen_scale2=props->getIntValue("partgen.scale2");
	m_partgen=new ParticleGeneratorPoint(	props->getIntValue("partgen.pos.x"), 
											props->getIntValue("partgen.pos.y"), 
											props->getIntValue("partgen.direction"), 
											m_partgen_scale1, 
											props->getIntValue("partgen.particle.throughput.nb"), 
											props->getIntValue("partgen.particle.throughput.time"), 
											props->getIntValue("partgen.particle.move.time"), 
											props->getIntValue("partgen.particle.move.distance"), 
											props->getIntValue("partgen.particle.lifespan"), 
											raf
										);
	m_partgen->setLocalEffect1SimpleTranslation();
	m_partgen->setLocalEffect2(new RotationEffect(props->getIntValue("partgen.particle.rotation.time")));
	m_fx_gravity=new GravityEffect(0.0f, 0.0f, 0.0f, props->getFloatValue("partgen.particle.gravity"));
	m_partgen->setLocalEffect3(m_fx_gravity);

	
	m_fx_wheelrot=new RotationEffect(props->getIntValue("fx.wheel.rotation"));
	
	m_img_fade=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	oslClearImage(m_img_fade, RGB(0,0,0));
	oslSwizzleImage(m_img_fade);
	int fadein=props->getIntValue("fx.fadein");
	int fadeout=props->getIntValue("fx.fadeout");
	m_fx_fadein=new ProgressiveAlphaEffect(fadein, fadein, 255, 0, 1);
	m_fx_fadeout=new ProgressiveAlphaEffect(fadeout, fadeout, 0, 255, 1);
	
	int time_transl=props->getIntValue("fx.excuse.translation.time");
	int time_standup=props->getIntValue("fx.excuse.standup");
	int time_wait=props->getIntValue("fx.excuse.wait");
	int time_rotate=props->getIntValue("fx.excuse.rotation.time");
	int angle=props->getIntValue("fx.excuse.angle");
	int rotation_angle=props->getIntValue("fx.excuse.rotation.angle");
	int transl_from=props->getIntValue("fx.excuse.translation.from");
	
	m_fx_excuse_rotation=new SerialLocalEffect(1);
	m_fx_excuse_rotation->addEffect(new RotateBackAndForthEffect(time_transl, angle, angle, 1)); //il reste penché
	m_fx_excuse_rotation->addEffect(new RotateBackAndForthEffect(time_standup, angle, -rotation_angle, 1)); //il se "redresse"
	WaitLocalEffect *waitfx=new WaitLocalEffect(time_wait, true);
	waitfx->setDirection(-rotation_angle);
	m_fx_excuse_rotation->addEffect(waitfx); //il attend
	m_fx_excuse_rotation->addEffect(new RotateBackAndForthEffect(time_rotate, -rotation_angle, rotation_angle)); //il remue
	m_fx_excuse_move=new TranslationEffect(transl_from, 0, 0, 0, time_transl, time_transl, true, 1);
	
	

	
	m_delayer_generator=new Delayer(time_transl+time_standup+time_wait, 1);
	
	int daaafadein=props->getIntValue("fx.daaa.fadein");
	m_fx_daaa_fadein=new ProgressiveAlphaEffect(daaafadein, daaafadein, 0, 255, 1);
	
	m_fx_daaa_transl=new SerialLocalEffect *[12]; //daaapresents
	int daaatranslwait=props->getIntValue("fx.daaa.transl.wait");
	int daaatransldelay=props->getIntValue("fx.daaa.transl.delay");
	int daaatranslheight=props->getIntValue("fx.daaa.transl.height");
	int daaatransltime=props->getIntValue("fx.daaa.transl.time");
	for(int i=0;i<12;i++)
	{
		m_fx_daaa_transl[i]=new SerialLocalEffect(1);
		m_fx_daaa_transl[i]->addEffect(new WaitLocalEffect(daaatranslwait+i*daaatransldelay));
		m_fx_daaa_transl[i]->addEffect(new ShakeEffect(0, daaatranslheight, daaatransltime)); //celui la il boucle
	}
	
	//image daaa presents:
	int imgdaaax=props->getIntValue("img.daaa.pos.x");
	int imgdaaay=props->getIntValue("img.daaa.pos.y");
	m_img_daaa=new OSL_IMAGE*[12];
	m_img_daaa[0]=res->readPngFile("daaa/d.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[0], imgdaaax, imgdaaay);
	m_img_daaa[1]=res->readPngFile("daaa/a.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[1], imgdaaax+29, imgdaaay);
	m_img_daaa[2]=res->readPngFile("daaa/a.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[2], imgdaaax+57, imgdaaay);
	m_img_daaa[3]=res->readPngFile("daaa/a.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[3], imgdaaax+85, imgdaaay);
	m_img_daaa[4]=res->readPngFile("daaa/p.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[4], imgdaaax+122, imgdaaay);
	m_img_daaa[5]=res->readPngFile("daaa/r.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[5], imgdaaax+157, imgdaaay);
	m_img_daaa[6]=res->readPngFile("daaa/e.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[6], imgdaaax+177, imgdaaay);
	m_img_daaa[7]=res->readPngFile("daaa/s.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[7], imgdaaax+199, imgdaaay);
	m_img_daaa[8]=res->readPngFile("daaa/e.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[8], imgdaaax+221, imgdaaay);
	m_img_daaa[9]=res->readPngFile("daaa/n.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[9], imgdaaax+244, imgdaaay);
	m_img_daaa[10]=res->readPngFile("daaa/t.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[10], imgdaaax+275, imgdaaay);
	m_img_daaa[11]=res->readPngFile("daaa/s.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_daaa[11], imgdaaax+294, imgdaaay);
	
	for(int i=0;i<12;i++)
	{
		oslImageSetCenter(m_img_daaa[i], 0, int(m_img_daaa[i]->sizeY/2));
	}
	
	//m_music=res->readBgmFile("kleinjiga.bgm");
	//m_music=oslLoadSoundFileMP3("res/logo.rmx/kleinjiga.mp3", OSL_FMT_STREAM);
	m_music=fwkLoadSoundFileMP3("res/menus/logo/kleinjiga.mp3", OSL_FMT_STREAM);
	m_delayer_music=new Delayer(props->getIntValue("music.delay"), 1);
	
	m_timer_end=new Timer();
	m_music_length=props->getIntValue("music.end");
	
	delete props;
	delete res;	
	
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_RIGHT|OSL_KEYMASK_DOWN|OSL_KEYMASK_LEFT|OSL_KEYMASK_TRIANGLE|OSL_KEYMASK_SQUARE|OSL_KEYMASK_L|OSL_KEYMASK_R,10,10);
}

Logo::~Logo()
{
	oslDeleteImage(m_img_excuse);
	oslDeleteImage(m_img_wheel);
	oslDeleteImage(m_img_bg);
	oslDeleteImage(m_img_fade);
	
	for(int i=0;i<12;i++)
	{
		oslDeleteImage(m_img_daaa[i]);
		delete m_fx_daaa_transl[i];
	}
	delete[] m_fx_daaa_transl;
	delete[] m_img_daaa;
		
	delete m_partgen;
	delete m_fx_wheelrot;
	delete m_fx_fadein;
	delete m_fx_fadeout;
	delete m_fx_excuse_rotation;
	delete m_fx_excuse_move;
	delete m_delayer_generator;
	delete m_fx_daaa_fadein;
	
	oslDeleteSound(m_music);
	delete m_delayer_music;
	delete m_timer_end;
}
		
void Logo::go()
{
	
	debug("logo go");
	
	//démarrage de tout
	m_fx_wheelrot->start();
	m_partgen->generationOn();
	m_fx_fadein->start();
	m_fx_excuse_rotation->start();
	m_fx_excuse_move->start();
	//m_delayer_generator->start(); //start tout seul celui la
	m_fx_daaa_fadein->start();
	for(int i=0;i<12;i++)
	{
		m_fx_daaa_transl[i]->start();
	}
	
	
	bool skip=false; //skip phase de dessin si trop de calculs
	while(!m_fx_fadeout->finished()) //on quitte des que le fadeout a fini
	{
		//oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		keys();
				
		//phase d'update
		m_fx_wheelrot->update();
		m_fx_fadein->update();
		if(m_fx_fadeout->isRunning())
		{
			m_partgen->generationOff();
			m_fx_fadeout->update();
		}
		m_partgen->update();
		m_fx_excuse_rotation->update();
		m_fx_excuse_move->update();
		m_delayer_generator->update();
		if(m_delayer_generator->get())
		{
			m_partgen->start();
		}
		m_fx_daaa_fadein->update();
		for(int i=0;i<12;i++)
		{
			m_fx_daaa_transl[i]->update();
		}
		m_delayer_music->update();
		if(m_delayer_music->get())
		{
			oslPlaySound(m_music, 0);
			m_timer_end->start();
		}
		if(m_timer_end->getTime()>m_music_length)
		{
			m_fx_fadeout->start();
		}
		
		
		//phase de dessin
		if(!skip)
		{
			oslStartDrawing();
			drawScene();
			//oslBenchmarkTest(OSL_BENCH_END); //TODO: peut planter si frameskip??
			//oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
	    skip = oslSyncFrame();
	}
	//petit dessin noir pour eviter le clignotement
	for(int i=0;i<5;i++)
	{
		oslStartDrawing();
		oslClearScreen(RGB(0,0,0));
		oslEndDrawing();
		oslEndFrame();
		oslSyncFrame();
	}
}

void Logo::drawScene()
{
	//bg + roue
	oslDrawImage(m_img_bg);
	//fx::draw(m_img_wheel, m_fx_wheelrot);
	
	m_partgen->draw();
	
	if(m_fx_daaa_fadein->isRunning())
	{
		m_fx_daaa_fadein->activate();
		for(int i=0;i<12;i++)
		{
			oslDrawImage(m_img_daaa[i]);
		}
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}	
	else
	{
		for(int i=0;i<12;i++)
		{
			fx::draw(m_img_daaa[i], m_fx_daaa_transl[i]);
		}
	}
	
	//excuse
	fx::draw(m_img_excuse, m_fx_excuse_rotation, m_fx_excuse_move);
	
	
	
	//fade in / out
	if(m_fx_fadein->isRunning())
	{
		m_fx_fadein->activate();
		oslDrawImage(m_img_fade);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	if(m_fx_fadeout->isRunning() || m_fx_fadeout->finished())
	{
		m_fx_fadeout->activate();
		oslDrawImage(m_img_fade);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	//oslDrawString(10,260,toString(m_test).c_str());
	//oslDrawString(60,260,toString(m_partgen->getCurrentNbParticles()).c_str());
}

void Logo::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start || osl_pad.pressed.cross || osl_pad.pressed.circle)  
	{
		m_fx_fadeout->start();
	}


	//vitesse des particules
	if (osl_pad.pressed.up)
	{
		int t=m_partgen->getPTranslTime();
		t+=5;
		m_partgen->setPTranslTime(t);
		m_partgen->setPLifespan(t);
	}
	else if(osl_pad.pressed.down)
	{
		int t=m_partgen->getPTranslTime();
		if(t>=10)
		{
			t-=5;
			m_partgen->setPTranslTime(t);
			m_partgen->setPLifespan(t);
		}
	}
	//distance de jet des particules
	else if(osl_pad.pressed.left)
	{
		int d=m_partgen->getPDistance();
		if(d>=30)
		{
			d-=20;
			m_partgen->setPDistance(d);
		}
	}
	else if(osl_pad.pressed.right)
	{
		int d=m_partgen->getPDistance();
		d+=20;
		m_partgen->setPDistance(d);
	}
	
	//débit des particules
	if (osl_pad.pressed.R)
	{
		int thrp_p=m_partgen->getThroughputNbParticles();
		int thrp_t=m_partgen->getThroughputFrameTime();
		thrp_p++;
		m_partgen->setThroughput(thrp_p, thrp_t);
	}
	else if (osl_pad.pressed.L)
	{
		int thrp_p=m_partgen->getThroughputNbParticles();
		int thrp_t=m_partgen->getThroughputFrameTime();
		if(thrp_p>1)
			thrp_p--;
		m_partgen->setThroughput(thrp_p, thrp_t);	
	}
	
	//gravité
	if(osl_pad.pressed.triangle)
	{
		m_fx_gravity->setAccelerationY(m_fx_gravity->getAccelerationY()+0.01f);
	}
	else if(osl_pad.pressed.square)
	{
		m_fx_gravity->setAccelerationY(m_fx_gravity->getAccelerationY()-0.01f);
	}
	
	//scale
	if(osl_pad.pressed.select)
	{
		if(m_partgen->getScale()==m_partgen_scale1)
		{
			m_partgen->setScale(m_partgen_scale2);
		}
		else
		{
			m_partgen->setScale(m_partgen_scale1);
		}
	}
	
	if(osl_pad.analogX>=80 || osl_pad.analogX<-80 || osl_pad.analogY>80 || osl_pad.analogY<-80)
	{
		m_test=oslAnalogGetAngle(osl_pad.analogX, osl_pad.analogY);
		m_partgen->setOrientation(m_test);
	}
}





















