#include "code/game/menus/SoloGame.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include <string>
#include "code/game/menus/MenusDefines.h"
#include "code/framework/Pos.h"
#include "code/game/FlagMatrix.h"
#include "code/game/gamedef.h"
#include "code/game/levels/LevelFactory.h"
#include "code/game/score/Scorer.h"
#include "code/game/grid/GridEventResult.h"
//#include "code/game/profile/Profile.h"
//#include "code/game/profile/ProfileManager.h"
#include "code/game/menus/qj/dialogs/YouLooseDialog.h"
#include "code/game/menus/qj/dialogs/YouWinDialog.h"
#include "code/game/menus/qj/dialogs/LevelRecordsDialog.h"
#include "code/game/saves/GameSave.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace std;
using namespace utils;


/*
 * tests effectués:
 * enlever les dialogues : /
 * enlever les sauvegardes : /
 * 
 */


SoloGame::SoloGame(string level, int difficulty)
{
	//malloc_trim(0);
	
	PFL::logConstructionStart("SoloGame");
	m_difficulty=difficulty;
	m_levelcode=level;
	
	m_level=LevelFactory::newLevel(m_levelcode, m_difficulty, 1);	//memory lost: 0	 
	
	//memory lost: 0
	m_grid=new GameGrid(m_level->getNbCols(), m_level->getNbLines(), GAMEDEF_PIECE_WIDTH, GAMEDEF_PIECE_HEIGHT); //centrée par défaut
	
	
	m_currentPair=m_level->getNewPair();
	orientatePair(m_currentPair);
	m_pieceConnected=NULL;
	m_pieceUnconnected=NULL;
	m_nextPair=m_level->getNewPair();   //new Pair(m_grid);
	orientatePair(m_nextPair);
	m_skip=false;
	m_action=MENU_NOACTION;
	
	m_frameCounter4Drop=0;
	m_frameCounter4AddLine=0;
	m_frameCounter4MinimumWait=0;
	
	m_fast_drop_on=false;
	
	m_phase=PHASE_NORMAL;
	//m_pieceUnconnectedPos=Pos();
	//m_pieceConnectedPos=Pos();
	m_frameCounter4PieceUnconnectedDrop=0;
	
	m_fx_transl=NULL;
	recreateTranslationEffect();
	m_fx_transl->start();
	
	m_comboc=new ComboCounter();
	
	m_scorer=new Scorer(m_level->getCode(), difficulty);
	
	m_timer=new Timer();
	
	oslSetTextColor(RGB(255,255,255));  
	
	PFL::logConstructionEnd("SoloGame");
}

SoloGame::~SoloGame()
{
	PFL::logDestructionStart("SoloGame");
	
	if(m_level!=NULL)
		delete m_level;
	
	if(m_grid!=NULL)
		delete m_grid;
	
	if(m_currentPair!=NULL)
		delete m_currentPair;
		
	if(m_nextPair!=NULL)
		delete m_nextPair;
		
	if(m_pieceConnected!=NULL)
		delete m_pieceConnected;
		
	if(m_pieceUnconnected!=NULL)
		delete m_pieceUnconnected;
	
		
	if(m_fx_transl!=NULL)
		delete m_fx_transl;
		
	delete m_comboc;
	
	delete m_scorer;
	
	delete m_timer;
	
	PFL::logDestructionEnd("SoloGame");
}

bool SoloGame::cellIsEmptyOrPieceIsNull(AbstractPiece *piece, Pos pos)
{
	return cellIsEmptyOrPieceIsNull(piece, pos.x, pos.y);
}

bool SoloGame::cellIsEmptyOrPieceIsNull(AbstractPiece *piece, int col, int line)
{
	if(piece->getType()==PIECE_TYPE_NULL)
		return true;
	
	return m_grid->cellIsEmpty(col, line);
}

void SoloGame::recreateTranslationEffect()
{
	if(m_fx_transl!=NULL)
		delete m_fx_transl;
	m_fx_transl=new SerialLocalEffect(1);
	
	int timeTotal;
	int timeWait;
	int timeTransl;
	
	if(m_phase==PHASE_NORMAL)
	{
		if(m_currentPair==NULL)
			fatalError("SoloGame::recreateTranslationEffect : m_currentPair ne doit pas etre NULL en PHASE_NORMAL");
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		//la il faut l'animation, la piece n'est pas bloquée
		//if(m_grid->cellIsEmpty(cur0.x, cur0.y+1) && m_grid->cellIsEmpty(cur1.x, cur1.y+1))
		if(		cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(0), cur0.x, cur0.y+1)
			&&	cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(1), cur1.x, cur1.y+1))
		{
			if(!m_fast_drop_on)
				timeTotal=m_level->getDropSpeed();
			else
				timeTotal=m_level->getFastDropSpeed();
			timeWait=int(timeTotal/2.0f);
			timeTransl=timeTotal-timeWait;
			m_fx_transl->addEffect(new WaitLocalEffect((unsigned int)(timeWait)));
			m_fx_transl->addEffect(new TranslationEffect(0, 0, 0, m_grid->getCellHeight(), timeTransl, timeTransl, true, 1));
		}
		//la il faut pas d'animation, car la paire est bloquée
		else
		{
			m_fx_transl->addEffect(new WaitLocalEffect((unsigned int)(1)));
		}
	}
	else if(m_phase==PHASE_WAIT_SECOND_CONNECTION)
	{
		timeTotal=m_level->getPieceUnconnectedDropSpeed();
		//if(m_grid->cellIsEmpty(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y+1))//la place est libre pour descendre
		if(cellIsEmptyOrPieceIsNull(m_pieceUnconnected, m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y+1))
		{
			m_fx_transl->addEffect(new TranslationEffect(0, 0, 0, m_grid->getCellHeight(), timeTotal, timeTotal, true, 1));
		}
		else//la il faut pas d'animation, car la piece est bloquée
		{
			m_fx_transl->addEffect(new WaitLocalEffect((unsigned int)(1)));
		}
	}
}

int SoloGame::goDebug()
{
	LevelRecordsDialog *lrdialog=new LevelRecordsDialog(m_levelcode, m_difficulty);
	lrdialog->go();
	delete lrdialog;
	
	m_level->start();
	m_timer->start();
	
	return MENU_MAIN;
}

int SoloGame::go()
{
	//debug("go");
	
	/*
	LevelRecordsMenu *lrmenu=new LevelRecordsMenu(m_level);
	lrmenu->go();
	delete lrmenu;
	*/
	

	LevelRecordsDialog *lrdialog=new LevelRecordsDialog(m_levelcode, m_difficulty);
	lrdialog->go();
	delete lrdialog;
	
	m_level->start();
	m_timer->start();
	while(m_action==MENU_NOACTION)
	{
		//oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		
		//phase d'update
		int gameState=update();
		if(m_level->isGameOver())
			gameState=GAME_OVER;
		
		if(gameState==GAME_OVER || gameState==GAME_WON)
		{
			m_timer->stop();

			//un dernier update (certains levels en ont besoin pour l'affichage, ie tetris)
			m_level->update(m_comboc, m_grid, m_nextPair, m_currentPair);
			
			for(int i=0;i<100;i++) //1+ sec pour se rendre compte
			{
				oslStartDrawing();
				drawScene();
				oslEndDrawing();
				oslEndFrame();
				oslSyncFrame();
			}
						
			int quitorretry=onGameEnd(gameState);
			if(quitorretry==GAME_RETRY)
			{
				restartLevel();
				//return MENU_REDOTHESAME;
			}
			else //GAME_QUIT
			{
				m_action=MENU_MAIN;
			}
		}
		/*else if(gameState==GAME_WON)
		{
			//un dernier update (certains levels en ont besoin pour l'affichage, ie tetris)
			m_level->update(m_comboc, m_grid, m_nextPair, m_currentPair);
			
			for(int i=0;i<100;i++) //1+ sec pour se rendre compte
			{
				oslStartDrawing();
				drawScene();
				oslEndDrawing();
				oslEndFrame();
				oslSyncFrame();
			}
			
			m_action=MENU_MAIN;	
			int quitorretry=onGameEnd(GAME_WON);
			if(quitorretry==GAME_RETRY)
				restartLevel();
			else //GAME_QUIT
				m_action=MENU_MAIN;
		}*/
		else
		{	
			if (!m_skip)
			{
				//phase de dessin
				oslStartDrawing();
				drawScene();
				//oslBenchmarkTest(OSL_BENCH_END);
				//oslSysBenchmarkDisplay();
				oslEndDrawing();
			}
			
			//End of a frame
	        oslEndFrame();
			//Synchronize to ensure the game runs at 60 fps
	        m_skip = oslSyncFrame();
		}
	}
	//il a choisi une option
	//enableDebug();
	//debug("fin boucle soloGame");
	//enableDebug(false);	
	
	return MENU_MAIN; //TODO: renvoyer le bon endroit (possible d'envoyer autre chose que main??)
}

int SoloGame::update()
{
	if(m_level->isPaused())
	{
		m_level->update(m_comboc, m_grid, m_nextPair, m_currentPair, m_fx_transl); //va appeler update during pause sur les fils
		return GAME_RUNNING;
	}
	
	m_fx_transl->update();
	m_frameCounter4AddLine++;
	m_nextPair->update();
	if(m_currentPair!=NULL)
	{
		m_currentPair->update();
	}
	if(m_pieceConnected!=NULL)
	{
		m_pieceConnected->update();
	}
	if(m_pieceUnconnected!=NULL)
	{
		m_pieceUnconnected->update();
	}
		
	//ajout d'une ligne d'en bas
	if(m_level->getAddLineEvent() || ((m_level->getUpSpeed()!=-1 && m_frameCounter4AddLine>m_level->getUpSpeed()) && !m_grid->isEvent()))
	{
		m_frameCounter4AddLine=0;
		//if(m_linepattern!=NULL)
		//{
			list<AbstractPiece *> *pline=m_level->getNextLine(); //la prochaine ligne a ajouter
			if(pline!=NULL) //si c NULL, c qu'on ne veut pas en ajouter
			{
				bool ok=m_grid->addLine(pline); //false=gameover
				if(!ok)
				{
					//oslDebug("GAME OVER");
					return GAME_OVER;
				}
				
				//on fait remonter la piece actuelle si elle superpose une piece dans la grille
				if(m_phase==PHASE_NORMAL)
				{
					int x0,y0,x1,y1;
					x0=m_currentPair->getGridPos(0).x;
					y0=m_currentPair->getGridPos(0).y;
					x1=m_currentPair->getGridPos(1).x;
					y1=m_currentPair->getGridPos(1).y;
					
	//				if(	(y0>=0 && y1>=0) && 
	//					(!m_grid->getAt(x0, y0)->isEmpty() || !m_grid->getAt(x1, y1)->isEmpty())
	//				  )
	//				{
	//					m_currentPair->move(DIRECTION_UP);
	//				}
					if(	(y0>=0 && !m_grid->getAt(x0, y0)->isEmpty()) || 
						(y1>=0 && !m_grid->getAt(x1, y1)->isEmpty()) )
					{
						m_currentPair->move(DIRECTION_UP);
					}
					
				}
				else if(m_phase==PHASE_WAIT_SECOND_CONNECTION)
				{
					if(		(m_pieceUnconnectedPos.y==0 && m_pieceUnconnected->getType()!=PIECE_TYPE_NULL)
					 	 || (m_pieceConnectedPos.y==0 && m_pieceConnected->getType()!=PIECE_TYPE_NULL))
					{
						//oslDebug("GAME OVER");
						return GAME_OVER;
					}
					m_pieceUnconnectedPos.y--;
					m_pieceConnectedPos.y--;
				}
			}
		//}
	}
	
	//PHASE NORMALE DE DESCENTE D'UNE PAIRE
	if(m_phase==PHASE_NORMAL)
	{
		//score
		if(m_fast_drop_on)
		{
			m_scorer->notifyFastdropFrame();
		}
		
		//on update la grille uniquement ici, comme ca pendant les autres phases les animations et
		//les degringolages sont stoppées momentanément
		m_grid->update();
		//debug("update m_grid");

		FlagMatrix * pack=m_grid->cleanAndCollapse();
				
		m_frameCounter4Drop++;
		//la piece tombe, que se passe t il?
		if(pairShouldGoDown()) //faut qu'elle descende d'une case
		{
			//verifier dans la grille que la place est libre
			//faire tomber si c bon
			//sinon attendre un peu et fusionner dans la grille
			//puis changer la paire courante (et la prochaine)
			Pos cur0=m_currentPair->getGridPos(0);
			Pos cur1=m_currentPair->getGridPos(1);
			
			//elle est superposée à des pieces de la grille, game over
			//if(!m_grid->cellIsEmpty(cur0.x, cur0.y) || !m_grid->cellIsEmpty(cur1.x, cur1.y))
			if(!cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(0), cur0) || !cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(1), cur1))
			{
				delete pack;
				return GAME_OVER;
			}
			
			//la place est libre
			//if(m_grid->cellIsEmpty(cur0.x, cur0.y+1) && m_grid->cellIsEmpty(cur1.x, cur1.y+1))
			if(		cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(0), cur0.x, cur0.y+1)
				&&	cellIsEmptyOrPieceIsNull(m_currentPair->getPiece(1), cur1.x, cur1.y+1))
			{
				m_currentPair->move(DIRECTION_DOWN);
				
				recreateTranslationEffect();
				m_fx_transl->start();
			}
			else //connection, une piece non null a connecté
			{
				m_phase=PHASE_WAIT_SECOND_CONNECTION;
				
				//on detache
				if(m_grid->cellIsEmpty(cur0.x, cur0.y+1))
				{
					m_pieceConnected=m_currentPair->detach(1);
					m_pieceUnconnected=m_currentPair->detach(0);
					m_pieceUnconnectedPos=cur0;
					m_pieceConnectedPos=cur1;
				}
				else
				{	
					m_pieceConnected=m_currentPair->detach(0);
					m_pieceUnconnected=m_currentPair->detach(1);
					m_pieceUnconnectedPos=cur1;
					m_pieceConnectedPos=cur0;
				}
				
				if(m_currentPair->getOrientation()==ORIENTATION_EAST || m_currentPair->getOrientation()==ORIENTATION_WEST)
				{
					recreateTranslationEffect();
					m_fx_transl->start();
				}
				else
				{
					//la piece est verticale, c a d que la piece unconnected va se connecter de suite
					m_fx_transl->reset();
					m_fx_transl->stop();
				}
				
				//on detruit
				delete m_currentPair;
				m_currentPair=NULL;
			}
			
			m_frameCounter4Drop=0;
		}
		//uniquement pendant cette phase car de toute facon l'update se fait que pendant cette phase
		//on detruit les pieces qui ont fini d'exploser
		//m_grid->clean();
		//on deplace les pieces qui ont dégringolé
		//FlagMatrix * pack=m_grid->collapse(); //on récupère les positions des pieces ayant atterri
		//debug("clean and collapse");
		
		//et on enchaine les combos
		
		GridEventResult ger=m_grid->startExplosions(pack); 
		delete pack;
		
		//le level peut decider de faire exploser des pieces
		GridEventResult ger2;
		int nbspe=m_level->getNbSpecialExplosions();
		if(nbspe>0)
		{
			ger2.addExplSpecial(nbspe);		
		}
				
		//recherche s'il se passe qqch dans la grille ou pas		
		m_comboc->update();
		if(ger.containsExpl() || ger2.containsExpl())
		{
			if(ger.containsExpl())
			{
				m_comboc->notifyNewExplosions(true); //quand on enchaine avec un bloc c une combo forcément!
			}
			if(ger2.containsExpl())
			{
				m_comboc->notifyNewExplosions(); //là c'est pas forcément une combo
			}
			if(m_comboc->getCombos()>1)
			{
				int nb=m_grid->notifyCombo(); //pour diminuer le nombre sur les pieces combo
				m_scorer->notifyComboDecrements(nb, m_comboc->getCombos());
				
			}
			
			//score
			m_scorer->notifyExplosions(ger, m_comboc->getCombos());
			m_scorer->notifyExplosions(ger2, m_comboc->getCombos());
		}
		else //pas d'explosion
		{
			if(m_grid->isEvent()) //est ce qu'une piece explose ou tombe?
			{
				m_comboc->notifyEvents();
			}
			else //il ne se passe rien
			{
				if(m_comboc->notifyNoEvents()) //true s'il se passait qqch mais que c fini maintenant
				{
					int nbExplCombo=m_grid->explodeComboPieces();
					if(nbExplCombo>0) //au - une piece combo a commencé à exploser
					{
						m_comboc->notifyNewExplosions(); //finalement il se passe qqch alors 
						if(m_comboc->getCombos()>1) //ca doit etre forcement le cas non?
						{
							int nb=m_grid->notifyCombo(); //pour diminuer le nombre sur les pieces combo
							m_scorer->notifyComboDecrements(nb, m_comboc->getCombos());
						}
												
						
						//TODO: est ce que ca doit compter dans les combos ou un simple notifyEvents suffit?
						
						//score
						m_scorer->notifyComboExplosions(nbExplCombo, m_comboc->getCombos());
					}
				}
			}
		}
	}
	
	//PHASE OU LA PIECE SEULE TOMBE
	else if(m_phase==PHASE_WAIT_SECOND_CONNECTION)
	{
		m_frameCounter4PieceUnconnectedDrop++;
		m_frameCounter4MinimumWait++;
		if(m_frameCounter4PieceUnconnectedDrop>=m_level->getPieceUnconnectedDropSpeed()) 
		{//la piece seule doit descendre d'une case
			if(pieceUnconnectedShouldGoDown()) //elle peut descendre, c'est libre
			{
				m_pieceUnconnectedPos.y++;
			}			
			else //connection de la piece seule
			{
				//on verifie que les pieces sont dans la grille, sinon game over
				if(isGameOver())
				{
					//oslDebug("GAME OVER");	
					return GAME_OVER;
				}
				
				//on regarde si on a fini le level
				if(m_level->finished())
				{
					//GAGNÉ!
					return GAME_WON;
				}
				
				//on fixe les pieces dans la grille
				bool pieceConnectedIsNull=true;
				bool pieceUnconnectedIsNull=true;
				if(m_pieceConnected->getType()!=PIECE_TYPE_NULL)
				{
					pieceConnectedIsNull=false;
					m_grid->getAt(m_pieceConnectedPos.x, m_pieceConnectedPos.y)->setPiece(m_pieceConnected);
					//on les fait dégringoler si ca dégringole sous eux ou si y a plus rien (TODO:utile??)
					if(	m_pieceConnectedPos.y+1<m_grid->getNbLines() && 
						(	m_grid->getAt(m_pieceConnectedPos.x, m_pieceConnectedPos.y+1)->isFalling() ||
							m_grid->getAt(m_pieceConnectedPos.x, m_pieceConnectedPos.y+1)->isEmpty()))
					{
						m_grid->getAt(m_pieceConnectedPos.x, m_pieceConnectedPos.y)->startFalling();
					}
					else
					{
						m_grid->getAt(m_pieceConnectedPos.x, m_pieceConnectedPos.y)->dontFall();
					}
				}
				else //m_pieceConnected->getType()==PIECE_TYPE_NULL
					delete m_pieceConnected;
				//dans les 2 cas:
				m_pieceConnected=NULL;
				
				//debug("piece unconnected");
				if(m_pieceUnconnected->getType()!=PIECE_TYPE_NULL)
				{
					pieceUnconnectedIsNull=false;
					m_grid->getAt(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y)->setPiece(m_pieceUnconnected);
					if(	m_pieceUnconnectedPos.y+1<m_grid->getNbLines() && 
						(	m_grid->getAt(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y+1)->isFalling() ||
							m_grid->getAt(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y+1)->isEmpty()))
					{
						m_grid->getAt(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y)->startFalling();
					}
					else
					{
						m_grid->getAt(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y)->dontFall();
					}
				}
				else
					delete m_pieceUnconnected;
				//dans les 2 cas:
				m_pieceUnconnected=NULL;				
				
				
				

				
				
				//on cherche s'il y a explosion a l'endroit des 2 pieces qu'on a ajouté
				GridEventResult ger;
				if(!pieceConnectedIsNull)
					ger.addExpl(m_grid->startExplosions(m_pieceConnectedPos));
				if(!pieceUnconnectedIsNull)
					ger.addExpl(m_grid->startExplosions(m_pieceUnconnectedPos));
				if(ger.containsExpl())
				{
					m_comboc->notifyNewExplosions(); //la on force pas, faut qu'il se depeche
					if(m_comboc->getCombos()>1)
					{
						int nb=m_grid->notifyCombo(); //pour diminuer le nombre sur les pieces combo
						m_scorer->notifyComboDecrements(nb, m_comboc->getCombos());
					}
				}
				
				//score
				m_scorer->notifyExplosions(ger, m_comboc->getCombos());
				
				//ca a duré assez longtemps, on envoie la paire suivante
				if(m_frameCounter4MinimumWait>=m_level->getMinimumWaitBeforeNewPair()) 
				{
					gotoPhaseNormal();
				}
				else //ca n'a pas duré assez longtemps, on laisse du répis
				{
					m_phase=PHASE_WAIT_MORE;
				}	
				
			}
			m_frameCounter4PieceUnconnectedDrop=0;
			recreateTranslationEffect();
			m_fx_transl->start();
		}
	}
	
	//PHASE D'ATTENTE SUPPLEMENTAIRE
	else if(m_phase==PHASE_WAIT_MORE)
	{
		m_frameCounter4MinimumWait++;
		if(m_frameCounter4MinimumWait>=m_level->getMinimumWaitBeforeNewPair()) 
		{
			gotoPhaseNormal();
		}
	}
	
	
	
	//TODO: verifier que faire les updates (v) après la mise a jour des pieces (^) de la grille c ok
	m_level->update(m_comboc, m_grid, m_nextPair, m_currentPair, m_fx_transl);
	
	//score combo
	m_scorer->notifyComboNumber(m_comboc->getCombos());
	
	//debug("fin sologame update");
	return GAME_RUNNING;
}



bool SoloGame::isGameOver() //dans grid
{
	if(		(m_pieceConnected->getType()!=PIECE_TYPE_NULL && m_pieceConnectedPos.y<0)
		|| 	(m_pieceUnconnected->getType()!=PIECE_TYPE_NULL && m_pieceUnconnectedPos.y<0))
		return true;
	
	return false;
}

void SoloGame::gotoPhaseNormal()
{
	if(m_currentPair!=NULL)
		fatalError("SoloGame::gotoPhaseNormal : currentPair != NULL");
	
	m_phase=PHASE_NORMAL;
	//on prend la nouvelle paire et regenere une suivante
	m_currentPair=m_nextPair;
	
	//m_nextPair=new Pair(m_grid);
	m_nextPair=m_level->getNewPair();
	orientatePair(m_nextPair);
	
	m_fast_drop_on=false;
	recreateTranslationEffect();
	m_fx_transl->start();
	m_frameCounter4MinimumWait=0;
}

void SoloGame::orientatePair(Pair *pair)
{
	if(pair->getPiece(0)->getType()==PIECE_TYPE_NULL)
		fatalError("SoloGame : la 1ere piece n'a pas le droit d'etre null! uniquement la 2ème");
	if(pair->getPiece(1)->getType()==PIECE_TYPE_NULL)
		pair->setOrientation(ORIENTATION_NORTH);
	else
		pair->setOrientation(ORIENTATION_EAST);
}

//dit quand la paire doit descendre d'une case
bool SoloGame::pairShouldGoDown()
{
	//TODO: rajouter que la paire peut descendre suite a une demande du level (event)
	
	
	int speed=m_level->getDropSpeed();
	if(m_fast_drop_on)
		speed=m_level->getFastDropSpeed();
		
	return speed>0 && m_frameCounter4Drop>=speed;
}

bool SoloGame::pieceUnconnectedShouldGoDown()
{
	//if(m_pieceUnconnected->getType()==PIECE_TYPE_NULL)
	//	return false; //c'est pas une vraie piece, on passe a la suite
	
	if(m_pieceUnconnectedPos.y==m_pieceConnectedPos.y-1)
		return false; //la piece connectee est en dessous
	if(!m_grid->cellIsEmpty(m_pieceUnconnectedPos.x, m_pieceUnconnectedPos.y+1))
		return false; //la case de la grille est remplie
	
	return true;
}

void SoloGame::drawScene()
{
	if(m_level==NULL)
		fatalError("SoloGame::drawScene : m_level is NULL");
	if(m_grid==NULL)
		fatalError("SoloGame::drawScene : m_grid is NULL");
	
	oslClearScreen(RGB(0,0,0));
	if(m_phase==PHASE_NORMAL)
	{
		m_level->draw(m_grid, m_currentPair, m_nextPair, m_fx_transl, m_comboc, m_scorer);
		//oslDrawString(10, 260, "PHASE_NORMAL");
	}
	else if(m_phase==PHASE_WAIT_SECOND_CONNECTION)
	{
		m_level->draw(m_grid, m_pieceConnected, m_pieceConnectedPos, m_pieceUnconnected, m_pieceUnconnectedPos, m_nextPair, m_fx_transl, m_comboc, m_scorer);
		//oslDrawString(10, 260, "PHASE_WAIT_SECOND_CONNECTION");
	}
	else if(m_phase==PHASE_WAIT_MORE)
	{
		m_level->draw(m_grid, m_nextPair, m_comboc, m_scorer);
		//oslDrawString(10, 260, "PHASE_WAIT_MORE");
	}
	
	/*
	oslDrawString(10, 200, string("score="+toString(m_scorer->getScoreCurrent())).c_str());
	oslDrawString(10, 210, string("maxc="+toString(m_scorer->getMaxcombosCurrent())).c_str());
	
	oslDrawString(10, 230, "best are:");
	oslDrawString(10, 240, string("score="+toString(m_scorer->getScoreBestSelf())).c_str());
	oslDrawString(10, 250, string("maxc="+toString(m_scorer->getMaxcombosBestSelf())).c_str());
	*/
	/*oslDrawString(10, 200, "arena:");
	oslDrawString(10, 210, toString(mallinfo().arena).c_str());
	oslDrawString(10, 220, "fordblks:");
	oslDrawString(10, 230, toString(mallinfo().fordblks).c_str());
	*/
	
	oslDrawString(10, 210, toString(mallinfo().arena).c_str());
	oslDrawString(10, 220, toString(mallinfo().uordblks).c_str());
	
	oslDrawString(10, 230, toString(sceKernelMaxFreeMemSize()).c_str());
	oslDrawString(10, 240, toString(sceKernelTotalFreeMemSize()).c_str());
	
	oslDrawString(10, 150, toString(m_timer->getTime()).c_str());
	oslDrawString(10, 160, toString(int(m_level->getDuration())).c_str());
}

int SoloGame::keys()
{
	//TODO: ne pas oublier d'envoyer les evenements des touches au level (pressed right, moved right etc);
	oslReadKeys();
	
	//debuggage
	if(osl_pad.pressed.triangle)
	{
		//enableDebug();
		//debug("demande ligne suppl");
		m_level->buttonPushEvent(OSL_KEY_TRIANGLE, m_grid);
		
		//TODO: c'est temporaire, le mettre sur start!
		
	}
	else if(osl_pad.pressed.square)
	{
		//enableDebug();
		//debug("demande ligne suppl");
		m_level->buttonPushEvent(OSL_KEY_SQUARE, m_grid);
	}
	
	if(osl_pad.pressed.start)
	{
		if(m_level->isPaused())
		{
			m_level->resume();
			m_timer->start();
		}
		else
		{
			m_level->pause();
			m_timer->stop();
		}
	}
	
	if(osl_pad.pressed.right)
	{
		moveRight();
	}
	else if(osl_pad.pressed.left)
	{
		moveLeft();
	}
	
	if(osl_pad.pressed.circle)
	{
		rotateClockwise();
	}
	else if(osl_pad.pressed.cross)
	{
		rotateCounterClockwise();
	}
	
	//vitesse de tombée
	if(osl_pad.pressed.down)
	{
		m_fast_drop_on=true;
	}
	else if(osl_pad.released.down && m_fast_drop_on)
	{
		m_fast_drop_on=false;	
		recreateTranslationEffect();
	}
	
	if(osl_pad.pressed.R)
	{
		//enableDebug();
		teleportRight();
	}
	else if(osl_pad.pressed.L)
	{
		//enableDebug(false);
		teleportLeft();
	}
	
	if(osl_pad.pressed.select)
	{
		//enableDebug();
		bool waspaused=m_level->isPaused();
		
		if(!waspaused)
			m_level->pause_specific();
		
		utils::screenshot();
		
		if(!waspaused)
			m_level->resume_specific();
		//debug("screenshot taken");
		//enableDebug(false);
	}
	
	return MENU_NOACTION;
}


void SoloGame::moveRight()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de bouger la paire.
	{
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		//la place est libre
		//if((cur0.y<0 || m_grid->cellIsEmpty(cur0.x+1, cur0.y)) && (cur1.y<0 || m_grid->cellIsEmpty(cur1.x+1, cur1.y)))
		if(m_grid->cellIsEmpty(cur0.x+1, cur0.y) && m_grid->cellIsEmpty(cur1.x+1, cur1.y))
		{
			//TODO: annuler l'effet de descente (pas joli a l'affichage) si la case en dessous a droite est pleine
			m_currentPair->move(DIRECTION_RIGHT);
		}
	}
}

void SoloGame::moveLeft()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de bouger la paire.
	{
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		//la place est libre
		//if((cur0.y<0 || m_grid->cellIsEmpty(cur0.x-1, cur0.y)) && (cur1.y<0 || m_grid->cellIsEmpty(cur1.x-1, cur1.y)))
		if(m_grid->cellIsEmpty(cur0.x-1, cur0.y) && m_grid->cellIsEmpty(cur1.x-1, cur1.y))
		{
			m_currentPair->move(DIRECTION_LEFT);
		}
	}
}

void SoloGame::rotateCounterClockwise()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de tourner la paire.
	{
		if(m_currentPair->getPiece(0)->getType()==PIECE_TYPE_NULL || m_currentPair->getPiece(1)->getType()==PIECE_TYPE_NULL)
			return;//pas 2 vraies pieces: on tourne pas
		
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
//		if(cur0.y<=-1 || cur1.y<=-1) //on n'autorise pas a bouger la piece qd elle arrive
//			return;
		
		
		switch(m_currentPair->getOrientation())
		{
			case ORIENTATION_EAST	:
				//aucun cas valide ou ca empecherait de bouger comme ca, la case du dessus est toujours
				//libre 
				m_currentPair->setOrientation(ORIENTATION_NORTH);
				break;
				
			case ORIENTATION_NORTH	:
				//la case de gauche est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x-1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_WEST);	
				}
				//la case de gauche est pas libre, mais celle de droite oui,
				//on tourne et bouge la piece
				else if(m_grid->cellIsEmpty(cur0.x+1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_WEST);
					m_currentPair->move(DIRECTION_RIGHT);
				}
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				break;
			
			case ORIENTATION_WEST	:
				//la case en dessous est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x, cur0.y+1))
				{
					m_currentPair->setOrientation(ORIENTATION_SOUTH);	
				}
				//la case en dessous n'est pas libre, mais la case a gauche et gauche en dessous le sont,
				//on tourne et bouge a gauche
				else if(m_grid->cellIsEmpty(cur0.x-1, cur0.y+1))
				{
					m_currentPair->setOrientation(ORIENTATION_SOUTH);
					m_currentPair->move(DIRECTION_LEFT);
				}
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				break;
			
			case ORIENTATION_SOUTH	:
				//la case a droite est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x+1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_EAST);	
				}
				//la case de droite est pas libre, mais celle de gauche oui, on tourne et bouge
				else if(m_grid->cellIsEmpty(cur0.x-1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_EAST);
					m_currentPair->move(DIRECTION_LEFT);
				}
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				break;
		}
	}
}

void SoloGame::rotateClockwise()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de tourner la paire.
	{
		if(m_currentPair->getPiece(0)->getType()==PIECE_TYPE_NULL || m_currentPair->getPiece(1)->getType()==PIECE_TYPE_NULL)
			return;//pas 2 vraies pieces: on tourne pas
		
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		//if(cur0.y<=-1 || cur1.y<=-1) //on n'autorise pas a bouger la piece qd elle arrive
		//	return;
		
		switch(m_currentPair->getOrientation())
		{
			case ORIENTATION_EAST	:
				//la case en dessous est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x, cur0.y+1))
				{
					m_currentPair->setOrientation(ORIENTATION_SOUTH);	
				}
				//la case en dessous n'est pas libre, mais la case a droite et droite en dessous le sont,
				//on tourne et bouge a droite
				else if(m_grid->cellIsEmpty(cur0.x+1, cur0.y+1))
				{
					m_currentPair->setOrientation(ORIENTATION_SOUTH);
					m_currentPair->move(DIRECTION_RIGHT);
				}
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				break;
				
			case ORIENTATION_NORTH	:
				//la case a droite est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x+1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_EAST);	
				}
				//la case de droite est pas libre, mais celle de gauche oui, on tourne et bouge
				else if(m_grid->cellIsEmpty(cur0.x-1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_EAST);
					m_currentPair->move(DIRECTION_LEFT);
				}
				//la case de droite est pas libre, on peut faire demi tour carrément?
				/*else if(m_grid->cellIsEmpty(cur0.x, cur0.y+1))
				{
					m_currentPair->setOrientation(ORIENTATION_SOUTH);
				}*/
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				
				break;
			
			case ORIENTATION_WEST	:
				//aucun cas valide ou ca empecherait de bouger comme ca, la case du dessus est toujours
				//libre 
				m_currentPair->setOrientation(ORIENTATION_NORTH);
				break;
			
			case ORIENTATION_SOUTH	:
				//la case de gauche est libre, on tourne
				if(m_grid->cellIsEmpty(cur0.x-1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_WEST);	
				}
				//la case de gauche est pas libre, mais celle de roite oui,
				//on tourne et bouge la piece
				else if(m_grid->cellIsEmpty(cur0.x+1, cur0.y))
				{
					m_currentPair->setOrientation(ORIENTATION_WEST);
					m_currentPair->move(DIRECTION_RIGHT);
				}
				else //on est sympa, on echange les 2 pieces
				{
					m_currentPair->switchPieces();
				}
				break;
		}
	}
}

void SoloGame::teleportRight()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de bouger la paire.
	{
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		for(int i=m_grid->getNbCols()-1;i>cur0.x;i--)
		{
			if(m_grid->cellIsEmpty(i,cur0.y) && m_grid->cellIsEmpty(cur1.x+i-cur0.x, cur1.y))
			{
				m_currentPair->goTo(i, cur0.y);
				return;
			}
		}
	}
}

void SoloGame::teleportLeft()
{
	if(m_phase==PHASE_NORMAL && !m_level->isPaused()) //y a que la qu'il a le droit de bouger la paire.
	{
		Pos cur0=m_currentPair->getGridPos(0);
		Pos cur1=m_currentPair->getGridPos(1);
		
		for(int i=0;i<cur0.x;i++)
		{
			if(m_grid->cellIsEmpty(i,cur0.y) && m_grid->cellIsEmpty(cur1.x+i-cur0.x, cur1.y))
			{
				m_currentPair->goTo(i, cur0.y);
				return;
			}
		}
	}
}

/*
void SoloGame::pdr4cResultMenu(int res)
{
	OSL_IMAGE *bg=NULL;
	if(res==GAME_OVER)
	{
		bg=oslLoadImageFilePNG("res/pdr4c/menus/youloose.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	}
	else if(res==GAME_WON)
	{
		bg=oslLoadImageFilePNG("res/pdr4c/menus/youwin.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	}
	else
	{
		oslFatalError("SoloGame::pdr4cResultMenu : res should be GAME_OVER or GAME_WON");
	}
	
	bool end=false;
	while(!end)
	{
		oslStartDrawing();
		oslDrawImageXY(bg,0,0);
		oslEndDrawing();
		oslEndFrame();
		oslSyncFrame();
		
		oslReadKeys();
		if(osl_pad.pressed.cross || osl_pad.pressed.circle || osl_pad.pressed.start)
		{
			end=true;
		}
	}
	
	oslDeleteImage(bg);
}
*/

int SoloGame::onGameEnd(int game_result)
{
	//m_levelcode
	//m_difficulty
	float duration=m_level->getDuration();
	//delete m_level;
	//m_level=NULL;
	//delete m_grid;
	//m_grid=NULL;
	
	if(game_result==GAME_WON)
	{
		GameSave *gamesave=new GameSave();
		gamesave->addWonForLevel(m_levelcode, m_difficulty);
		int bestscore=gamesave->getScoreForLevel(m_levelcode, m_difficulty);
		int bestcombos=gamesave->getCombosForLevel(m_levelcode, m_difficulty);
		if(m_scorer->isBestCombos())
		{
			gamesave->setCombosForLevel(m_levelcode, m_difficulty, m_scorer->getMaxcombosCurrent());
		}
		if(m_scorer->isBestScore())
		{
			gamesave->setScoreForLevel(m_levelcode, m_difficulty, m_scorer->getScoreCurrent());
		}
		
		PFL::logMessage("Sologame::onGameEnd GAME_WON! saving...");
		gamesave->save();
		PFL::logMessage("Sologame::onGameEnd save ok.");
		delete gamesave;
		
		YouWinDialog *windialog=new YouWinDialog(m_levelcode, m_difficulty, m_scorer->getScoreCurrent(), m_scorer->getMaxcombosCurrent(), bestscore, bestcombos);
		int choice=windialog->go();
		delete windialog;
		
		if(choice==DIALOG_NO)
		{
			return GAME_QUIT;
		}
		else if(choice==DIALOG_YES)
		{
			return GAME_RETRY;
		}
	}
	else //GAME_OVER
	{
		GameSave *gamesave=new GameSave();
		PFL::logMessage("Sologame::onGameEnd GAME_OVER! saving...");
		gamesave->addLostForLevel(m_levelcode, m_difficulty);
		gamesave->save();
		delete gamesave;
		PFL::logMessage("Sologame::onGameEnd save ok.");
		YouLooseDialog *loosedialog=new YouLooseDialog(duration, m_timer->getTime());
		int choice=loosedialog->go();
		delete loosedialog;
		
		//malloc_trim(0);
		
		PFL::logMessage("Sologame::onGameEnd user answer is "+toString(choice));
		if(choice==DIALOG_NO)
		{
			return GAME_QUIT;
		}
		else if(choice==DIALOG_YES)
		{
			return GAME_RETRY;
		}
	}
	
	return GAME_QUIT;
}


void SoloGame::restartLevel()
{
	//unsigned int interrupt=pspSdkDisableInterrupts();
	PFL::logFunctionStart("Sologame::restartLevel");
	//string levelcode=m_level->getCode();
	//delete m_level;
	//if(m_level!=NULL)
	//	fatalError("SoloGame::restartLevel : le level n'est pas null!");
	//m_level=LevelFactory::newLevel(m_levelcode, m_difficulty, 1);
	m_level->reset();
	m_grid->reset();
	/*
	PFL::logMessage("Sologame::restartLevel grid reconstruction...");
	if(m_grid==NULL)
		fatalError("SoloGame::restartLevel : m_grid is NULL");
	delete m_grid;
	m_grid=new GameGrid(m_level->getNbCols(), m_level->getNbLines(), GAMEDEF_PIECE_WIDTH, GAMEDEF_PIECE_HEIGHT); //centrée par défaut
	*/
	PFL::logMessage("Sologame::restartLevel currentPair reconstruction...");
	if(m_currentPair!=NULL)
		delete m_currentPair;
	m_currentPair=m_level->getNewPair();
	orientatePair(m_currentPair);
	
	PFL::logMessage("Sologame::restartLevel pieceConnected & pieceUnconnected deletion...");
	if(m_pieceConnected!=NULL)
		delete m_pieceConnected;
	m_pieceConnected=NULL;
	if(m_pieceUnconnected!=NULL)
		delete m_pieceUnconnected;
	m_pieceUnconnected=NULL;
	
	PFL::logMessage("Sologame::restartLevel nextPair reconstruction...");
	if(m_nextPair!=NULL)
		delete m_nextPair;
	m_nextPair=m_level->getNewPair();   //new Pair(m_grid);
	orientatePair(m_nextPair);
	
	m_frameCounter4Drop=0;
	m_frameCounter4AddLine=0;
	m_frameCounter4MinimumWait=0;
	
	m_fast_drop_on=false;
	
	m_phase=PHASE_NORMAL;
	m_frameCounter4PieceUnconnectedDrop=0;
	
	PFL::logMessage("Sologame::restartLevel translationEffect reconstruction...");
	recreateTranslationEffect();
	m_fx_transl->start();
	
	PFL::logMessage("Sologame::restartLevel comboCounter reconstruction...");
	delete m_comboc;
	m_comboc=new ComboCounter();
	
	PFL::logMessage("Sologame::restartLevel scorer reconstruction...");
	delete m_scorer;
	m_scorer=new Scorer(m_levelcode, m_difficulty);
	
	//pspSdkEnableInterrupts(interrupt);	
	LevelRecordsDialog *lrdialog=new LevelRecordsDialog(m_levelcode, m_difficulty);
	lrdialog->go();
	delete lrdialog;
		
	m_level->start();
	m_timer->restart();
	PFL::logFunctionEnd("Sologame::restartLevel");
}










