#ifndef SOLOGAME_H_
#define SOLOGAME_H_

#include "code/game/Pair.h"
#include "code/game/pieces/Abstract/AbstractPiece.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/PositionsBuffer.h"
#include "code/framework/Matrix.h"
#include "code/game/combos/ComboCounter.h"
#include <string>
#include "code/game/score/Scorer.h"
#include "code/framework/time/Timer.h"

using namespace std;

//la paire est entiere et on peut jouer avec
#define PHASE_NORMAL 1
//la paire est désolidarisée, et l'une des pièces tombe (ou pas)
#define PHASE_WAIT_SECOND_CONNECTION 2
//les pieces sont connectées, et si ca n'a pas attendu assez longtemps que ca dégringole,
//on marque une petite pause quand meme (mais pas trop :) !!)
#define PHASE_WAIT_MORE 3

#define GAME_RUNNING 0
#define GAME_OVER 1
#define GAME_WON 2
#define GAME_RETRY 4
#define GAME_QUIT 5

class SoloGame
{
	private:
		bool m_skip;
		int m_action;
		AbstractLevel *m_level;
		GameGrid *m_grid; //la grille
		Pair *m_currentPair;
		Pair *m_nextPair;
		AbstractPiece *m_pieceConnected;
		AbstractPiece *m_pieceUnconnected;
		Pos m_pieceUnconnectedPos;
		Pos m_pieceConnectedPos;
		int m_difficulty;
		string m_levelcode;
		
		int m_frameCounter4Drop;
		int m_frameCounter4PieceUnconnectedDrop;
		int m_frameCounter4AddLine;
		int m_frameCounter4MinimumWait;
		
		short int m_phase;
		
		bool m_fast_drop_on; //true si le joueur appuie sur "bas" pour accelerer la descente
		
		SerialLocalEffect * m_fx_transl; //effet de descente progressive des pieces
		
		ComboCounter *m_comboc;
		
		Scorer *m_scorer;
		
		Timer *m_timer;
		
				
		bool pairShouldGoDown();
		void recreateTranslationEffect();
		void gotoPhaseNormal();
		bool pieceUnconnectedShouldGoDown();
		bool isGameOver();
		//void pdr4cResultMenu(int res);
		
		//va s'occuper d'afficher les scores, et de sauvegarder le profil
		int onGameEnd(int game_result);
		void restartLevel();
		
		bool cellIsEmptyOrPieceIsNull(AbstractPiece *piece, Pos pos);
		bool cellIsEmptyOrPieceIsNull(AbstractPiece *piece, int col, int line);
		
		void orientatePair(Pair *pair);
		
	public:
		SoloGame(string level, int difficulty=GAMEDEF_DIFFICULTY_NORMAL);
		~SoloGame();
		int go();
		int goDebug();
		void drawScene();
		int keys();
		int update(); 
		void moveRight();
		void moveLeft();
		void rotateClockwise();
		void rotateCounterClockwise();
		void teleportRight();
		void teleportLeft();
};

#endif /*SOLOGAME_H_*/
