#include "code/game/menus/pdr4c/PDR4CMainMenu.h"
#include <oslib/oslib.h>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/I18nResourceBundle.h"
#include "code/game/menus/pdr4c/PDR4CMenuItemManager.h"
#include "code/framework/check/loaders.h"

using namespace std;
using namespace utils;

#define VINYL_X 328
#define VINYL_Y 91

#define PDR4C_MAINMENU_MUSIC "res/pdr4c/menus/snd/drinkambiance.bgm"

//construction
PDR4CMainMenu::PDR4CMainMenu(string locale)
{
	//enableDebug();
	//debug("nb images in anim buffer: "+toString(Anim::getNbBufferedImages()));
	//enableDebug(false);
	//oslInitAudio();
	
	m_locale=locale;
	m_action=MENU_NOACTION;
	m_skip=false;
	
	m_img_pdr4cdrops=fwkLoadImageFilePNG("res/pdr4c/menus/pdr4cdrops.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_pdr4cdrops);
	
	//m_anim_vinyl=new Anim(2, "vinyl", "res/pdr4c/menus/vinyl/", 20, OSL_PF_4444);
	
	//BmpFont::initBmpFont(); //carbonblock24, carbon_block
	BmpFont * carbonblock=new BmpFont("res/fonts/bmpFonts/carbonblock24/", "carbon_block", OSL_PF_4444);
	
	m_menu=new PDR4CMenuItemManager();
	
	m_menu->init(locale, carbonblock);
		
	delete carbonblock;
	//BmpFont::freeRes(); //plus besoin des ressources des fonts
	
	
	//musique sons
	m_music=fwkLoadSoundFileBGM(PDR4C_MAINMENU_MUSIC, OSL_FMT_STREAM);
	oslSetSoundLoop(m_music, true);
	//m_timer_music=new Timer();
	//m_musicLength=utils::bgmDuration(PDR4C_MAINMENU_MUSIC)-100; //-100 pour correction...
	
	m_snd_move=fwkLoadSoundFileBGM("res/pdr4c/menus/snd/whoosh2.bgm", OSL_FMT_NONE);
	//if(!m_snd_move) oslFatalError("chargement impossible du son whoosh2.bgm");
	m_snd_cancel=fwkLoadSoundFileBGM("res/pdr4c/menus/snd/main_menu_cancel.bgm", OSL_FMT_NONE);
	//if(!m_snd_cancel) oslFatalError("chargement impossible du son main_menu_cancel.bgm");
	m_snd_select=fwkLoadSoundFileBGM("res/pdr4c/menus/snd/applause.bgm", OSL_FMT_NONE);
	//if(!m_snd_select) oslFatalError("chargement impossible du son shawee.bgm");
	
	//m_snd_select->volumeLeft=0x1500;
	//m_snd_select->volumeRight=0x1500;
	m_snd_move->volumeLeft=0x1500;
	m_snd_move->volumeRight=0x1500;
	m_snd_cancel->volumeLeft=0x2500;
	m_snd_cancel->volumeRight=0x2500;
	
	m_chan_mgr=new ChannelManager();
	
	m_bgcolor=getRandomBGColor();
	
}

//destruction
PDR4CMainMenu::~PDR4CMainMenu()
{
	//enableDebug();
	//debug("destruction du main");
	
	oslDeleteImage(m_img_pdr4cdrops);
	//delete m_anim_vinyl;
	delete m_menu;
	
	oslDeleteSound(m_music);
	oslDeleteSound(m_snd_move);
	oslDeleteSound(m_snd_cancel);
	oslDeleteSound(m_snd_select);
	
	delete m_chan_mgr;
	//delete m_timer_music;
	
}

//demarrage du menu
int PDR4CMainMenu::go(bool debugmode)
{
	m_chan_mgr->playSoundAndReserve(m_music); //on reserve indefiniment
	//m_timer_music->start();
	
	while(m_action==MENU_NOACTION)
	{
		//oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		//m_anim_vinyl->update();
		m_action=m_menu->update(); 
		/*if(m_timer_music->getTime()>=m_musicLength)
		{
			m_chan_mgr->playSoundAndReserve(m_music, m_musicLength+10); //+10 pour etre sur de répéter sur un autre channel
			m_timer_music->restart();
		}*/
		
		if (!m_skip && !debugmode)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			//oslBenchmarkTest(OSL_BENCH_END);
			//oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		if(!debugmode)
		{
			//End of a frame
	        oslEndFrame();
			//Synchronize to ensure the game runs at 60 fps
	        m_skip = oslSyncFrame();
			//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
		}
		else
		{
			m_action=MENU_CANCEL; //juste pour le faire sortir de la boucle
		}
	}
	//il a choisi une option
	return m_action;	
}

//dessins
void PDR4CMainMenu::drawScene()
{
	oslClearScreen(m_bgcolor);
	oslDrawImageXY(m_img_pdr4cdrops,240,80);
	//m_anim_vinyl->drawXY(328,91);
	
	m_menu->drawXY(240, 185);
}

//gestion des touches
int PDR4CMainMenu::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		//return MENU_QUIT;
	}
	
	if (osl_pad.pressed.cross)
	{
		int action=m_menu->selectAction();
		if(action==MENU_CANCEL) //cliqué sur cancel
		{
			if(m_menu->cancelAction())
			{
				m_chan_mgr->restartSound(m_snd_cancel);
			}
		}
		else if(action!=MENU_OCCUPIED)
		{
			m_chan_mgr->restartSound(m_snd_select);
		}
	}
	else if (osl_pad.pressed.square)
	{
		if(m_menu->cancelAction())
		{
			m_chan_mgr->restartSound(m_snd_cancel);
		}
	}
	
	if (osl_pad.pressed.up)
	{
		if(m_menu->previousMenuAction())
		{
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	else if (osl_pad.pressed.down)
	{
		if(m_menu->nextMenuAction())
		{
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	if (osl_pad.pressed.right)
	{
		m_menu->previousValueAction();
	}
	else if (osl_pad.pressed.left)
	{
		m_menu->nextValueAction();
	}
	
	if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	if(osl_pad.pressed.R && osl_pad.held.L)
	{
		//m_bgcolor=getRandomBGColor();
	}
	
	return MENU_NOACTION;
}

int PDR4CMainMenu::getRandomBGColor()
{
	int r=random(0,7);//fastrandom(0,7);
	switch(r)
	{
		case 1:
		{
			return RGB(149,0,149);
			break;
		}
		case 2:
		{
			return RGB(128,128,192);
			break;
		}
		case 3:
		{
			return RGB(244,189,62);
			break;
		}
		case 4:
		{
			return RGB(0,168,84);
			break;
		}
		case 5:
		{
			return RGB(128,128,0);
			break;
		}
		case 6:
		{
			return RGB(108,0,108);
			break;
		}
		case 7:
		{
			return RGB(0,89,0);
			break;
		}
	}
	
	return RGB(34,47,201);
}





