#include "code/game/menus/qj/QJLogo.h"
#include <oslib/oslib.h>
#include "code/framework/Properties.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/time/Delayer.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"

using namespace utils;

QJLogo::QJLogo()
{
	Properties *props=new Properties("res/qj/logo/qjlogo.properties");
	
	m_img_splash=fwkLoadImageFilePNG("res/qj/logo/summer-contest-splash.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_splash);
	oslImageCenterScreen(m_img_splash);
	m_snd_punch=fwkLoadSoundFileBGM("res/qj/logo/punch.bgm", OSL_FMT_NONE);
	m_snd_punch->volumeLeft=0x4000;
	m_snd_punch->volumeRight=0x4000;
	m_fx_size=new SizeWowEffect(props->getIntValue("fx.size.shrink"), props->getIntValue("fx.size.grow"), 
								props->getIntValue("fx.size.shrink"), props->getIntValue("fx.size.grow"), 
								props->getIntValue("fx.size.time"), props->getIntValue("fx.size.time"), true, 1);
	m_fx_vanish=new ProgressiveAlphaEffect(	props->getIntValue("fx.fadeout.time"), props->getIntValue("fx.fadeout.time"), 
											0, 255, 1);
	
	m_fx_shake=new ShakeEffect(	props->getIntValue("fx.shake.width"), props->getIntValue("fx.shake.height"), 
								props->getIntValue("fx.shake.time"), true, props->getIntValue("fx.shake.loops"));
	
	m_delayer_vanish=new Delayer(props->getIntValue("vanish.delay"), 1);
	m_delayer_sound=new Delayer(props->getIntValue("sound.delay"), 1);
	delete props;
}

QJLogo::~QJLogo()
{
	//enableDebug();
	
	oslDeleteImage(m_img_splash);
	
	oslDeleteSound(m_snd_punch);
	
	delete m_fx_size;
	delete m_fx_vanish;
	
	delete m_delayer_vanish;
	
	delete m_delayer_sound;
	
	delete m_fx_shake;
}


void QJLogo::go()
{
	//démarrage
	m_fx_size->start();
	
	bool skip=false; //skip phase de dessin si trop de calculs
	while(!m_fx_vanish->finished()) //on quitte des que le fadeout a fini
	{
		//touches
		keys();
		
		//phase d'update
		m_fx_size->update();
		m_fx_vanish->update();
		m_delayer_vanish->update();
		m_delayer_sound->update();
		m_fx_shake->update();
		
		if(m_delayer_sound->get())
		{
			oslPlaySound(m_snd_punch, 0);
			m_fx_shake->start();
		}
		if(m_delayer_vanish->get())
		{
			m_fx_vanish->start();
		}
		
		//phase de dessin
		if(!skip)
		{
			oslStartDrawing();
			drawScene();
			oslEndDrawing();
		}
		oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
	    skip = oslSyncFrame();
	}
}

void QJLogo::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start || osl_pad.pressed.cross || osl_pad.pressed.circle)  
	{
		m_fx_vanish->start();
	}
	
	if(osl_pad.pressed.R)
	{
		oslPlaySound(m_snd_punch, 0);
		m_fx_shake->restart();
	}
}

void QJLogo::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	fx::draw(m_img_splash, m_fx_size->isRunning()?m_fx_size:NULL, m_fx_shake->isRunning()?m_fx_shake:NULL);

	if(m_fx_vanish->isRunning() || m_fx_vanish->finished())
	{
		m_fx_vanish->activate();
		oslDrawFillRect(0,0,480,272,RGB(0,0,0));
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
}



















