#include "code/game/menus/qj/QJMenuItemManager.h"
#include "code/framework/menus/MenuItem.h"
#include "code/framework/utils.h"
#include "code/framework/I18nResourceBundle.h"
#include <string>
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/Properties.h"
using namespace utils;
using namespace std;

#define translationtime 8


QJMenuItemManager::QJMenuItemManager()
{
	m_root=NULL;//new MenuItem("root", 0, NULL, 5);
	m_current=NULL;
	m_state=MENUSTATE_WAIT;
		
	m_fx_down_appear = new AppearDownEffect(translationtime);
	m_fx_down_translation = new TranslationEffect(0,0,0,-20,translationtime,translationtime,true,1); //voir dans initAsMainMenu
	m_fx_down_vanish = new VanishUpEffect(translationtime);
	m_fx_up_translation = new TranslationEffect(0,0,0,20,translationtime,translationtime,true,1); //lorsqu'on appuie sur haut, les items descendent
	m_fx_up_appear = new AppearUpEffect(translationtime); //lorsqu'on appuie sur haut, l'item du dessus apparait
	m_fx_up_vanish = new VanishDownEffect(translationtime);
	
	m_fx_blink=new BlinkEffect(9, 9, false, 11);
	
	m_distance=0;
	
	Properties *props=new Properties("res/qj/menu/qjmenu.properties");
	u32 colorfrom=RGB(	props->getIntValue("fx.selecteditem.tint.from.r"),
						props->getIntValue("fx.selecteditem.tint.from.g"),
						props->getIntValue("fx.selecteditem.tint.from.b")
	);
	u32 colorto=RGB(	props->getIntValue("fx.selecteditem.tint.to.r"),
						props->getIntValue("fx.selecteditem.tint.to.g"),
						props->getIntValue("fx.selecteditem.tint.to.b")
	);
	
	int time=props->getIntValue("fx.selecteditem.tint.time");
	
	m_fx_tint=new ProgressiveTintEffect(time, time, colorfrom, colorto);
	m_fx_tint->start();
	
	m_params=new AlphaParamsState();
	
	delete props;
	
}

QJMenuItemManager::~QJMenuItemManager()
{
	delete m_root;
	
	delete m_fx_blink;
	delete m_fx_down_appear;
	delete m_fx_down_translation;
	delete m_fx_down_vanish;
	delete m_fx_up_appear;
	delete m_fx_up_translation;
	delete m_fx_up_vanish;
	delete m_fx_tint;
	delete m_params;
}

void QJMenuItemManager::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3)
{
		
		if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getBeforePreviousChild()->getImage(), x, y, m_fx_up_appear, fx1, fx2, fx3);
		}
		
		
		if(m_state==MENUSTATE_CANCELING)
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, fx1, fx2, fx3);
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, m_fx_down_vanish, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, m_fx_up_translation, fx1, fx2, fx3);
		}
		else
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, fx1, fx2, fx3);
				
		
		
		if(m_state==MENUSTATE_WAIT)
		{
			//oslSetAlpha(OSL_FX_TINT, RGB(0, 0, 255));
			m_params->saveState();
			m_fx_tint->activate();
			
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance);
			
			m_params->restoreState();
			
			
			//oslSetAlpha(OSL_FX_DEFAULT, 0);
		}
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_blink);
			//oslSetAlpha(OSL_FX_DEFAULT, 0);
			//oslDrawImageXY(m_current->getSelectedChild()->getImage(), x, y+dist);
		}
		else if(m_state==MENUSTATE_CANCELING)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_down_translation);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_up_translation);
		}
		
		
		if(m_state==MENUSTATE_CANCELING)
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), fx1, fx2, fx3);
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), m_fx_down_translation, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), m_fx_up_vanish, fx1, fx2, fx3);
		}
		else
			//oslDrawImageXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance));
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), fx1, fx2, fx3);
		
	
	
		
		if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getAfterNextChild()->getImage(), x, y+(2*m_distance), m_fx_down_appear, fx1, fx2, fx3);
		}
}

int QJMenuItemManager::selectAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		if(m_current->hasValues())
		{
			return MENU_NOACTION;	
		}
		//else
		if(m_current->getSelectedChild()->getId()==MENU_CANCEL)
		{
			if(cancelAction())
				return MENU_CANCEL;
			return MENU_NOACTION;
		}
	
		m_state=MENUSTATE_SELECTIONNING;
		m_fx_blink->restart();
	
	
		if(m_current->getSelectedChild()->hasChildren())
		{	
			return MENU_NOACTION;
		}
		return m_current->getSelectedChild()->getId();
	}
	return MENU_OCCUPIED;
}

bool QJMenuItemManager::cancelAction()
{
	if(m_current->getParent()!=NULL)
	{
		m_state=MENUSTATE_CANCELING;
		
		return true;	
	}
	return false;
}

bool QJMenuItemManager::nextMenuAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		m_state=MENUSTATE_GOINGUP;
		m_fx_down_appear->restart();
		m_fx_down_vanish->restart();
		m_fx_down_translation->restart();
		return true;
	}
	
	//m_current->gotoNextChild();
	return false;
}

bool QJMenuItemManager::previousMenuAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		m_state=MENUSTATE_GOINGDOWN;
		m_fx_up_appear->restart();
		m_fx_up_vanish->restart();
		m_fx_up_translation->restart();
		return true;
	}
	return false;
}

bool QJMenuItemManager::nextValueAction()
{
	m_current->gotoNextValue();
	return false;
}

bool QJMenuItemManager::previousValueAction()
{
	//TODO: ajouter changements de valeurs
	m_current->gotoPreviousValue();
	return false;
}

int QJMenuItemManager::update()
{
	//TODO: utiliser un manager
	//fx update.
	m_fx_blink->update();
	m_fx_down_appear->update();
	m_fx_down_translation->update();
	m_fx_down_vanish->update();
	m_fx_up_appear->update();
	m_fx_up_translation->update();
	m_fx_up_vanish->update();
	m_fx_tint->update();
	
	
	if(m_state==MENUSTATE_SELECTIONNING)
	{
		if(m_fx_blink->finished())
		{
			if(m_current->getSelectedChild()->hasChildren())
			{
				m_state=MENUSTATE_WAIT;
				m_current=m_current->getSelectedChild();	
				return MENU_NOACTION;
			}
			return m_current->getSelectedChild()->getId();
		}	
	}
	else if(m_state==MENUSTATE_CANCELING)
	{
		//if(m_fx_cancel->finished())
		//{
			m_state=MENUSTATE_WAIT;
			m_current=m_current->getParent();
		//}
		return MENU_NOACTION;
	}
	else if(m_state==MENUSTATE_GOINGUP) //les items montent, on veut le nextChild
	{
		if(m_fx_down_translation->finished())
		{
			m_state=MENUSTATE_WAIT;
			m_current->gotoNextChild();
		}
		return MENU_NOACTION;
	}
	else if(m_state==MENUSTATE_GOINGDOWN) //les items descendent, on veut le previousChild
	{
		if(m_fx_up_translation->finished())
		{
			m_state=MENUSTATE_WAIT;
			m_current->gotoPreviousChild();
		}
		return MENU_NOACTION;
	}
	
	return MENU_NOACTION;
}


void QJMenuItemManager::init(string locale, BmpFont * f)
{
	
	I18nResourceBundle * bundle=new I18nResourceBundle(locale, "QJ_menu.properties");
	
	m_root=new MenuItem("root", f, 0, NULL, 5);	
	MenuItem *levels=m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_levels"), f, 1, m_root, 11));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_8bits_easy"), f, TEST_LEVEL8BITS_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_8bits_normal"), f, TEST_LEVEL8BITS_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_8bits_hard"), f, TEST_LEVEL8BITS_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_tetris_easy"), f, TEST_LEVELTETRIS_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_tetris_normal"), f, TEST_LEVELTETRIS_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_tetris_hard"), f, TEST_LEVELTETRIS_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_tetris_vhard"), f, TEST_LEVELTETRIS_VHARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_sexy_easy"), f, TEST_LEVELSEXY_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_sexy_normal"), f, TEST_LEVELSEXY_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_sexy_hard"), f, TEST_LEVELSEXY_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("qj_cancel"), f, MENU_CANCEL, levels));
	m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_howtoplay"), f, QJ_TUTORIAL, m_root));
	m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_credits"), f,  QJ_CREDITS, m_root));
	m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_scores"), f,  QJ_SCORES, m_root));
	
	MenuItem *bonus=m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_bonus"), f, 1, m_root, 5));
		bonus->addSubMenu(new MenuItem(bundle->getMessage("qj_8bits_blixt"), f, TEST_LEVEL8BITS_BLIXT, bonus));
		bonus->addSubMenu(new MenuItem(bundle->getMessage("qj_frozen_vhard"), f, PDR4C_LEVELFROZEN_VHARD, bonus));
		bonus->addSubMenu(new MenuItem(bundle->getMessage("qj_voltigeur_hard"), f, TEST_LEVELVOLTIGEUR_HARD, bonus));
		bonus->addSubMenu(new MenuItem(bundle->getMessage("qj_logo_sequence"), f, LOGO_SEQUENCE, bonus));
		bonus->addSubMenu(new MenuItem(bundle->getMessage("qj_cancel"), f, MENU_CANCEL, bonus));
	
	m_root->addSubMenu(new MenuItem(bundle->getMessage("qj_quit"), f, MENU_QUIT, m_root));
	
	m_current=m_root;
		
	delete bundle;
	
	//debug("done");
	//enableDebug();
	
	m_distance=(m_current->getSelectedChild()->getImage()->sizeY * 9)/10;
	
	//debug("distance="+toString(m_distance));
	//enableDebug(false);
	
	//devrait etre dans le constructeur mais c seulement là qu'on connait m_distance
	delete m_fx_down_translation;
	delete m_fx_up_translation;
	m_fx_up_translation = new TranslationEffect(0,0,0,m_distance,translationtime,translationtime,true,1);
	m_fx_down_translation = new TranslationEffect(0,0,0,-m_distance,translationtime,translationtime,true,1);
	
}












