#include "code/game/menus/qj/dialogs/LevelRecordsDialog.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include "code/game/saves/GameSave.h"
#include "code/framework/imgFilters/imgFilters.h"
#include <iostream>
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace utils;

LevelRecordsDialog::LevelRecordsDialog(string level_code, int level_difficulty)
: AbstractDialog("res/qj/dialogs/levelrecords.png")
{
	PFL::logConstructionStart("LevelRecordsDialog");
	
	BmpFont *font_big=new BmpFont("res/fonts/bmpFonts/angie/22/", "angie", OSL_PF_8888);
	
	m_text_title=new BmpText("Records:", font_big);
	m_text_title->compile();
	oslSwizzleImage(m_text_title->getImage());
	
	delete font_big;
	
	
	BmpFont *font_small=new BmpFont("res/fonts/bmpFonts/angie/20/", "angie", OSL_PF_8888);
	
	m_text_scoreTitle=new BmpText("Best score:", font_small);
	m_text_scoreTitle->compile();
	oslSwizzleImage(m_text_scoreTitle->getImage());
	
	GameSave *gamesave=new GameSave();
	int bestscore=gamesave->getScoreForLevel(level_code, level_difficulty);
	int bestcombos=gamesave->getCombosForLevel(level_code, level_difficulty);
	delete gamesave;
	
	m_text_score=new BmpText(toString(bestscore), font_small);
	m_text_score->compile();
	oslSwizzleImage(m_text_score->getImage());
	
	m_text_combosTitle=new BmpText("Best combos:", font_small);
	m_text_combosTitle->compile();
	oslSwizzleImage(m_text_combosTitle->getImage());
	
	
	m_text_combos=new BmpText(toString(bestcombos), font_small);	
	m_text_combos->compile();
	oslSwizzleImage(m_text_combos->getImage());
	
	m_text_ok=new BmpText("OK", font_small);
	m_text_ok->compile();
	oslSwizzleImage(m_text_ok->getImage());
	
	delete font_small;
	
	//m_music=fwkLoadSoundFileMP3("res/qj/dialogs/meh.mp3", OSL_FMT_STREAM);
	m_music=fwkLoadSoundFileMP3("res/qj/dialogs/cant_feel_better.mp3", OSL_FMT_STREAM);
	oslSetSoundLoop(m_music, true);
	
	m_anim_buttons=new Anim("res/qj/dialogs/cross.png", 125.0f);
	m_anim_buttons->addImage("res/qj/dialogs/square.png", OSL_PF_8888);
	
	PFL::logConstructionEnd("LevelRecordsDialog");
}

LevelRecordsDialog::~LevelRecordsDialog()
{
	PFL::logDestructionStart("LevelRecordsDialog");
	//ca a planté apres ca une fois.
	
	PFL::logMessage("deleting texts...");
	delete m_text_title;
	delete m_text_scoreTitle;
	delete m_text_score;
	delete m_text_combosTitle;
	delete m_text_combos;		
	delete m_text_ok;
	
		
	PFL::logMessage("deleting music...");
	fwkDeleteSound(m_music);
	
	PFL::logMessage("deleting anim...");
	delete m_anim_buttons;
	
	PFL::logDestructionEnd("LevelRecordsDialog");
}




void LevelRecordsDialog::startSpecific()
{
	oslPlaySound(m_music, 0);
	m_anim_buttons->start();
}

void LevelRecordsDialog::updateSpecific()
{
	m_anim_buttons->update();
}

void LevelRecordsDialog::drawSpecific()
{
	m_text_title->drawXY(118, 68);
	
	m_text_scoreTitle->drawXY(119, 108);
	m_text_score->drawXY(364-m_text_score->getWidth(), 108);
	
	m_text_combosTitle->drawXY(119, 138);
	m_text_combos->drawXY(364-m_text_combos->getWidth(), 138);
	
	m_text_ok->drawXY(340, 178);	
	m_anim_buttons->drawXY(320, 189);
}

int LevelRecordsDialog::keys()
{
	oslReadKeys();
	
	if(osl_pad.pressed.square || osl_pad.pressed.cross)
	{
		return DIALOG_OK;
	}
	else if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	
	return DIALOG_NONE;
}

		
		
		
