#include "code/game/menus/qj/dialogs/YouLooseDialog.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include "code/framework/imgFilters/imgFilters.h"
#include <iostream>
#include "code/game/menus/qj/dialogs/AbstractDialog.h"
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace utils;

YouLooseDialog::YouLooseDialog(float level_duration, float time)
: AbstractDialog("res/qj/dialogs/youloose.png")
{
	PFL::logConstructionStart("YouLooseDialog");
	
	float percent=(time*100)/level_duration;
	ostringstream ost;
	ost.setf(ios::fixed,ios::floatfield);
	ost.precision(2);
	ost << percent << "%";
	
		
	m_font_big=new BmpFont("res/fonts/bmpFonts/angie/22/", "angie", OSL_PF_8888);
	m_font_small=new BmpFont("res/fonts/bmpFonts/angie/20/", "angie", OSL_PF_8888);
	m_text_title=new BmpText("You loose!", m_font_big);
	m_text_completed=new BmpText("Completed:", m_font_small);
	m_text_percent=new BmpText(percent>=100?"No luck!":ost.str(), m_font_small);
	m_text_retry=new BmpText("Retry?", m_font_small);
	m_text_yes=new BmpText("Yes", m_font_small);
	m_text_no=new BmpText("No", m_font_small);
	
	m_img_cross=fwkLoadImageFilePNG("res/qj/dialogs/cross.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_square=fwkLoadImageFilePNG("res/qj/dialogs/square.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	PFL::logConstructionEnd("YouLooseDialog");
}

YouLooseDialog::~YouLooseDialog()
{
	PFL::logDestructionStart("YouLooseDialog");
	
	delete m_font_big;
	delete m_font_small;
	delete m_text_title;
	delete m_text_completed;
	delete m_text_percent;
	delete m_text_retry;
	delete m_text_yes;
	delete m_text_no;
	
	oslDeleteImage(m_img_cross);
	oslDeleteImage(m_img_square);
	
	PFL::logDestructionEnd("YouLooseDialog");
}

void YouLooseDialog::drawSpecific()
{	
	m_text_title->drawXY(118, 73);
	m_text_completed->drawXY(118, 108);
	m_text_percent->drawXY(364-m_text_percent->getWidth(), 108);
	m_text_retry->drawXY(118, 143);
	
	oslDrawImageXY(m_img_cross, 250, 180);
	m_text_yes->drawXY(270, 169);
	oslDrawImageXY(m_img_square, 320, 180);
	m_text_no->drawXY(340, 169);
}

int YouLooseDialog::keys()
{
	oslReadKeys();
	
	if(osl_pad.pressed.square)
	{
		return DIALOG_NO;
	}
	else if(osl_pad.pressed.cross)
	{
		return DIALOG_YES;
	}
	else if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	return DIALOG_NONE;
}

void YouLooseDialog::startSpecific()
{
	
}

void YouLooseDialog::updateSpecific()
{
	
}
		
		
