#ifndef ABSTRACTPIECE_H_
#define ABSTRACTPIECE_H_

#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"

//colors:
#define PIECE_COLOR_BLUE 0
#define PIECE_COLOR_RED 1
#define PIECE_COLOR_GREEN 2
#define PIECE_COLOR_YELLOW 3
#define PIECE_COLOR_PURPLE 4
#define PIECE_COLOR_ORANGE 5
#define PIECE_COLOR_NONE -1

//shapes
#define PIECE_SHAPE_CIRCLE 0
#define PIECE_SHAPE_CROSS 1
#define PIECE_SHAPE_SQUARE 2
#define PIECE_SHAPE_TRIANGLE 3
#define PIECE_SHAPE_STAR 4
#define PIECE_SHAPE_DIAMOND 5
#define PIECE_SHAPE_NONE -1

#define PIECE_MODE_COLOR 0
#define PIECE_MODE_SHAPE 1
#define PIECE_MODE_SPECIAL 2
#define PIECE_MODE_ANY 3

#define PIECE_TYPE_NORMAL 0
#define PIECE_TYPE_COMBO 1
#define PIECE_TYPE_SHIELDED 2
#define PIECE_TYPE_JOKER 3
#define PIECE_TYPE_LEVEL 4
#define PIECE_TYPE_NULL 5

#define PIECE_EVENT_COMBO_REDUCE 0
#define PIECE_EVENT_COMBO_EXPLODE 1
#define PIECE_EVENT_COMBO_ADD 2
#define PIECE_EVENT_COMBO_GET 3
#define PIECE_EVENT_LEVEL_GETID 4
#define PIECE_EVENT_LEVEL_ID_ADD 5
#define PIECE_EVENT_LEVEL_ID_REDUCE 6

#define PIECE_NB_SHAPES 6
#define PIECE_NB_COLORS 6

//une seule pièce (rond carré triangle etc), pour l'objet contenant 2 pièces,
//voir Pair.
class AbstractPiece
{
	private:
		
	protected:
		//static
		static const OSL_COLOR color_blue=RGB(0,0,255);
		static const OSL_COLOR color_red=RGB(255,0,0);
		static const OSL_COLOR color_green=RGB(0,255,0);
		static const OSL_COLOR color_purple=RGB(128,0,128);
		static const OSL_COLOR color_yellow=RGB(255,255,0);
		static const OSL_COLOR color_orange=RGB(255,128,0);
	
		//int m_x, m_y; //positions a l'ecran en px
		
	public:
		static OSL_COLOR getOslColor(int color_num);
	
		AbstractPiece(){}
		virtual ~AbstractPiece(){}
		virtual AbstractPiece *clone() const=0;
		
		virtual int getShape()=0; //shapenum
		virtual int getColor()=0; //colornum
		virtual int getByMode(int mode); //shape ou color suivant le mode
		virtual int getType()=0; //typenum
		virtual void setColor(int color)=0;
		virtual void setShape(int shape)=0;
		
		virtual void startDestruct()=0; //demarre l'anim de destruction
		virtual void drawXY(int x, int y, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL)=0;
		virtual void update()=0; 
		virtual bool destructed()=0; //informe de la fin de l'anim d'explosion
		virtual bool destructing()=0; //informe qu'elle est en train d'exploser
		
		virtual void setExplosionTime(float time)=0;
		
		virtual int specialEvent(int e)=0; //pour les pieces spéciales
		
		virtual bool isRandom(); //nous dit si la piece a été choisie aléatoirement a la construction
		
		//statiques
		//static void initPieceGeneration(); //a appeler avant de créer des pieces, ca buffer des ressources
		//static void endPieceGeneration(); //a appeler pour libérer les ressources
		
		//void setPixPos(int x, int y);
};

#endif /*ABSTRACTPIECE_H_*/
