#include "code/game/pieces/impl/PieceJoker.h"
#include "code/game/pieces/PiecesBufferFactory.h"

void PieceJoker::initFx()
{
	m_fx_size1=new SizeWowEffect(10, 10, 10, 10, 15, 15);
	m_fx_size2=new SizeWowEffect(20, 20, 0, 0, 27, 27);
	m_fx_rot1=new RotationEffect(171, false);
	m_fx_rot2=new RotationEffect(97, false);
	
	m_fx_size1->start();
	m_fx_size2->start();
	m_fx_rot1->start();
	m_fx_rot2->start();
}

PieceJoker::PieceJoker(int shape, int color)
{
	m_color_num=color;
	m_shape_num=shape;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->setDelayTotal(m_anim_destr->getDelayTotal()/2.0f);
	
	m_img_ray1=PiecesBufferFactory::getJokerImage1();
	m_img_ray2=PiecesBufferFactory::getJokerImage2();
	
	initFx();
		
	
	m_random=false;
}

PieceJoker::PieceJoker()
{
	m_color_num=random(0,5);
	m_shape_num=random(0,5);
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->setDelayTotal(m_anim_destr->getDelayTotal()/2);
	
	m_img_ray1=PiecesBufferFactory::getJokerImage1();
	m_img_ray2=PiecesBufferFactory::getJokerImage2();
	
	initFx();
		
	
	m_random=true;
}

void PieceJoker::setExplosionTime(float time)
{
	m_anim_destr->setDelayTotal(time);
}

PieceJoker::~PieceJoker()
{
	delete m_anim_destr;
	delete m_fx_size1;
	delete m_fx_rot1;
	delete m_fx_rot2;
	delete m_fx_size2;
}

PieceJoker::PieceJoker(const PieceJoker &piece)//copie
{
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->setDelayTotal(m_anim_destr->getDelayTotal()/2);
	
	initFx();
	
	m_img_ray1=PiecesBufferFactory::getJokerImage1();
	m_img_ray2=PiecesBufferFactory::getJokerImage2();
	
	
	m_random=piece.m_random;
}

PieceJoker& PieceJoker::operator =(const PieceJoker& piece)
{
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->setDelayTotal(m_anim_destr->getDelayTotal()/2);
	
	initFx();
	
	m_img_ray1=PiecesBufferFactory::getJokerImage1();
	m_img_ray2=PiecesBufferFactory::getJokerImage2();
	
	
	
	m_random=piece.m_random;
	
	return *this;
}

PieceJoker * PieceJoker::clone() const
{
	PieceJoker *p=new PieceJoker(m_shape_num, m_color_num);
	p->m_random=m_random;
	return p;
}

int PieceJoker::getShape()
{
	return m_shape_num;
}

int PieceJoker::getColor()
{
	return m_color_num;
}

void PieceJoker::setColor(int color)
{
	m_color_num=color;
	m_color=getOslColor(m_color_num);
}

void PieceJoker::setShape(int shape)
{
	m_shape_num=shape;
	Anim *old=m_anim_destr;
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->sync(old);
	delete old;
}

int PieceJoker::getType()
{
	if(destructing() || destructed())
		return PIECE_TYPE_NORMAL; //se fait passer pour une piece normale apres explosion
	return PIECE_TYPE_JOKER;
}

void PieceJoker::startDestruct()
{
	m_anim_destr->start();
}

void PieceJoker::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	fx::drawXY(m_img_ray1, x, y, m_fx_size1, m_fx_rot1, fx1, fx2, fx3);
	fx::drawXY(m_img_ray2, x, y, m_fx_size2, m_fx_rot2, fx1, fx2, fx3);

	//TODO: mélange les effets au lieu de les setter
	oslSetAlpha(OSL_FX_TINT, m_color);
	//m_anim_destr->draw(x,y);
	fx::drawXY(m_anim_destr->getImage(), x, y, fx1, fx2, fx3, fx4, fx5);
	oslSetAlpha(OSL_FX_DEFAULT, 0); //TODO: idem, remet comme c'était

}

void PieceJoker::update()
{
	m_anim_destr->update();
	m_fx_rot1->update();
	m_fx_rot2->update();
	m_fx_size1->update();
	m_fx_size2->update();
	
}
 
bool PieceJoker::destructed()
{
	return m_anim_destr->finished();
}

bool PieceJoker::destructing()
{
	return m_anim_destr->isRunning();
}

bool PieceJoker::isRandom()
{
	return m_random;
}





