#include "code/game/settings.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/resRepo/ResRepoFactory.h"
#include "code/framework/check/PrettyFileLogger.h"
#include "code/game/pieces/PiecesBufferFactory.h"
#include <iostream>

using namespace utils;

//OSL_PF_8888 32 bits
//OSL_PF_5650 16bit
#include "code/game/gamedef.h"

using namespace std;

void init()
{
	oslInitAudioME(OSL_FMT_MP3);
	
	scePowerSetClockFrequency(333, 333, 166);
	sceDisplayWaitVblankStart();
	sceDisplayWaitVblankStart();
	
	if(fileExists(PFL_LOG_FILE))
	{
		if(sceIoRemove(PFL_LOG_FILE)!=0) //failure
		{
			fatalError("cannot delete file '"+string(PFL_LOG_FILE)+"'");
		}
	}
	
	PFL::logFunctionStart("init()");
	
	oslInit(OSL_IF_USEOWNCALLBACKS);    //(0);						//The lib
	oslInitGfx(GAMEDEF_SCREENMODE, true);		//Gfx //osl_pf_8888
	//oslInitConsole();				//Text //done by oslInitGfx()
	oslInitAudio();					//Sound
	
	oslSetDithering(false); //pas de dithering, ie pas de mélange de couleurs
	
	//Enable frameskipping, 0 frame out of 1 is skipped by default (60 fps)
	oslSetFrameskip(1);
	//But skip a maximum of 2 frames out of 3, else the game seems very laggy
	oslSetMaxFrameskip(3);
	
	oslSetTextColor(RGBA(255,255,255,255));
	oslSetBkColor(RGBA(0,0,0,0));
	
	initrandom();
	
	//initialisation du systeme de fonts bitmap
	BmpFont::initBmpFont();
	
	PFL::logMessage("buffer resources...");
	PiecesBufferFactory::bufferResources();
	
	if(!directoryExists("screens/"))
	{
		createDirectory("screens/");
	}
	if(!directoryExists("temp/"))
	{
		createDirectory("temp/");
	}
	
	//weird initialization of iostreams :( needed or else program will crash when arena is >10 000 000
	ostringstream ost;
	ost.setf(ios::fixed,ios::floatfield);
	ost.precision(2);
	ost << 15.0256f;
	
	PFL::logFunctionEnd("init()");
}

void deinit()
{
	BmpFont::freeRes();
	//ProfileManager::deleteProfiles();
	PiecesBufferFactory::unbufferResources();
}

void initKeyRepeat4Menus()
{
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_RIGHT|OSL_KEYMASK_DOWN|OSL_KEYMASK_LEFT,GAMEDEF_INMENU_KEY_WAIT_BEFORE_REPEAT,GAMEDEF_INMENU_KEY_REPEAT);
}

void initKeyRepeat4Game()
{
	oslSetKeyAutorepeat(OSL_KEYMASK_RIGHT|OSL_KEYMASK_LEFT,GAMEDEF_INGAME_KEY_WAIT_BEFORE_REPEAT,GAMEDEF_INGAME_KEY_REPEAT);
}






