#include "code/tests/TestScenario.h"
#include <oslib/oslib.h>
#include <string>
#include "code/framework/utils.h"
#include "code/game/menus/MenusDefines.h"


using namespace std;
using namespace utils;

//construction
TestScenario::TestScenario()
{
	//oslInitAudio();
	
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_DOWN,40,4);
	
	m_action=MENU_NOACTION;
	m_skip=false;
	
	//m_music=oslLoadSoundFileBGM("res/tests/testcenter/test.bgm", OSL_FMT_STREAM);
	m_music=oslLoadSoundFileMP3("res/tests/testcenter/test.mp3", OSL_FMT_STREAM);
	m_scenario=new EventsScenario("res/tests/testcenter/test.scenario");
	
	for(int i=0;i<28;i++)
	{
		int x=(i%7)*68;
		int y=int(i/7)*68;
		m_positions[i]=Pos(x,y);
	}
	
	m_alpha=new ProgressiveAlphaEffect *[28];
	for(int i=0;i<28;i++)
	{
		m_alpha[i]=new ProgressiveAlphaEffect(30, 30, 255, 0, 1);
	}
	
	for(int i=0;i<28;i++)
	{
		m_used[i]=false; //dit si le num devenement est utilisé
	}
	
	m_colors[0]=RGB(150,200,0);
	m_colors[1]=RGB(255,0,0);
	m_colors[2]=RGB(0,255,0);
	m_colors[3]=RGB(0,0,255);
	m_colors[4]=RGB(255,255,0);
	m_colors[5]=RGB(255,0,255);
	m_colors[6]=RGB(0,255,255);
	m_colors[7]=RGB(0,128,255);
	m_colors[8]=RGB(0,255,128);
	m_colors[9]=RGB(255,128,0);
	m_colors[10]=RGB(128,255,0);
	m_colors[11]=RGB(128,0,255);
	m_colors[12]=RGB(255,0,128);
	m_colors[13]=RGB(128,128,255);
	m_colors[14]=RGB(128,255,128);
	m_colors[15]=RGB(255,128,128);
	m_colors[16]=RGB(255,128,255);
	m_colors[17]=RGB(255,255,128);
	m_colors[18]=RGB(128,255,255);
	m_colors[19]=RGB(255,255,255);
	m_colors[20]=RGB(200,0,0);
	m_colors[21]=RGB(0,200,0);
	m_colors[22]=RGB(0,0,200);
	m_colors[23]=RGB(200,200,0);
	m_colors[24]=RGB(0,200,200);
	m_colors[25]=RGB(200,0,200);
	m_colors[26]=RGB(0,150,200);
	m_colors[27]=RGB(200,0,150);
	
	m_test=0;
}

//destruction
TestScenario::~TestScenario()
{
	for(int i=0;i<28;i++)
	{
		delete m_alpha[i];
	}
	delete[] m_alpha;
	
	delete m_scenario;
	
	oslDeleteSound(m_music);
	
	//oslDeinitAudio();
}

//demarrage du menu
int TestScenario::go()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
	
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		update();
				
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslBenchmarkTest(OSL_BENCH_END);
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle TestAnimationSimple, return "+toString(m_action));
	enableDebug(false);
	
	return m_action;	
}

void TestScenario::update()
{
	m_scenario->update();
	for(int i=1;i<=28;i++)
	{
		m_alpha[i-1]->update();
		if(m_scenario->hasEvent(i))
		{
			m_used[i-1]=true;
			m_alpha[i-1]->restart();
			m_test=i;
		}
	}
}

//dessins
void TestScenario::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	oslSetTextColor(RGB(0,0,0));
	oslSetBkColor(RGBA(0,0,0,0));
	for(int i=0;i<28;i++)
	{
		if(m_used[i])
		{
			m_alpha[i]->activate();
			oslDrawFillRect(m_positions[i].x, m_positions[i].y, m_positions[i].x+67, m_positions[i].y+67, m_colors[i]);
			oslSetAlpha(OSL_FX_DEFAULT, 0);
			oslDrawString(m_positions[i].x+30, m_positions[i].y+30, toString(i+1).c_str());
		}
	}
	oslSetTextColor(RGB(255,255,255));
	oslSetBkColor(RGBA(128,128,128,128));
	
	//oslDrawString(10, 250, ("delay total: " + toString(m_anim->getDelayTotal())).c_str());
	//oslDrawString(10, 260, ("event " + toString(m_test)).c_str());
	oslDrawString(10, 260, ("time: " + toString(m_scenario->getTime())).c_str());
}

//gestion des touches
int TestScenario::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		enableDebug();
		debug("start!");
		enableDebug(false);
		return MENU_MAIN;
	}
	
	if(osl_pad.pressed.square)
	{
		if(m_scenario->isRunning())
		{
			m_scenario->pause();
			oslPauseSound(m_music,1);
		}
		else
		{
			m_scenario->start();
			oslPauseSound(m_music,0);
		}
	}
	
	if(osl_pad.pressed.cross)
	{
		m_scenario->restart();
		oslStopSound(m_music);
		oslPlaySound(m_music, 0);
	}
	
	return MENU_NOACTION;
}













































